@@ -20,6 +20,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "quakedef.h"
 
+extern cvar_t cl_capturevideo;
 int old_vsync = 0;
 
 void CL_FinishTimeDemo (void);
@@ -87,7 +88,7 @@ void CL_StopPlayback (void)
 	if (cls.timedemo)
 		CL_FinishTimeDemo ();
 
-	if (COM_CheckParm("-demo") || COM_CheckParm("-demolooponly"))
+	if (COM_CheckParm("-demo") || COM_CheckParm("-capturedemo"))
 		Host_Quit_f();
 
 }
@@ -99,7 +100,7 @@ CL_WriteDemoMessage
 Dumps the current net message, prefixed by the length and view angles
 ====================
 */
-void CL_WriteDemoMessage (void)
+void CL_WriteDemoMessage (sizebuf_t *message)
 {
 	int		len;
 	int		i;
@@ -108,14 +109,14 @@ void CL_WriteDemoMessage (void)
 	if (cls.demopaused) // LordHavoc: pausedemo
 		return;
 
-	len = LittleLong (net_message.cursize);
+	len = LittleLong (message->cursize);
 	FS_Write (cls.demofile, &len, 4);
 	for (i=0 ; i<3 ; i++)
 	{
 		f = LittleFloat (cl.viewangles[i]);
 		FS_Write (cls.demofile, &f, 4);
 	}
-	FS_Write (cls.demofile, net_message.data, net_message.cursize);
+	FS_Write (cls.demofile, message->data, message->cursize);
 }
 
 /*
@@ -152,23 +153,41 @@ void CL_ReadDemoMessage(void)
 				}
 				if (cls.td_lastframe == -1)
 				{
-					// we start counting on the second frame
-					// (after parsing connection stuff)
-					cls.td_startframe = host_framecount + 1;
+					// render a couple frames before we start counting
+					cls.td_startframe = host_framecount + 3;
 				}
 				cls.td_lastframe = host_framecount;
+				cls.td_onesecondframes++;
+				// don't read any new messages during the warm-up period
+				if (host_framecount < cls.td_startframe)
+					return;
 				// if this is the first official frame we can now grab the real
 				// td_starttime so the bogus time on the first frame doesn't
 				// count against the final report
 				if (host_framecount == cls.td_startframe)
-					cls.td_starttime = realtime;
-				if (host_framecount > cls.td_startframe + 2)
 				{
-					cls.td_minframetime = min(cls.td_minframetime, cl.realframetime);
-					cls.td_maxframetime = max(cls.td_maxframetime, cl.realframetime);
+					cls.td_starttime = realtime;
+					cls.td_onesecondnexttime = realtime + 1;
+					cls.td_onesecondframes = 0;
+					cls.td_onesecondminframes = 0;
+					cls.td_onesecondmaxframes = 0;
+					cls.td_onesecondavgframes = 0;
+					cls.td_onesecondavgcount = 0;
+				}
+				if (realtime >= cls.td_onesecondnexttime)
+				{
+					if (cls.td_onesecondavgcount == 0)
+					{
+						cls.td_onesecondminframes = cls.td_onesecondframes;
+						cls.td_onesecondmaxframes = cls.td_onesecondframes;
+					}
+					cls.td_onesecondminframes = min(cls.td_onesecondminframes, cls.td_onesecondframes);
+					cls.td_onesecondmaxframes = max(cls.td_onesecondmaxframes, cls.td_onesecondframes);
+					cls.td_onesecondavgframes += cls.td_onesecondframes;
+					cls.td_onesecondavgcount++;
+					cls.td_onesecondframes = 0;
+					cls.td_onesecondnexttime++;
 				}
-				else
-					cls.td_minframetime = cls.td_maxframetime = cl.realframetime;
 			}
 			else if (cl.time <= cl.mtime[0])
 			{
@@ -216,6 +235,9 @@ stop recording a demo
 */
 void CL_Stop_f (void)
 {
+	sizebuf_t buf;
+	unsigned char bufdata[64];
+
 	if (!cls.demorecording)
 	{
 		Con_Print("Not recording a demo.\n");
@@ -223,9 +245,12 @@ void CL_Stop_f (void)
 	}
 
 // write a disconnect message to the demo file
-	SZ_Clear (&net_message);
-	MSG_WriteByte (&net_message, svc_disconnect);
-	CL_WriteDemoMessage ();
+	// LordHavoc: don't replace the net_message when doing this
+	buf.data = bufdata;
+	buf.maxsize = sizeof(bufdata);
+	SZ_Clear(&buf);
+	MSG_WriteByte(&buf, svc_disconnect);
+	CL_WriteDemoMessage(&buf);
 
 // finish up
 	FS_Close (cls.demofile);
@@ -265,6 +290,9 @@ void CL_Record_f (void)
 		return;
 	}
 
+	if (cls.state == ca_connected)
+		CL_Disconnect();
+
 	// write the forced cd track number, or -1
 	if (c == 4)
 	{
@@ -362,7 +390,7 @@ CL_FinishTimeDemo
 void CL_FinishTimeDemo (void)
 {
 	int frames;
-	double time; // LordHavoc: changed timedemo accuracy to double
+	double time, totalfpsavg;
 	double fpsmin, fpsavg, fpsmax; // report min/avg/max fps
 
 	cls.timedemo = false;
@@ -370,12 +398,13 @@ void CL_FinishTimeDemo (void)
 // the first frame didn't count
 	frames = (host_framecount - cls.td_startframe) - 1;
 	time = realtime - cls.td_starttime;
-	fpsmin = cls.td_maxframetime > 0 ? 1.0 / cls.td_maxframetime : 0;
-	fpsavg = time > 0 ? frames / time : 0;
-	fpsmax = cls.td_minframetime > 0 ? 1.0 / cls.td_minframetime : 0;
+	totalfpsavg = time > 0 ? frames / time : 0;
+	fpsmin = cls.td_onesecondminframes;
+	fpsavg = cls.td_onesecondavgcount ? cls.td_onesecondavgframes / cls.td_onesecondavgcount : 0;
+	fpsmax = cls.td_onesecondmaxframes;
 	// LordHavoc: timedemo now prints out 7 digits of fraction, and min/avg/max
-	Con_Printf("%i frames %5.7f seconds %5.7f fps\nmin/avg/max: %5.7f/%5.7f/%5.7f\n", frames, time, fpsavg, fpsmin, fpsavg, fpsmax);
-	Log_Printf("benchmark.log", "date %s | enginedate %s | demo %s | commandline %s | result %i frames %5.7f seconds %5.7f fps min/avg/max: %5.7f/%5.7f/%5.7f\n", Sys_TimeString("%Y-%m-%d %H:%M:%S"), buildstring, cls.demoname, cmdline.string, frames, time, fpsavg, fpsmin, fpsavg, fpsmax);
+	Con_Printf("%i frames %5.7f seconds %5.7f fps, one-second min/avg/max: %.0f %.0f %.0f\n", frames, time, totalfpsavg, fpsmin, fpsavg, fpsmax);
+	Log_Printf("benchmark.log", "date %s | enginedate %s | demo %s | commandline %s | result %i frames %5.7f seconds %5.7f fps, one-second min/avg/max: %.0f %.0f %.0f\n", Sys_TimeString("%Y-%m-%d %H:%M:%S"), buildstring, cls.demoname, cmdline.string, frames, time, totalfpsavg, fpsmin, fpsavg, fpsmax);
 	if (COM_CheckParm("-benchmark"))
 		Host_Quit_f();
 }
@@ -395,6 +424,8 @@ void CL_TimeDemo_f (void)
 		return;
 	}
 
+	srand(0); // predictable random sequence for benchmarking
+
 	CL_PlayDemo_f ();
 
 // cls.td_starttime will be grabbed at the second frame of the demo, so
