@@ -27,7 +27,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "quakedef.h"
 
-cvar_t registered = {0, "registered","0", "indicates if this is running registered quake (whether gfx/qpop.lmp was found)"};
+cvar_t registered = {0, "registered","0", "indicates if this is running registered quake (whether gfx/pop.lmp was found)"};
 cvar_t cmdline = {0, "cmdline","0", "contains commandline the engine was launched with"};
 
 char com_token[MAX_INPUTLINE];
@@ -671,13 +671,17 @@ once for each character in the string, so it may be stateful and e.g. implement
 kerning. At the beginning of the string, it will be called for the char 0 to
 initialize a clean state; the return value in this case however gets ignored.
 
+Wrapped lines get the isContinuation flag set and are continuationWidth less wide.
+
 The sum of the return values of the processLine function will be returned.
 ==============
 */
-int COM_Wordwrap(const char *string, size_t length, float maxWidth, COM_CharWidthFunc_t charWidth, void *passthroughCW, COM_LineProcessorFunc processLine, void *passthroughPL)
+int COM_Wordwrap(const char *string, size_t length, float continuationWidth, float maxWidth, COM_CharWidthFunc_t charWidth, void *passthroughCW, COM_LineProcessorFunc processLine, void *passthroughPL)
 {
+	qboolean isContinuation = false;
 	float currentWordSpace = 0;
 	const char *currentWord = 0;
+	float minReserve = 0;
 
 	float spaceUsedInLine = 0;
 	const char *currentLine = 0;
@@ -687,6 +691,12 @@ int COM_Wordwrap(const char *string, size_t length, float maxWidth, COM_CharWidt
 
 	int result = 0;
 	charWidth(passthroughCW, 0);
+	minReserve = charWidth(passthroughCW, 'm') + charWidth(passthroughCW, ' '); // FIXME assuming that is 'm' fits, anything does
+
+	if(maxWidth < continuationWidth + minReserve)
+		maxWidth = continuationWidth + minReserve;
+
+	charWidth(passthroughCW, 0);
 
 	for(p = string; p < string + length; ++p)
 	{
@@ -702,7 +712,7 @@ int COM_Wordwrap(const char *string, size_t length, float maxWidth, COM_CharWidt
 		if(!currentLine)
 		{
 			currentLine = p;
-			spaceUsedInLine = 0;
+			spaceUsedInLine = isContinuation ? continuationWidth : 0;
 			currentLineEnd = 0;
 		}
 
@@ -718,26 +728,32 @@ int COM_Wordwrap(const char *string, size_t length, float maxWidth, COM_CharWidt
 			// 2. I can just add the word - then append it, output current line and go to next one.
 			else if(spaceUsedInLine + currentWordSpace <= maxWidth)
 			{
-				result += processLine(passthroughPL, currentLine, p - currentLine, spaceUsedInLine + currentWordSpace);
+				result += processLine(passthroughPL, currentLine, p - currentLine, spaceUsedInLine + currentWordSpace, isContinuation);
 				currentLine = 0;
+				isContinuation = true;
 			}
 			// 3. Otherwise, output current line and go to next one, where I can add the word.
-			else if(currentWordSpace + w <= maxWidth)
+			else if(continuationWidth + currentWordSpace + w <= maxWidth)
 			{
 				if(currentLineEnd)
-					result += processLine(passthroughPL, currentLine, currentLineEnd - currentLine, spaceUsedInLine - currentLineFinalWhitespace);
+					result += processLine(passthroughPL, currentLine, currentLineEnd - currentLine, spaceUsedInLine - currentLineFinalWhitespace, isContinuation);
 				currentLine = currentWord;
-				spaceUsedInLine = currentWordSpace + w;
+				spaceUsedInLine = continuationWidth + currentWordSpace + w;
 				currentLineEnd = p;
 				currentLineFinalWhitespace = w;
+				isContinuation = true;
 			}
 			// 4. We can't even do that? Then output both current and next word as new lines.
 			else
 			{
 				if(currentLineEnd)
-					result += processLine(passthroughPL, currentLine, currentLineEnd - currentLine, spaceUsedInLine - currentLineFinalWhitespace);
-				result += processLine(passthroughPL, currentWord, p - currentWord, currentWordSpace);
+				{
+					result += processLine(passthroughPL, currentLine, currentLineEnd - currentLine, spaceUsedInLine - currentLineFinalWhitespace, isContinuation);
+					isContinuation = true;
+				}
+				result += processLine(passthroughPL, currentWord, p - currentWord, currentWordSpace, isContinuation);
 				currentLine = 0;
+				isContinuation = true;
 			}
 			currentWord = 0;
 		}
@@ -746,38 +762,84 @@ int COM_Wordwrap(const char *string, size_t length, float maxWidth, COM_CharWidt
 			// 1. I can add the word - then do it.
 			if(spaceUsedInLine + currentWordSpace <= maxWidth)
 			{
-				result += processLine(passthroughPL, currentLine, p - currentLine, spaceUsedInLine + currentWordSpace);
-				currentLine = 0;
+				result += processLine(passthroughPL, currentLine, p - currentLine, spaceUsedInLine + currentWordSpace, isContinuation);
 			}
 			// 2. Otherwise, output current line, next one and make tabula rasa.
 			else
 			{
 				if(currentLineEnd)
-					processLine(passthroughPL, currentLine, currentLineEnd - currentLine, spaceUsedInLine - currentLineFinalWhitespace);
-				result += processLine(passthroughPL, currentWord, p - currentWord, currentWordSpace);
-				currentLine = 0;
+				{
+					processLine(passthroughPL, currentLine, currentLineEnd - currentLine, spaceUsedInLine - currentLineFinalWhitespace, isContinuation);
+					isContinuation = true;
+				}
+				result += processLine(passthroughPL, currentWord, p - currentWord, currentWordSpace, isContinuation);
 			}
 			currentWord = 0;
+			currentLine = 0;
+			isContinuation = false;
 		}
 		else
 		{
 			currentWordSpace += w;
-			if(currentWordSpace > maxWidth)
+			if(
+				spaceUsedInLine + currentWordSpace > maxWidth // can't join this line...
+				&&
+				continuationWidth + currentWordSpace > maxWidth // can't join any other line...
+			)
 			{
 				// this word cannot join ANY line...
 				// so output the current line...
 				if(currentLineEnd)
-					result += processLine(passthroughPL, currentLine, currentLineEnd - currentLine, spaceUsedInLine - currentLineFinalWhitespace);
+				{
+					result += processLine(passthroughPL, currentLine, currentLineEnd - currentLine, spaceUsedInLine - currentLineFinalWhitespace, isContinuation);
+					isContinuation = true;
+				}
 
 				// then this word's beginning...
-				result += processLine(passthroughPL, currentWord, p - currentWord, currentWordSpace - w);
+				if(isContinuation)
+				{
+					// it may not fit, but we know we have to split it into maxWidth - continuationWidth pieces
+					float pieceWidth = maxWidth - continuationWidth;
+					const char *pos = currentWord;
+					currentWordSpace = 0;
 
-				// and use the rest of this word as new start of a line
-				currentWordSpace = w;
-				currentWord = p;
-				spaceUsedInLine = 0;
-				currentLine = p;
-				currentLineEnd = 0;
+					// reset the char width function to a state where no kerning occurs (start of word)
+					charWidth(passthroughCW, ' ');
+					while(pos <= p)
+					{
+						float w = charWidth(passthroughCW, *pos);
+						if(currentWordSpace + w > pieceWidth) // this piece won't fit any more
+						{
+							// print everything until it
+							result += processLine(passthroughPL, currentWord, pos - currentWord, currentWordSpace, true);
+							// go to here
+							currentWord = pos;
+							currentWordSpace = 0;
+						}
+						currentWordSpace += w;
+						++pos;
+					}
+					// now we have a currentWord that fits... set up its next line
+					// currentWordSpace has been set
+					// currentWord has been set
+					spaceUsedInLine = continuationWidth;
+					currentLine = currentWord;
+					currentLineEnd = 0;
+					isContinuation = true;
+				}
+				else
+				{
+					// we have a guarantee that it will fix (see if clause)
+					result += processLine(passthroughPL, currentWord, p - currentWord, currentWordSpace - w, isContinuation);
+
+					// and use the rest of this word as new start of a line
+					currentWordSpace = w;
+					currentWord = p;
+					spaceUsedInLine = continuationWidth;
+					currentLine = p;
+					currentLineEnd = 0;
+					isContinuation = true;
+				}
 			}
 		}
 	}
@@ -792,12 +854,15 @@ int COM_Wordwrap(const char *string, size_t length, float maxWidth, COM_CharWidt
 	{
 		// Can I append the current word?
 		if(spaceUsedInLine + currentWordSpace <= maxWidth)
-			result += processLine(passthroughPL, currentLine, p - currentLine, spaceUsedInLine + currentWordSpace);
+			result += processLine(passthroughPL, currentLine, p - currentLine, spaceUsedInLine + currentWordSpace, isContinuation);
 		else
 		{
 			if(currentLineEnd)
-				result += processLine(passthroughPL, currentLine, currentLineEnd - currentLine, spaceUsedInLine - currentLineFinalWhitespace);
-			result += processLine(passthroughPL, currentWord, p - currentWord, currentWordSpace);
+			{
+				result += processLine(passthroughPL, currentLine, currentLineEnd - currentLine, spaceUsedInLine - currentLineFinalWhitespace, isContinuation);
+				isContinuation = true;
+			}
+			result += processLine(passthroughPL, currentWord, p - currentWord, currentWordSpace, isContinuation);
 		}
 	}
 
@@ -806,14 +871,15 @@ int COM_Wordwrap(const char *string, size_t length, float maxWidth, COM_CharWidt
 
 /*
 ==============
-COM_ParseToken
+COM_ParseToken_Simple
 
 Parse a token out of a string
 ==============
 */
-int COM_ParseToken(const char **datapointer, int returnnewline)
+int COM_ParseToken_Simple(const char **datapointer, int returnnewline)
 {
 	int len;
+	int c;
 	const char *data = *datapointer;
 
 	len = 0;
@@ -858,7 +924,10 @@ skipwhite:
 		data++;
 		while (*data && (data[0] != '*' || data[1] != '/'))
 			data++;
-		data += 2;
+		if (*data)
+			data++;
+		if (*data)
+			data++;
 		goto skipwhite;
 	}
 	else if (*data == '\"')
@@ -866,34 +935,23 @@ skipwhite:
 		// quoted string
 		for (data++;*data != '\"';data++)
 		{
-			if (*data == '\\' && data[1] == '"' )
-				data++;
 			if (!*data || len >= (int)sizeof(com_token) - 1)
 			{
 				com_token[0] = 0;
 				*datapointer = NULL;
 				return false;
 			}
-			com_token[len++] = *data;
-		}
-		com_token[len] = 0;
-		*datapointer = data+1;
-		return true;
-	}
-	else if (*data == '\'')
-	{
-		// quoted string
-		for (data++;*data != '\'';data++)
-		{
-			if (*data == '\\' && data[1] == '\'' )
-				data++;
-			if (!*data || len >= (int)sizeof(com_token) - 1)
+			c = *data;
+			if (*data == '\\')
 			{
-				com_token[0] = 0;
-				*datapointer = NULL;
-				return false;
+				data++;
+				c = *data;
+				if (c == 'n')
+					c = '\n';
+				else if (c == 't')
+					c = '\t';
 			}
-			com_token[len++] = *data;
+			com_token[len++] = c;
 		}
 		com_token[len] = 0;
 		*datapointer = data+1;
@@ -902,12 +960,12 @@ skipwhite:
 	else if (*data == '\r')
 	{
 		// translate Mac line ending to UNIX
-		com_token[len++] = '\n';
+		com_token[len++] = '\n';data++;
 		com_token[len] = 0;
 		*datapointer = data;
 		return true;
 	}
-	else if (*data == '\n' || *data == '{' || *data == '}' || *data == ')' || *data == '(' || *data == ']' || *data == '[' || *data == '\'' || *data == ':' || *data == ',' || *data == ';')
+	else if (*data == '\n')
 	{
 		// single character
 		com_token[len++] = *data++;
@@ -918,7 +976,7 @@ skipwhite:
 	else
 	{
 		// regular word
-		for (;*data > ' ' && *data != '{' && *data != '}' && *data != ')' && *data != '(' && *data != ']' && *data != '[' && *data != '\'' && *data != ':' && *data != ',' && *data != ';' && *data != '\'' && *data != '"';data++)
+		for (;*data > ' ';data++)
 		{
 			if (len >= (int)sizeof(com_token) - 1)
 			{
@@ -936,12 +994,260 @@ skipwhite:
 
 /*
 ==============
-COM_ParseTokenConsole
+COM_ParseToken_QuakeC
+
+Parse a token out of a string
+==============
+*/
+int COM_ParseToken_QuakeC(const char **datapointer, int returnnewline)
+{
+	int len;
+	int c;
+	const char *data = *datapointer;
+
+	len = 0;
+	com_token[0] = 0;
+
+	if (!data)
+	{
+		*datapointer = NULL;
+		return false;
+	}
+
+// skip whitespace
+skipwhite:
+	// line endings:
+	// UNIX: \n
+	// Mac: \r
+	// Windows: \r\n
+	for (;*data <= ' ' && ((*data != '\n' && *data != '\r') || !returnnewline);data++)
+	{
+		if (*data == 0)
+		{
+			// end of file
+			*datapointer = NULL;
+			return false;
+		}
+	}
+
+	// handle Windows line ending
+	if (data[0] == '\r' && data[1] == '\n')
+		data++;
+
+	if (data[0] == '/' && data[1] == '/')
+	{
+		// comment
+		while (*data && *data != '\n' && *data != '\r')
+			data++;
+		goto skipwhite;
+	}
+	else if (data[0] == '/' && data[1] == '*')
+	{
+		// comment
+		data++;
+		while (*data && (data[0] != '*' || data[1] != '/'))
+			data++;
+		if (*data)
+			data++;
+		if (*data)
+			data++;
+		goto skipwhite;
+	}
+	else if (*data == '\"' || *data == '\'')
+	{
+		// quoted string
+		char quote = *data;
+		for (data++;*data != quote;data++)
+		{
+			if (!*data || len >= (int)sizeof(com_token) - 1)
+			{
+				com_token[0] = 0;
+				*datapointer = NULL;
+				return false;
+			}
+			c = *data;
+			if (*data == '\\')
+			{
+				data++;
+				c = *data;
+				if (c == 'n')
+					c = '\n';
+				else if (c == 't')
+					c = '\t';
+			}
+			com_token[len++] = c;
+		}
+		com_token[len] = 0;
+		*datapointer = data+1;
+		return true;
+	}
+	else if (*data == '\r')
+	{
+		// translate Mac line ending to UNIX
+		com_token[len++] = '\n';data++;
+		com_token[len] = 0;
+		*datapointer = data;
+		return true;
+	}
+	else if (*data == '\n' || *data == '{' || *data == '}' || *data == ')' || *data == '(' || *data == ']' || *data == '[' || *data == ':' || *data == ',' || *data == ';')
+	{
+		// single character
+		com_token[len++] = *data++;
+		com_token[len] = 0;
+		*datapointer = data;
+		return true;
+	}
+	else
+	{
+		// regular word
+		for (;*data > ' ' && *data != '{' && *data != '}' && *data != ')' && *data != '(' && *data != ']' && *data != '[' && *data != ':' && *data != ',' && *data != ';';data++)
+		{
+			if (len >= (int)sizeof(com_token) - 1)
+			{
+				com_token[0] = 0;
+				*datapointer = NULL;
+				return false;
+			}
+			com_token[len++] = *data;
+		}
+		com_token[len] = 0;
+		*datapointer = data;
+		return true;
+	}
+}
+
+/*
+==============
+COM_ParseToken_VM_Tokenize
+
+Parse a token out of a string
+==============
+*/
+int COM_ParseToken_VM_Tokenize(const char **datapointer, int returnnewline)
+{
+	int len;
+	int c;
+	const char *data = *datapointer;
+
+	len = 0;
+	com_token[0] = 0;
+
+	if (!data)
+	{
+		*datapointer = NULL;
+		return false;
+	}
+
+// skip whitespace
+skipwhite:
+	// line endings:
+	// UNIX: \n
+	// Mac: \r
+	// Windows: \r\n
+	for (;*data <= ' ' && ((*data != '\n' && *data != '\r') || !returnnewline);data++)
+	{
+		if (*data == 0)
+		{
+			// end of file
+			*datapointer = NULL;
+			return false;
+		}
+	}
+
+	// handle Windows line ending
+	if (data[0] == '\r' && data[1] == '\n')
+		data++;
+
+	if (data[0] == '/' && data[1] == '/')
+	{
+		// comment
+		while (*data && *data != '\n' && *data != '\r')
+			data++;
+		goto skipwhite;
+	}
+	else if (data[0] == '/' && data[1] == '*')
+	{
+		// comment
+		data++;
+		while (*data && (data[0] != '*' || data[1] != '/'))
+			data++;
+		if (*data)
+			data++;
+		if (*data)
+			data++;
+		goto skipwhite;
+	}
+	else if (*data == '\"' || *data == '\'')
+	{
+		char quote = *data;
+		// quoted string
+		for (data++;*data != quote;data++)
+		{
+			if (!*data || len >= (int)sizeof(com_token) - 1)
+			{
+				com_token[0] = 0;
+				*datapointer = NULL;
+				return false;
+			}
+			c = *data;
+			if (*data == '\\')
+			{
+				data++;
+				c = *data;
+				if (c == 'n')
+					c = '\n';
+				else if (c == 't')
+					c = '\t';
+			}
+			com_token[len++] = c;
+		}
+		com_token[len] = 0;
+		*datapointer = data+1;
+		return true;
+	}
+	else if (*data == '\r')
+	{
+		// translate Mac line ending to UNIX
+		com_token[len++] = '\n';data++;
+		com_token[len] = 0;
+		*datapointer = data;
+		return true;
+	}
+	else if (*data == '\n' || *data == '{' || *data == '}' || *data == ')' || *data == '(' || *data == ']' || *data == '[' || *data == ':' || *data == ',' || *data == ';')
+	{
+		// single character
+		com_token[len++] = *data++;
+		com_token[len] = 0;
+		*datapointer = data;
+		return true;
+	}
+	else
+	{
+		// regular word
+		for (;*data > ' ' && *data != ',' && *data != ';' && *data != '{' && *data != '}' && *data != ')' && *data != '(' && *data != ']' && *data != '[' && *data != ':' && *data != ',' && *data != ';';data++)
+		{
+			if (len >= (int)sizeof(com_token) - 1)
+			{
+				com_token[0] = 0;
+				*datapointer = NULL;
+				return false;
+			}
+			com_token[len++] = *data;
+		}
+		com_token[len] = 0;
+		*datapointer = data;
+		return true;
+	}
+}
+
+/*
+==============
+COM_ParseToken_Console
 
 Parse a token out of a string, behaving like the qwcl console
 ==============
 */
-int COM_ParseTokenConsole(const char **datapointer)
+int COM_ParseToken_Console(const char **datapointer)
 {
 	int len;
 	const char *data = *datapointer;
@@ -985,6 +1291,9 @@ skipwhite:
 				*datapointer = NULL;
 				return false;
 			}
+			// allow escaped " case
+			if (*data == '\\' && data[1] == '\"')
+				data++;
 			com_token[len++] = *data;
 		}
 		com_token[len] = 0;
@@ -1363,6 +1672,135 @@ int COM_ReadAndTokenizeLine(const char **text, char **argv, int maxargc, char *t
 	return argc;
 }
 
+/*
+============
+COM_StringLengthNoColors
+
+calculates the visible width of a color coded string.
+
+*valid is filled with TRUE if the string is a valid colored string (that is, if
+it does not end with an unfinished color code). If it gets filled with FALSE, a
+fix would be adding a STRING_COLOR_TAG at the end of the string.
+
+valid can be set to NULL if the caller doesn't care.
+
+For size_s, specify the maximum number of characters from s to use, or 0 to use
+all characters until the zero terminator.
+============
+*/
+size_t
+COM_StringLengthNoColors(const char *s, size_t size_s, qboolean *valid)
+{
+	const char *end = size_s ? (s + size_s) : NULL;
+	size_t len = 0;
+	for(;;)
+	{
+		switch((s == end) ? 0 : *s)
+		{
+			case 0:
+				if(valid)
+					*valid = TRUE;
+				return len;
+			case STRING_COLOR_TAG:
+				++s;
+				switch((s == end) ? 0 : *s)
+				{
+					case 0: // ends with unfinished color code!
+						++len;
+						if(valid)
+							*valid = FALSE;
+						return len;
+					case STRING_COLOR_TAG: // escaped ^
+						++len;
+						break;
+					case '0': case '1': case '2': case '3': case '4':
+					case '5': case '6': case '7': case '8': case '9': // color code
+						break;
+					default: // not a color code
+						++len; // STRING_COLOR_TAG
+						++len; // the character
+						break;
+				}
+				break;
+			default:
+				++len;
+				break;
+		}
+		++s;
+	}
+	// never get here
+}
+
+/*
+============
+COM_StringDecolorize
+
+removes color codes from a string.
+
+If escape_carets is true, the resulting string will be safe for printing. If
+escape_carets is false, the function will just strip color codes (for logging
+for example).
+
+If the output buffer size did not suffice for converting, the function returns
+FALSE. Generally, if escape_carets is false, the output buffer needs
+strlen(str)+1 bytes, and if escape_carets is true, it can need strlen(str)+2
+bytes. In any case, the function makes sure that the resulting string is
+zero terminated.
+
+For size_in, specify the maximum number of characters from in to use, or 0 to use
+all characters until the zero terminator.
+============
+*/
+qboolean
+COM_StringDecolorize(const char *in, size_t size_in, char *out, size_t size_out, qboolean escape_carets)
+{
+#define APPEND(ch) do { if(--size_out) { *out++ = (ch); } else { *out++ = 0; return FALSE; } } while(0)
+	const char *end = size_in ? (in + size_in) : NULL;
+	if(size_out < 1)
+		return FALSE;
+	for(;;)
+	{
+		switch((in == end) ? 0 : *in)
+		{
+			case 0:
+				*out++ = 0;
+				return TRUE;
+			case STRING_COLOR_TAG:
+				++in;
+				switch((in == end) ? 0 : *in)
+				{
+					case 0: // ends with unfinished color code!
+						APPEND(STRING_COLOR_TAG);
+						// finish the code by appending another caret when escaping
+						if(escape_carets)
+							APPEND(STRING_COLOR_TAG);
+						*out++ = 0;
+						return TRUE;
+					case STRING_COLOR_TAG: // escaped ^
+						APPEND(STRING_COLOR_TAG);
+						// append a ^ twice when escaping
+						if(escape_carets)
+							APPEND(STRING_COLOR_TAG);
+						break;
+					case '0': case '1': case '2': case '3': case '4':
+					case '5': case '6': case '7': case '8': case '9': // color code
+						break;
+					default: // not a color code
+						APPEND(STRING_COLOR_TAG);
+						APPEND(*in);
+						break;
+				}
+				break;
+			default:
+				APPEND(*in);
+				break;
+		}
+		++in;
+	}
+	// never get here
+#undef APPEND
+}
+
 // written by Elric, thanks Elric!
 char *SearchInfostring(const char *infostring, const char *key)
 {
