@@ -46,7 +46,7 @@ typedef struct
 
 	double addtime;
 	int mask;
-	
+
 	int height; // recalculated line height when needed (-1 to unset)
 }
 con_lineinfo;
@@ -62,19 +62,21 @@ int con_lines_count;
 
 cvar_t con_notifytime = {CVAR_SAVE, "con_notifytime","3", "how long notify lines last, in seconds"};
 cvar_t con_notify = {CVAR_SAVE, "con_notify","4", "how many notify lines to show"};
+cvar_t con_notifyalign = {CVAR_SAVE, "con_notifyalign", "", "how to align notify lines: 0 = left, 0.5 = center, 1 = right, empty string = game default)"};
+
 cvar_t con_chattime = {CVAR_SAVE, "con_chattime","30", "how long chat lines last, in seconds"};
-cvar_t con_chat = {CVAR_SAVE, "con_chat","5", "how many chat lines to show"};
+cvar_t con_chat = {CVAR_SAVE, "con_chat","0", "how many chat lines to show"};
+cvar_t con_chatpos = {CVAR_SAVE, "con_chatpos","0", "where to put chat (negative: lines from bottom of screen, positive: lines below notify, 0: at top)"};
 cvar_t con_chatwidth = {CVAR_SAVE, "con_chatwidth","1.0", "relative chat window width"};
-cvar_t con_textsize = {CVAR_SAVE, "con_textsize","8", "console text size in virtual 2D pixels"};	//[515]: console text size in pixels
+cvar_t con_textsize = {CVAR_SAVE, "con_textsize","8", "console text size in virtual 2D pixels"};
 
 
 cvar_t sys_specialcharactertranslation = {0, "sys_specialcharactertranslation", "1", "terminal console conchars to ASCII translation (set to 0 if your conchars.tga is for an 8bit character set or if you want raw output)"};
 #ifdef WIN32
-cvar_t sys_colortranslation = {0, "sys_colortranslation", "0",
+cvar_t sys_colortranslation = {0, "sys_colortranslation", "0", "terminal console color translation (supported values: 0 = strip color codes, 1 = translate to ANSI codes, 2 = no translation)"};
 #else
-cvar_t sys_colortranslation = {0, "sys_colortranslation", "1",
+cvar_t sys_colortranslation = {0, "sys_colortranslation", "1", "terminal console color translation (supported values: 0 = strip color codes, 1 = translate to ANSI codes, 2 = no translation)"};
 #endif
-                                                               "terminal console color translation (supported values: 0 = strip color codes, 1 = translate to ANSI codes, 2 = no translation)"};
 
 int con_linewidth;
 int con_vislines;
@@ -438,8 +440,10 @@ void Con_Init (void)
 	// register our cvars
 	Cvar_RegisterVariable (&con_notifytime);
 	Cvar_RegisterVariable (&con_notify);
+	Cvar_RegisterVariable (&con_notifyalign);
 	Cvar_RegisterVariable (&con_chattime);
 	Cvar_RegisterVariable (&con_chat);
+	Cvar_RegisterVariable (&con_chatpos);
 	Cvar_RegisterVariable (&con_chatwidth);
 	Cvar_RegisterVariable (&con_textsize);
 
@@ -448,7 +452,7 @@ void Con_Init (void)
 	Cmd_AddCommand ("messagemode", Con_MessageMode_f, "input a chat message to say to everyone");
 	Cmd_AddCommand ("messagemode2", Con_MessageMode2_f, "input a chat message to say to only your team");
 	Cmd_AddCommand ("clear", Con_Clear_f, "clear console history");
-	Cmd_AddCommand ("maps", Con_Maps_f, "list information about available maps");	// By [515]
+	Cmd_AddCommand ("maps", Con_Maps_f, "list information about available maps");
 	Cmd_AddCommand ("condump", Con_ConDump_f, "output console history to a file (see also log_file)");
 
 	con_initialized = true;
@@ -519,12 +523,35 @@ char *Con_BytesLeft(int len)
 		{
 			if(len <= firstline_start - lastline_onepastend)
 				return lastline_onepastend;
-			else	
+			else
 				return NULL;
 		}
 	}
 }
 
+/*
+================
+Con_FixTimes
+
+Notifies the console code about the current time
+(and shifts back times of other entries when the time
+went backwards)
+================
+*/
+void Con_FixTimes()
+{
+	int i;
+	if(con_lines_count >= 1)
+	{
+		double diff = cl.time - (con_lines + CON_LINES_LAST)->addtime;
+		if(diff < 0)
+		{
+			for(i = 0; i < con_lines_count; ++i)
+				CON_LINES(i).addtime += diff;
+		}
+	}
+}
+
 /*
 ================
 Con_AddLine
@@ -536,6 +563,9 @@ void Con_AddLine(const char *line, int len, int mask)
 {
 	char *putpos;
 	con_lineinfo *p;
+
+	Con_FixTimes();
+
 	if(len >= CON_TEXTSIZE)
 	{
 		// line too large?
@@ -685,7 +715,20 @@ void Con_Print(const char *msg)
 				// play talk wav
 				if (*msg == 1)
 				{
-					S_LocalSound ("sound/misc/talk.wav");
+					if(gamemode == GAME_NEXUIZ)
+					{
+						if(msg[1] == '\r' && cl.foundtalk2wav)
+							S_LocalSound ("sound/misc/talk2.wav");
+						else
+							S_LocalSound ("sound/misc/talk.wav");
+					}
+					else
+					{
+						if (msg[1] == '(' && cl.foundtalk2wav)
+							S_LocalSound ("sound/misc/talk2.wav");
+						else
+							S_LocalSound ("sound/misc/talk.wav");
+					}
 					mask = CON_MASK_CHAT;
 				}
 				line[index++] = STRING_COLOR_TAG;
@@ -979,7 +1022,7 @@ void Con_DrawInput (void)
 		text += 1 + key_linepos - con_linewidth;
 
 	// draw it
-	DrawQ_ColoredString(0, con_vislines - con_textsize.value*2, text, con_linewidth, con_textsize.value, con_textsize.value, 1.0, 1.0, 1.0, 1.0, 0, NULL );
+	DrawQ_String(0, con_vislines - con_textsize.value*2, text, con_linewidth, con_textsize.value, con_textsize.value, 1.0, 1.0, 1.0, 1.0, 0, NULL, false );
 
 	// remove cursor
 //	key_lines[edit_line][key_linepos] = 0;
@@ -992,8 +1035,8 @@ typedef struct
 	float x;
 	float y;
 	float width;
-
 	float ymin, ymax;
+	const char *continuationString;
 
 	// PRIVATE:
 	int colorindex; // init to -1
@@ -1026,16 +1069,17 @@ float Con_CharWidthFunc(void *passthrough, char c)
 		return w;
 }
 
-int Con_CountLineFunc(void *passthrough, const char *line, size_t length, float width)
+int Con_CountLineFunc(void *passthrough, const char *line, size_t length, float width, qboolean isContinuation)
 {
 	(void) passthrough;
 	(void) line;
 	(void) length;
 	(void) width;
+	(void) isContinuation;
 	return 1;
 }
 
-int Con_DisplayLineFunc(void *passthrough, const char *line, size_t length, float width)
+int Con_DisplayLineFunc(void *passthrough, const char *line, size_t length, float width, qboolean isContinuation)
 {
 	con_text_info_t *ti = (con_text_info_t *) passthrough;
 
@@ -1044,20 +1088,26 @@ int Con_DisplayLineFunc(void *passthrough, const char *line, size_t length, floa
 	else if(ti->y > ti->ymax - ti->fontsize + 0.001)
 		(void) 0;
 	else
-		DrawQ_ColoredString(ti->x + (ti->width - width) * ti->alignment, ti->y, line, length, ti->fontsize, ti->fontsize, 1.0, 1.0, 1.0, 1.0, 0, &(ti->colorindex));
+	{
+		int x = ti->x + (ti->width - width) * ti->alignment;
+		if(isContinuation && *ti->continuationString)
+			x += DrawQ_String(x, ti->y, ti->continuationString, strlen(ti->continuationString), ti->fontsize, ti->fontsize, 1.0, 1.0, 1.0, 1.0, 0, NULL, false);
+		DrawQ_String(x, ti->y, line, length, ti->fontsize, ti->fontsize, 1.0, 1.0, 1.0, 1.0, 0, &(ti->colorindex), false);
+	}
 
 	ti->y += ti->fontsize;
 	return 1;
 }
 
 
-int Con_DrawNotifyRect(int mask_must, int mask_mustnot, float maxage, float x, float y, float width, float height, float fontsize, float alignment_x, float alignment_y)
+int Con_DrawNotifyRect(int mask_must, int mask_mustnot, float maxage, float x, float y, float width, float height, float fontsize, float alignment_x, float alignment_y, const char *continuationString)
 {
 	int i;
 	int lines = 0;
 	int maxlines = (int) floor(height / fontsize + 0.01f);
 	int startidx;
 	int nskip = 0;
+	int continuationWidth = 0;
 	double t = cl.time; // saved so it won't change
 	con_text_info_t ti;
 
@@ -1066,6 +1116,11 @@ int Con_DrawNotifyRect(int mask_must, int mask_mustnot, float maxage, float x, f
 	ti.width = width;
 	ti.ymin = y;
 	ti.ymax = y + height;
+	ti.continuationString = continuationString;
+
+	Con_CharWidthFunc(&ti, 0);
+	while(*continuationString)
+		continuationWidth += Con_CharWidthFunc(&ti, *continuationString++);
 
 	// first find the first line to draw by backwards iterating and word wrapping to find their length...
 	startidx = con_lines_count;
@@ -1080,10 +1135,10 @@ int Con_DrawNotifyRect(int mask_must, int mask_mustnot, float maxage, float x, f
 			continue;
 		if(maxage && (l->addtime < t - maxage))
 			continue;
-
+		
 		// WE FOUND ONE!
 		// Calculate its actual height...
-		mylines = COM_Wordwrap(l->start, l->len, width, Con_CharWidthFunc, &ti, Con_CountLineFunc, &ti);
+		mylines = COM_Wordwrap(l->start, l->len, continuationWidth, width, Con_CharWidthFunc, &ti, Con_CountLineFunc, &ti);
 		if(lines + mylines >= maxlines)
 		{
 			nskip = lines + mylines - maxlines;
@@ -1108,10 +1163,10 @@ int Con_DrawNotifyRect(int mask_must, int mask_mustnot, float maxage, float x, f
 			continue;
 		if(l->mask & mask_mustnot)
 			continue;
-		if(maxage && (t - l->addtime) > maxage)
+		if(maxage && (l->addtime < t - maxage))
 			continue;
 
-		COM_Wordwrap(l->start, l->len, width, Con_CharWidthFunc, &ti, Con_DisplayLineFunc, &ti);
+		COM_Wordwrap(l->start, l->len, continuationWidth, width, Con_CharWidthFunc, &ti, Con_DisplayLineFunc, &ti);
 	}
 
 	return lines;
@@ -1127,21 +1182,32 @@ Draws the last few lines of output transparently over the game top
 void Con_DrawNotify (void)
 {
 	float	x, v;
-	static double lasttime = 0.0;
+	float chatstart, notifystart;
 	float align;
 	char	temptext[MAX_INPUTLINE];
 	int numChatlines;
+	int chatpos;
 
-	if(cl.time < lasttime)
+	Con_FixTimes();
+
+#if 0
 	{
-		// fix up timers
+		double t0 = -99999999999;
 		int i;
 		for(i = 0; i < con_lines_count; ++i)
-			CON_LINES(i).addtime += cl.time - lasttime;
+		{
+			double t = CON_LINES(i).addtime;
+			if(t < t0)
+				printf("CONSOLE TIME WENT BACKWARDS:\n%f %.*s\n%f %.*s\n",
+					CON_LINES(i-1).addtime, CON_LINES(i-1).len, CON_LINES(i-1).start, 
+					CON_LINES(i).addtime, CON_LINES(i).len, CON_LINES(i).start);
+			t0 = t;
+		}
 	}
-	lasttime = cl.time;
+#endif
 
 	numChatlines = con_chat.integer;
+	chatpos = con_chatpos.integer;
 
 	if (con_notify.integer < 0)
 		Cvar_SetValueQuick(&con_notify, 0);
@@ -1149,20 +1215,50 @@ void Con_DrawNotify (void)
 		v = 8; // vertical offset
 	else
 		v = 0;
-	// GAME_NEXUIZ: center, otherwise left justify
 
-	align = 0;
-	if(gamemode == GAME_NEXUIZ)
-		align = 0.5;
-	v += con_textsize.value * Con_DrawNotifyRect(0, CON_MASK_HIDENOTIFY | (numChatlines ? CON_MASK_CHAT : 0), con_notifytime.value, 0, v, vid_conwidth.value, con_notify.value * con_textsize.value, con_textsize.value, align, 0.0);
+	// GAME_NEXUIZ: center, otherwise left justify
+	align = con_notifyalign.value;
+	if(!*con_notifyalign.string) // empty string, evaluated to 0 above
+	{
+		if(gamemode == GAME_NEXUIZ)
+			align = 0.5;
+	}
+
+	if(numChatlines)
+	{
+		if(chatpos == 0)
+		{
+			// first chat, input line, then notify
+			chatstart = v;
+			notifystart = v + (numChatlines + 1) * con_textsize.value;
+		}
+		else if(chatpos > 0)
+		{
+			// first notify, then (chatpos-1) empty lines, then chat, then input
+			notifystart = v;
+			chatstart = v + (con_notify.value + (chatpos - 1)) * con_textsize.value;
+		}
+		else // if(chatpos < 0)
+		{
+			// first notify, then much space, then chat, then input, then -chatpos-1 empty lines
+			notifystart = v;
+			chatstart = vid_conheight.value - (-chatpos-1 + numChatlines + 1) * con_textsize.value;
+		}
+	}
+	else
+	{
+		// just notify and input
+		notifystart = v;
+		chatstart = 0; // shut off gcc warning
+	}
+
+	v = notifystart + con_textsize.value * Con_DrawNotifyRect(0, CON_MASK_HIDENOTIFY | (numChatlines ? CON_MASK_CHAT : 0), con_notifytime.value, 0, notifystart, vid_conwidth.value, con_notify.value * con_textsize.value, con_textsize.value, align, 0.0, "");
 
 	// chat?
 	if(numChatlines)
 	{
-		float y1 = vid_conheight.value - 56;
-		float y0 = y1 - numChatlines * con_textsize.value;
-		Con_DrawNotifyRect(CON_MASK_CHAT, 0, con_chattime.value, 0, y0, vid_conwidth.value * con_chatwidth.value, y1 - y0, con_textsize.value, 0.0, 1.0);
-		v = y1;
+		v = chatstart + numChatlines * con_textsize.value;
+		Con_DrawNotifyRect(CON_MASK_CHAT, 0, con_chattime.value, 0, chatstart, vid_conwidth.value * con_chatwidth.value, v - chatstart, con_textsize.value, 0.0, 1.0, "^3\014\014\014 "); // 015 is > character in conchars.tga
 	}
 
 	if (key_dest == key_message)
@@ -1178,13 +1274,13 @@ void Con_DrawNotify (void)
 			sprintf(temptext, "say:%s%c", chat_buffer, (int) 10+((int)(realtime*con_cursorspeed)&1));
 		while ((int)strlen(temptext) >= con_linewidth)
 		{
-			DrawQ_ColoredString( 0, v, temptext, con_linewidth, con_textsize.value, con_textsize.value, 1.0, 1.0, 1.0, 1.0, 0, &colorindex );
+			DrawQ_String( 0, v, temptext, con_linewidth, con_textsize.value, con_textsize.value, 1.0, 1.0, 1.0, 1.0, 0, &colorindex, false );
 			strlcpy(temptext, &temptext[con_linewidth], sizeof(temptext));
 			v += con_textsize.value;
 		}
 		if (strlen(temptext) > 0)
 		{
-			DrawQ_ColoredString( 0, v, temptext, 0, con_textsize.value, con_textsize.value, 1.0, 1.0, 1.0, 1.0, 0, &colorindex );
+			DrawQ_String( 0, v, temptext, 0, con_textsize.value, con_textsize.value, 1.0, 1.0, 1.0, 1.0, 0, &colorindex, false );
 			v += con_textsize.value;
 		}
 	}
@@ -1203,7 +1299,7 @@ int Con_MeasureConsoleLine(int lineno)
 	con_text_info_t ti;
 	ti.fontsize = con_textsize.value;
 
-	return COM_Wordwrap(con_lines[lineno].start, con_lines[lineno].len, width, Con_CharWidthFunc, &ti, Con_CountLineFunc, NULL);
+	return COM_Wordwrap(con_lines[lineno].start, con_lines[lineno].len, 0, width, Con_CharWidthFunc, &ti, Con_CountLineFunc, NULL);
 }
 
 /*
@@ -1235,6 +1331,7 @@ int Con_DrawConsoleLine(float y, int lineno, float ymin, float ymax)
 	float width = vid_conwidth.value;
 
 	con_text_info_t ti;
+	ti.continuationString = "";
 	ti.alignment = 0;
 	ti.fontsize = con_textsize.value;
 	ti.x = 0;
@@ -1243,7 +1340,7 @@ int Con_DrawConsoleLine(float y, int lineno, float ymin, float ymax)
 	ti.ymax = ymax;
 	ti.width = width;
 
-	return COM_Wordwrap(con_lines[lineno].start, con_lines[lineno].len, width, Con_CharWidthFunc, &ti, Con_DisplayLineFunc, &ti);
+	return COM_Wordwrap(con_lines[lineno].start, con_lines[lineno].len, 0, width, Con_CharWidthFunc, &ti, Con_DisplayLineFunc, &ti);
 }
 
 /*
@@ -1305,8 +1402,8 @@ void Con_DrawConsole (int lines)
 	con_vislines = lines;
 
 // draw the background
-	DrawQ_Pic(0, lines - vid_conheight.integer, scr_conbrightness.value >= 0.01f ? Draw_CachePic("gfx/conback", true) : NULL, vid_conwidth.integer, vid_conheight.integer, scr_conbrightness.value, scr_conbrightness.value, scr_conbrightness.value, scr_conalpha.value, 0);
-	DrawQ_String(vid_conwidth.integer - strlen(engineversion) * con_textsize.value - con_textsize.value, lines - con_textsize.value, engineversion, 0, con_textsize.value, con_textsize.value, 1, 0, 0, 1, 0);
+	DrawQ_Pic(0, lines - vid_conheight.integer, scr_conbrightness.value >= 0.01f ? Draw_CachePic("gfx/conback", true) : NULL, vid_conwidth.integer, vid_conheight.integer, scr_conbrightness.value, scr_conbrightness.value, scr_conbrightness.value, cls.signon == SIGNONS ? scr_conalpha.value : 1.0, 0); // always full alpha when not in game
+	DrawQ_String(vid_conwidth.integer - strlen(engineversion) * con_textsize.value - con_textsize.value, lines - con_textsize.value, engineversion, 0, con_textsize.value, con_textsize.value, 1, 0, 0, 1, 0, NULL, true);
 
 // draw the text
 	if(con_lines_count > 0)
@@ -1441,7 +1538,7 @@ qboolean GetMapList (const char *s, char *completedname, int completednamebuffer
 				for (;;)
 				{
 					int l;
-					if (!COM_ParseTokenConsole(&data))
+					if (!COM_ParseToken_Simple(&data, false))
 						break;
 					if (com_token[0] == '{')
 						continue;
@@ -1452,7 +1549,7 @@ qboolean GetMapList (const char *s, char *completedname, int completednamebuffer
 					for (l = 0;l < (int)sizeof(keyname) - 1 && com_token[k+l] && com_token[k+l] > ' ';l++)
 						keyname[l] = com_token[k+l];
 					keyname[l] = 0;
-					if (!COM_ParseTokenConsole(&data))
+					if (!COM_ParseToken_Simple(&data, false))
 						break;
 					if (developer.integer >= 100)
 						Con_Printf("key: %s %s\n", keyname, com_token);
@@ -1492,7 +1589,7 @@ qboolean GetMapList (const char *s, char *completedname, int completednamebuffer
 				goto endcomplete;
 	}
 endcomplete:
-	if(p > o)
+	if(p > o && completedname && completednamebufferlength > 0)
 	{
 		memset(completedname, 0, completednamebufferlength);
 		memcpy(completedname, (t->filenames[0]+5), min(p, completednamebufferlength - 1));
