@@ -44,6 +44,10 @@ typedef long long fs_offset_t;
 extern char fs_gamedir [MAX_OSPATH];
 extern char fs_basedir [MAX_OSPATH];
 
+// list of active game directories (empty if not running a mod)
+#define MAX_GAMEDIRS 16
+extern int fs_numgamedirs;
+extern char fs_gamedirs[MAX_GAMEDIRS][MAX_QPATH];
 
 
 // ------ Main functions ------ //
@@ -66,9 +70,14 @@ int FS_Seek (qfile_t* file, fs_offset_t offset, int whence);
 fs_offset_t FS_Tell (qfile_t* file);
 fs_offset_t FS_FileSize (qfile_t* file);
 void FS_Purge (qfile_t* file);
+const char *FS_FileWithoutPath (const char *in);
 const char *FS_FileExtension (const char *in);
 int FS_CheckNastyPath (const char *path, qboolean isgamedir);
-qboolean FS_ChangeGameDir(const char *string);
+qboolean FS_CheckGameDir(const char *gamedir);
+qboolean FS_ChangeGameDirs(int numgamedirs, char gamedirs[][MAX_QPATH], qboolean complain, qboolean failmissing);
+qboolean FS_IsRegisteredQuakePack(const char *name);
+int FS_CRCFile(const char *filename, size_t *filesizepointer);
+void FS_Rescan(void);
 
 typedef struct fssearch_s
 {
