@@ -52,42 +52,16 @@ client_t *host_client;
 
 jmp_buf host_abortframe;
 
-// random seed
-cvar_t sv_random_seed = {0, "sv_random_seed", "", "random seed; when set, on every map start this random seed is used to initialize the random number generator. Don't touch it unless for benchmarking or debugging"};
-
 // pretend frames take this amount of time (in seconds), 0 = realtime
-cvar_t host_framerate = {0, "host_framerate","0", "locks frame timing to this value in seconds, 0.05 is 20fps for example, note that this can easily run too fast, use host_maxfps if you want to limit your framerate instead, or sys_ticrate to limit server speed"};
+cvar_t host_framerate = {0, "host_framerate","0", "locks frame timing to this value in seconds, 0.05 is 20fps for example, note that this can easily run too fast, use cl_maxfps if you want to limit your framerate instead, or sys_ticrate to limit server speed"};
 // shows time used by certain subsystems
 cvar_t host_speeds = {0, "host_speeds","0", "reports how much time is used in server/graphics/sound"};
-// LordHavoc: framerate independent slowmo
-cvar_t slowmo = {0, "slowmo", "1.0", "controls game speed, 0.5 is half speed, 2 is double speed"};
 // LordHavoc: framerate upper cap
 cvar_t cl_maxfps = {CVAR_SAVE, "cl_maxfps", "1000", "maximum fps cap, if game is running faster than this it will wait before running another frame (useful to make cpu time available to other programs)"};
 
-// print broadcast messages in dedicated mode
-cvar_t sv_echobprint = {CVAR_SAVE, "sv_echobprint", "1", "prints gamecode bprint() calls to server console"};
-
-cvar_t sys_ticrate = {CVAR_SAVE, "sys_ticrate","0.05", "how long a server frame is in seconds, 0.05 is 20fps server rate, 0.1 is 10fps (can not be set higher than 0.1), 0 runs as many server frames as possible (makes games against bots a little smoother, overwhelms network players)"};
-cvar_t sv_fixedframeratesingleplayer = {0, "sv_fixedframeratesingleplayer", "0", "allows you to use server-style timing system in singleplayer (don't run faster than sys_ticrate)"};
-
-cvar_t fraglimit = {CVAR_NOTIFY, "fraglimit","0", "ends level if this many frags is reached by any player"};
-cvar_t timelimit = {CVAR_NOTIFY, "timelimit","0", "ends level at this time (in minutes)"};
-cvar_t teamplay = {CVAR_NOTIFY, "teamplay","0", "teamplay mode, values depend on mod but typically 0 = no teams, 1 = no team damage no self damage, 2 = team damage and self damage, some mods support 3 = no team damage but can damage self"};
-
-cvar_t samelevel = {CVAR_NOTIFY, "samelevel","0", "repeats same level if level ends (due to timelimit or someone hitting an exit)"};
-cvar_t noexit = {CVAR_NOTIFY, "noexit","0", "kills anyone attempting to use an exit"};
-
 cvar_t developer = {0, "developer","0", "prints additional debugging messages and information (recommended for modders and level designers)"};
 cvar_t developer_entityparsing = {0, "developer_entityparsing", "0", "prints detailed network entities information each time a packet is received"};
 
-cvar_t skill = {0, "skill","1", "difficulty level of game, affects monster layouts in levels, 0 = easy, 1 = normal, 2 = hard, 3 = nightmare (same layout as hard but monsters fire twice)"};
-cvar_t deathmatch = {0, "deathmatch","0", "deathmatch mode, values depend on mod but typically 0 = no deathmatch, 1 = normal deathmatch with respawning weapons, 2 = weapons stay (players can only pick up new weapons)"};
-cvar_t coop = {0, "coop","0", "coop mode, 0 = no coop, 1 = coop mode, multiple players playing through the singleplayer game (coop mode also shuts off deathmatch)"};
-
-cvar_t pausable = {0, "pausable","1", "allow players to pause or not"};
-
-cvar_t temp1 = {0, "temp1","0", "general cvar for mods to use, in stock id1 this selects which death animation to use on players (0 = random death, other values select specific death scenes)"};
-
 cvar_t timestamps = {CVAR_SAVE, "timestamps", "0", "prints timestamps on console messages"};
 cvar_t timeformat = {CVAR_SAVE, "timeformat", "[%Y-%m-%d %H:%M:%S] ", "time format to use on timestamped console messages"};
 
@@ -222,31 +196,12 @@ static void Host_InitLocal (void)
 	Cmd_AddCommand("saveconfig", Host_SaveConfig_f, "save settings to config.cfg immediately (also automatic when quitting)");
 	Cmd_AddCommand("loadconfig", Host_LoadConfig_f, "reset everything and reload configs");
 
-	Cvar_RegisterVariable (&sv_random_seed);
 	Cvar_RegisterVariable (&host_framerate);
 	Cvar_RegisterVariable (&host_speeds);
-	Cvar_RegisterVariable (&slowmo);
 	Cvar_RegisterVariable (&cl_maxfps);
 
-	Cvar_RegisterVariable (&sv_echobprint);
-
-	Cvar_RegisterVariable (&sys_ticrate);
-	Cvar_RegisterVariable (&sv_fixedframeratesingleplayer);
-
-	Cvar_RegisterVariable (&fraglimit);
-	Cvar_RegisterVariable (&timelimit);
-	Cvar_RegisterVariable (&teamplay);
-	Cvar_RegisterVariable (&samelevel);
-	Cvar_RegisterVariable (&noexit);
-	Cvar_RegisterVariable (&skill);
 	Cvar_RegisterVariable (&developer);
 	Cvar_RegisterVariable (&developer_entityparsing);
-	Cvar_RegisterVariable (&deathmatch);
-	Cvar_RegisterVariable (&coop);
-
-	Cvar_RegisterVariable (&pausable);
-
-	Cvar_RegisterVariable (&temp1);
 
 	Cvar_RegisterVariable (&timestamps);
 	Cvar_RegisterVariable (&timeformat);
@@ -267,7 +222,7 @@ void Host_SaveConfig_f(void)
 // dedicated servers initialize the host but don't parse and set the
 // config.cfg cvars
 	// LordHavoc: don't save a config if it crashed in startup
-	if (host_framecount >= 3 && cls.state != ca_dedicated)
+	if (host_framecount >= 3 && cls.state != ca_dedicated && !COM_CheckParm("-benchmark") && !COM_CheckParm("-capturedemo"))
 	{
 		f = FS_Open ("config.cfg", "wb", false, false);
 		if (!f)
@@ -350,7 +305,7 @@ void SV_BroadcastPrint(const char *msg)
 
 	for (i = 0, client = svs.clients;i < svs.maxclients;i++, client++)
 	{
-		if (client->spawned && client->netconnection)
+		if (client->active && client->netconnection)
 		{
 			MSG_WriteByte(&client->netconnection->message, svc_print);
 			MSG_WriteString(&client->netconnection->message, msg);
@@ -414,7 +369,7 @@ if (crash = true), don't bother sending signofs
 void SV_DropClient(qboolean crash)
 {
 	int i;
-	Con_Printf("Client \"%s\" dropped\n", host_client->name);
+	Con_Printf("Client \"%s^%i\" dropped\n", host_client->name, STRING_COLOR_DEFAULT);
 
 	// make sure edict is not corrupt (from a level change for example)
 	host_client->edict = PRVM_EDICT_NUM(host_client - svs.clients + 1);
@@ -431,9 +386,9 @@ void SV_DropClient(qboolean crash)
 			buf.data = bufdata;
 			buf.maxsize = sizeof(bufdata);
 			MSG_WriteByte(&buf, svc_disconnect);
-			NetConn_SendUnreliableMessage(host_client->netconnection, &buf, sv.protocol);
-			NetConn_SendUnreliableMessage(host_client->netconnection, &buf, sv.protocol);
-			NetConn_SendUnreliableMessage(host_client->netconnection, &buf, sv.protocol);
+			NetConn_SendUnreliableMessage(host_client->netconnection, &buf, sv.protocol, 10000);
+			NetConn_SendUnreliableMessage(host_client->netconnection, &buf, sv.protocol, 10000);
+			NetConn_SendUnreliableMessage(host_client->netconnection, &buf, sv.protocol, 10000);
 		}
 		// break the net connection
 		NetConn_Close(host_client->netconnection);
@@ -454,11 +409,18 @@ void SV_DropClient(qboolean crash)
 		prog->globals.server->self = saveSelf;
 	}
 
+	// if a download is active, close it
+	if (host_client->download_file)
+	{
+		Con_DPrintf("Download of %s aborted when %s dropped\n", host_client->download_name, host_client->name);
+		FS_Close(host_client->download_file);
+		host_client->download_file = NULL;
+		host_client->download_name[0] = 0;
+		host_client->download_expectedposition = 0;
+		host_client->download_started = false;
+	}
+
 	// remove leaving player from scoreboard
-	//host_client->edict->fields.server->netname = PRVM_SetEngineString(host_client->name);
-	//if ((val = PRVM_GETEDICTFIELDVALUE(host_client->edict, eval_clientcolors)))
-	//	val->_float = 0;
-	//host_client->edict->fields.server->frags = 0;
 	host_client->name[0] = 0;
 	host_client->colors = 0;
 	host_client->frags = 0;
@@ -566,37 +528,31 @@ Runs all active servers
 static void Host_Init(void);
 void Host_Main(void)
 {
-	static double time1 = 0;
-	static double time2 = 0;
-	static double time3 = 0;
-	// these are static because of setjmp/longjmp warnings in mingw32 gcc 2.95.3
-	static double frameoldtime, framenewtime, frametime, cl_timer, sv_timer;
+	double time1 = 0;
+	double time2 = 0;
+	double time3 = 0;
+	double cl_timer, sv_timer;
+	double clframetime, deltarealtime, oldrealtime;
+	double wait;
+	int pass1, pass2, pass3;
 
 	Host_Init();
 
 	cl_timer = 0;
 	sv_timer = 0;
 
-	framenewtime = Sys_DoubleTime();
+	realtime = Sys_DoubleTime();
 	for (;;)
 	{
 		if (setjmp(host_abortframe))
 			continue;			// something bad happened, or the server disconnected
 
-		frameoldtime = framenewtime;
-		framenewtime = Sys_DoubleTime();
-		frametime = framenewtime - frameoldtime;
-		realtime += frametime;
+		oldrealtime = realtime;
+		realtime = Sys_DoubleTime();
 
-		// if there is some time remaining from last frame, rest the timers
-		if (cl_timer >= 0)
-			cl_timer = 0;
-		if (sv_timer >= 0)
-			sv_timer = 0;
-
-		// accumulate the new frametime into the timers
-		cl_timer += frametime;
-		sv_timer += frametime;
+		deltarealtime = realtime - oldrealtime;
+		cl_timer += deltarealtime;
+		sv_timer += deltarealtime;
 
 		if (slowmo.value < 0)
 			Cvar_SetValue("slowmo", 0);
@@ -605,25 +561,8 @@ void Host_Main(void)
 		if (cl_maxfps.value < 1)
 			Cvar_SetValue("cl_maxfps", 1);
 
-		// if the accumulators haven't become positive yet, keep waiting
-		if (!cls.timedemo && cl_timer <= 0 && sv_timer <= 0)
-		{
-			double wait;
-			int msleft;
-			if (cls.state == ca_dedicated)
-				wait = sv_timer;
-			else if (!sv.active)
-				wait = cl_timer;
-			else
-				wait = max(cl_timer, sv_timer);
-			msleft = (int)floor(wait * -1000.0);
-			if (msleft >= 1)
-				Sys_Sleep(msleft);
-			continue;
-		}
-
-		// keep the random time dependent
-		if(!*sv_random_seed.string)
+		// keep the random time dependent, but not when playing demos/benchmarking
+		if(!*sv_random_seed.string && !cls.demoplayback)
 			rand();
 
 		cl.islocalgame = NetConn_IsLocalGame();
@@ -631,6 +570,18 @@ void Host_Main(void)
 		// get new key events
 		Sys_SendKeyEvents();
 
+		NetConn_UpdateSockets();
+
+		// receive packets on each main loop iteration, as the main loop may
+		// be undersleeping due to select() detecting a new packet
+		if (sv.active)
+			NetConn_ServerFrame();
+
+		Curl_Run();
+
+		// check for commands typed to the host
+		Host_GetConsoleCommands();
+
 		// when a server is running we only execute console commands on server frames
 		// (this mainly allows frikbot .way config files to work properly by staying in sync with the server qc)
 		// otherwise we execute them on all frames
@@ -640,9 +591,31 @@ void Host_Main(void)
 			Cbuf_Execute();
 		}
 
-		NetConn_UpdateSockets();
+		//Con_Printf("%6.0f %6.0f\n", cl_timer * 1000000.0, sv_timer * 1000000.0);
 
-		Curl_Run();
+		// if the accumulators haven't become positive yet, wait a while
+		if (cls.state == ca_dedicated)
+			wait = sv_timer * -1000000.0;
+		else if (!sv.active)
+			wait = cl_timer * -1000000.0;
+		else
+			wait = max(cl_timer, sv_timer) * -1000000.0;
+		if (wait > 100000)
+			wait = 100000;
+		if (!cls.timedemo && wait > 0)
+		{
+			if (sv_checkforpacketsduringsleep.integer)
+			{
+				if (wait >= 1)
+					NetConn_SleepMicroseconds((int)wait);
+			}
+			else
+			{
+				if (wait >= 1000)
+					Sys_Sleep((int)wait / 1000);
+			}
+			continue;
+		}
 
 	//-------------------
 	//
@@ -650,91 +623,78 @@ void Host_Main(void)
 	//
 	//-------------------
 
-		// check for commands typed to the host
-		Host_GetConsoleCommands();
+		// limit the frametime steps to no more than 100ms each
+		if (cl_timer > 0.1)
+			cl_timer = 0.1;
+		if (sv_timer > 0.1)
+			sv_timer = 0.1;
 
-		if (sv_timer > 0)
+		if (sv.active && sv_timer > 0)
 		{
-			if (!sv.active)
+			// execute one or more server frames, with an upper limit on how much
+			// execution time to spend on server frames to avoid freezing the game if
+			// the server is overloaded, this execution time limit means the game will
+			// slow down if the server is taking too long.
+			int framecount, framelimit = 1;
+			double advancetime, aborttime = 0;
+
+			// run the world state
+			// don't allow simulation to run too fast or too slow or logic glitches can occur
+
+			// stop running server frames if the wall time reaches this value
+			if (sys_ticrate.value <= 0)
+				advancetime = sv_timer;
+			else if (cl.islocalgame && !sv_fixedframeratesingleplayer.integer)
 			{
-				// if there is no server, run server timing at 10fps
-				sv_timer -= 0.1;
+				// synchronize to the client frametime, but no less than 10ms and no more than sys_ticrate
+				advancetime = bound(0.01, cl_timer, sys_ticrate.value);
+				framelimit = 10;
+				aborttime = realtime + 0.1;
 			}
 			else
 			{
-				// execute one or more server frames, with an upper limit on how much
-				// execution time to spend on server frames to avoid freezing the game if
-				// the server is overloaded, this execution time limit means the game will
-				// slow down if the server is taking too long.
-				int framecount, framelimit = 1;
-				double advancetime, aborttime = 0;
-
-				// receive server packets now, which might contain rcon commands, which
-				// may change level or other such things we don't want to have happen in
-				// the middle of Host_Frame
-				NetConn_ServerFrame();
-
-				// run the world state
-				// don't allow simulation to run too fast or too slow or logic glitches can occur
-
-				// stop running server frames if the wall time reaches this value
-				if (sys_ticrate.value <= 0)
-					advancetime = sv_timer;
-				else if (cl.islocalgame && !sv_fixedframeratesingleplayer.integer)
+				advancetime = sys_ticrate.value;
+				// listen servers can run multiple server frames per client frame
+				if (cls.state == ca_connected)
 				{
-					// synchronize to the client frametime, but no less than 10ms and no more than sys_ticrate
-					advancetime = bound(0.01, cl_timer, sys_ticrate.value);
 					framelimit = 10;
-					aborttime = Sys_DoubleTime() + 0.1;
+					aborttime = realtime + 0.1;
 				}
-				else
-				{
-					advancetime = sys_ticrate.value;
-					// listen servers can run multiple server frames per client frame
-					if (cls.state == ca_connected)
-					{
-						framelimit = 10;
-						aborttime = Sys_DoubleTime() + 0.1;
-					}
-				}
-				advancetime = min(advancetime, 0.1);
-
-				// only advance time if not paused
-				// the game also pauses in singleplayer when menu or console is used
-				sv.frametime = advancetime * slowmo.value;
-				if (host_framerate.value)
-					sv.frametime = host_framerate.value;
-				if (sv.paused || (cl.islocalgame && (key_dest != key_game || key_consoleactive)))
-					sv.frametime = 0;
-
-				// setup the VM frame
-				SV_VM_Begin();
-
-				for (framecount = 0;framecount < framelimit && sv_timer > 0;framecount++)
-				{
-					sv_timer -= advancetime;
-
-					// move things around and think unless paused
-					if (sv.frametime)
-						SV_Physics();
-
-					// send all messages to the clients
-					SV_SendClientMessages();
-
-					// clear the general datagram
-					SV_ClearDatagram();
-
-					// if this server frame took too long, break out of the loop
-					if (framelimit > 1 && Sys_DoubleTime() >= aborttime)
-						break;
-				}
-
-				// end the server VM frame
-				SV_VM_End();
-
-				// send an heartbeat if enough time has passed since the last one
-				NetConn_Heartbeat(0);
 			}
+			advancetime = min(advancetime, 0.1);
+
+			// only advance time if not paused
+			// the game also pauses in singleplayer when menu or console is used
+			sv.frametime = advancetime * slowmo.value;
+			if (host_framerate.value)
+				sv.frametime = host_framerate.value;
+			if (sv.paused || (cl.islocalgame && (key_dest != key_game || key_consoleactive)))
+				sv.frametime = 0;
+
+			// setup the VM frame
+			SV_VM_Begin();
+
+			for (framecount = 0;framecount < framelimit && sv_timer > 0;framecount++)
+			{
+				sv_timer -= advancetime;
+
+				// move things around and think unless paused
+				if (sv.frametime)
+					SV_Physics();
+
+				// if this server frame took too long, break out of the loop
+				if (framelimit > 1 && Sys_DoubleTime() >= aborttime)
+					break;
+			}
+
+			// send all messages to the clients
+			SV_SendClientMessages();
+
+			// end the server VM frame
+			SV_VM_End();
+
+			// send an heartbeat if enough time has passed since the last one
+			NetConn_Heartbeat(0);
 		}
 
 	//-------------------
@@ -743,94 +703,89 @@ void Host_Main(void)
 	//
 	//-------------------
 
-		if (cl_timer > 0 || cls.timedemo)
+		if (cls.state != ca_dedicated && (cl_timer > 0 || cls.timedemo))
 		{
-			if (cls.state == ca_dedicated)
+			// decide the simulation time
+			if (cls.capturevideo.active)
 			{
-				// if there is no client, run client timing at 10fps
-				cl_timer -= 0.1;
-				if (host_speeds.integer)
-					time1 = time2 = Sys_DoubleTime();
+				if (cls.capturevideo.realtime)
+					clframetime = cl.realframetime = max(cl_timer, 1.0 / cls.capturevideo.framerate);
+				else
+				{
+					clframetime = 1.0 / cls.capturevideo.framerate;
+					cl.realframetime = max(cl_timer, clframetime);
+				}
+			}
+			else if (vid_activewindow)
+				clframetime = cl.realframetime = max(cl_timer, 1.0 / cl_maxfps.value);
+			else
+				clframetime = cl.realframetime = 0.1;
+
+			// apply slowmo scaling
+			clframetime *= cl.movevars_timescale;
+			// scale playback speed of demos by slowmo cvar
+			if (cls.demoplayback)
+			{
+				clframetime *= slowmo.value;
+				// if demo playback is paused, don't advance time at all
+				if (cls.demopaused)
+					clframetime = 0;
+			}
+
+			// host_framerate overrides all else
+			if (host_framerate.value)
+				clframetime = host_framerate.value;
+
+			if (cls.timedemo)
+				clframetime = cl.realframetime = cl_timer;
+
+			// deduct the frame time from the accumulator
+			cl_timer -= cl.realframetime;
+
+			cl.oldtime = cl.time;
+			cl.time += clframetime;
+
+			// Collect input into cmd
+			CL_Input();
+
+			// check for new packets
+			NetConn_ClientFrame();
+
+			// read a new frame from a demo if needed
+			CL_ReadDemoMessage();
+
+			// now that packets have been read, send input to server
+			CL_SendMove();
+
+			// update client world (interpolate entities, create trails, etc)
+			CL_UpdateWorld();
+
+			// update video
+			if (host_speeds.integer)
+				time1 = Sys_DoubleTime();
+
+			//ui_update();
+
+			CL_VideoFrame();
+
+			CL_UpdateScreen();
+
+			if (host_speeds.integer)
+				time2 = Sys_DoubleTime();
+
+			// update audio
+			if(cl.csqc_usecsqclistener)
+			{
+				S_Update(&cl.csqc_listenermatrix);
+				cl.csqc_usecsqclistener = false;
 			}
 			else
-			{
-				double frametime;
-				frametime = cl.realframetime = min(cl_timer, 1);
+				S_Update(&r_view.matrix);
 
-				// decide the simulation time
-				if (!cls.timedemo)
-				{
-					if (cls.capturevideo.active)
-					{
-						if (cls.capturevideo.realtime)
-							frametime = cl.realframetime = max(cl.realframetime, 1.0 / cls.capturevideo.framerate);
-						else
-						{
-							frametime = 1.0 / cls.capturevideo.framerate;
-							cl.realframetime = max(cl.realframetime, frametime);
-						}
-					}
-					else if (vid_activewindow)
-						frametime = cl.realframetime = max(cl.realframetime, 1.0 / cl_maxfps.value);
-					else
-						frametime = cl.realframetime = 0.1;
-
-					// deduct the frame time from the accumulator
-					cl_timer -= cl.realframetime;
-
-					// apply slowmo scaling
-					frametime *= slowmo.value;
-
-					// host_framerate overrides all else
-					if (host_framerate.value)
-						frametime = host_framerate.value;
-				}
-
-				cl.oldtime = cl.time;
-				cl.time += frametime;
-
-				// Collect input into cmd
-				CL_Move();
-
-				NetConn_ClientFrame();
-
-				if (cls.state == ca_connected)
-				{
-					CL_ReadFromServer();
-					// if running the server remotely, send intentions now after
-					// the incoming messages have been read
-					//if (!cl.islocalgame)
-					//	CL_SendCmd();
-				}
-
-				// update video
-				if (host_speeds.integer)
-					time1 = Sys_DoubleTime();
-
-				//ui_update();
-
-				CL_VideoFrame();
-
-				CL_UpdateScreen();
-
-				if (host_speeds.integer)
-					time2 = Sys_DoubleTime();
-
-				// update audio
-				if(csqc_usecsqclistener)
-				{
-					S_Update(&csqc_listenermatrix);
-					csqc_usecsqclistener = false;
-				}
-				else
-					S_Update(&r_view.matrix);
-
-				CDAudio_Update();
-			}
+			CDAudio_Update();
 
 			if (host_speeds.integer)
 			{
-				int pass1, pass2, pass3;
 				pass1 = (int)((time1 - time3)*1000000);
 				time3 = Sys_DoubleTime();
 				pass2 = (int)((time2 - time1)*1000000);
@@ -840,6 +795,12 @@ void Host_Main(void)
 			}
 		}
 
+		// if there is some time remaining from this frame, reset the timers
+		if (cl_timer >= 0)
+			cl_timer = 0;
+		if (sv_timer >= 0)
+			sv_timer = 0;
+
 		host_framecount++;
 	}
 }
@@ -881,7 +842,10 @@ static void Host_Init (void)
 	const char* os;
 
 	// LordHavoc: quake never seeded the random number generator before... heh
-	srand(time(NULL));
+	if (COM_CheckParm("-benchmark"))
+		srand(0); // predictable random sequence for -benchmark
+	else
+		srand(time(NULL));
 
 	// FIXME: this is evil, but possibly temporary
 // COMMANDLINEOPTION: Console: -developer enables warnings and other notices (RECOMMENDED for mod developers)
@@ -954,7 +918,7 @@ static void Host_Init (void)
 	// initialize ixtable
 	Mathlib_Init();
 
-	// initialize filesystem (including fs_basedir, fs_gamedir, -path, -game, scr_screenshot_name)
+	// initialize filesystem (including fs_basedir, fs_gamedir, -game, scr_screenshot_name)
 	FS_Init();
 
 	NetConn_Init();
@@ -963,6 +927,7 @@ static void Host_Init (void)
 	//PR_Cmd_Init();
 	PRVM_Init();
 	Mod_Init();
+	World_Init();
 	SV_Init();
 	Host_InitCommands();
 	Host_InitLocal();
@@ -1040,13 +1005,12 @@ static void Host_Init (void)
 		Cbuf_Execute();
 	}
 
-	// check for special demolooponly mode
-// COMMANDLINEOPTION: Client: -demolooponly <demoname> runs a playdemo and quits
-	i = COM_CheckParm("-demolooponly");
+// COMMANDLINEOPTION: Client: -capturedemo <demoname> captures a playdemo and quits
+	i = COM_CheckParm("-capturedemo");
 	if (i && i + 1 < com_argc)
 	if (!sv.active && !cls.demoplayback && !cls.connect_trying)
 	{
-		Cbuf_AddText(va("playdemo %s\n", com_argv[i + 1]));
+		Cbuf_AddText(va("playdemo %s\ncl_capturevideo 1\n", com_argv[i + 1]));
 		Cbuf_Execute();
 	}
 
@@ -1089,6 +1053,11 @@ void Host_Shutdown(void)
 		Con_Print("recursive shutdown\n");
 		return;
 	}
+	if (setjmp(host_abortframe))
+	{
+		Con_Print("aborted the quitting frame?!?\n");
+		return;
+	}
 	isdown = true;
 
 	// be quiet while shutting down
