@@ -762,30 +762,16 @@ unsigned char *LoadWAL (const unsigned char *f, int filesize, int matchwidth, in
 }
 
 
-static void Image_StripImageExtension (const char *in, char *out, size_t size_out)
+void Image_StripImageExtension (const char *in, char *out, size_t size_out)
 {
-	const char *end, *temp;
+	const char *ext;
 
 	if (size_out == 0)
 		return;
 
-	end = in + strlen(in);
-	if ((end - in) >= 4)
-	{
-		temp = end - 4;
-		if (strcmp(temp, ".tga") == 0
-		 || strcmp(temp, ".pcx") == 0
-		 || strcmp(temp, ".lmp") == 0
-		 || strcmp(temp, ".png") == 0
-		 || strcmp(temp, ".jpg") == 0)
-			end = temp;
-		while (in < end && size_out > 1)
-		{
-			*out++ = *in++;
-			size_out--;
-		}
-		*out++ = 0;
-	}
+	ext = FS_FileExtension(in);
+	if (ext && (!strcmp(ext, "tga") || !strcmp(ext, "pcx") || !strcmp(ext, "lmp") || !strcmp(ext, "png") || !strcmp(ext, "jpg")))
+		FS_StripExtension(in, out, size_out);
 	else
 		strlcpy(out, in, size_out);
 }
@@ -879,7 +865,7 @@ unsigned char *loadimagepixels (const char *filename, qboolean complain, int mat
 		firstformat = imageformats_textures;
 	else if (!strcasecmp(name, "gfx"))
 		firstformat = imageformats_gfx;
-	else if (!strchr(name, '/'))
+	else if (!strchr(basename, '/'))
 		firstformat = imageformats_nopath;
 	else
 		firstformat = imageformats_other;
@@ -900,6 +886,11 @@ unsigned char *loadimagepixels (const char *filename, qboolean complain, int mat
 					Mem_CheckSentinelsGlobal();
 				return data;
 			}
+			else
+			{
+				if (developer.integer >= 1)
+					Con_DPrintf("Error loading image %s (file loaded but decode failed)\n", name);
+			}
 		}
 	}
 	if (complain)
@@ -1495,182 +1486,43 @@ void Image_MipReduce(const unsigned char *in, unsigned char *out, int *width, in
 
 void Image_HeightmapToNormalmap(const unsigned char *inpixels, unsigned char *outpixels, int width, int height, int clamp, float bumpscale)
 {
-	int x, y;
-	const unsigned char *p0, *p1, *p2;
+	int x, y, x1, x2, y1, y2;
+	const unsigned char *b, *row[3];
+	int p[5];
 	unsigned char *out;
 	float iwidth, iheight, ibumpscale, n[3];
 	iwidth = 1.0f / width;
 	iheight = 1.0f / height;
-	ibumpscale = (255.0f * 3.0f) / bumpscale;
+	ibumpscale = (255.0f * 6.0f) / bumpscale;
 	out = outpixels;
-	for (y = 0;y < height;y++)
+	for (y = 0, y1 = height-1;y < height;y1 = y, y++)
 	{
-		for (x = 0;x < width;x++)
+		y2 = y + 1;if (y2 >= height) y2 = 0;
+		row[0] = inpixels + (y1 * width) * 4;
+		row[1] = inpixels + (y  * width) * 4;
+		row[2] = inpixels + (y2 * width) * 4;
+		for (x = 0, x1 = width-1;x < width;x1 = x, x++)
 		{
-			p0 = inpixels + (y * width + x) * 4;
-			if (x == width - 1)
-			{
-				if (clamp)
-					p1 = inpixels + (y * width + x) * 4;
-				else
-					p1 = inpixels + (y * width) * 4;
-			}
-			else
-				p1 = inpixels + (y * width + x + 1) * 4;
-			if (y == height - 1)
-			{
-				if (clamp)
-					p2 = inpixels + (y * width + x) * 4;
-				else
-					p2 = inpixels + x * 4;
-			}
-			else
-				p2 = inpixels + ((y + 1) * width + x) * 4;
-			/*
-			dv[0][0] = iwidth;
-			dv[0][1] = 0;
-			dv[0][2] = ((p0[0] + p0[1] + p0[2]) * ibumpscale) - ((p1[0] + p1[1] + p1[2]) * ibumpscale);
-			dv[1][0] = 0;
-			dv[1][1] = iheight;
-			dv[1][2] = ((p2[0] + p2[1] + p2[2]) * ibumpscale) - ((p0[0] + p0[1] + p0[2]) * ibumpscale);
-			n[0] = dv[0][1]*dv[1][2]-dv[0][2]*dv[1][1];
-			n[1] = dv[0][2]*dv[1][0]-dv[0][0]*dv[1][2];
-			n[2] = dv[0][0]*dv[1][1]-dv[0][1]*dv[1][0];
-			*/
-			n[0] = ((p0[0] + p0[1] + p0[2]) - (p1[0] + p1[1] + p1[2]));
-			n[1] = ((p2[0] + p2[1] + p2[2]) - (p0[0] + p0[1] + p0[2]));
+			x2 = x + 1;if (x2 >= width) x2 = 0;
+			// left, right
+			b = row[1] + x1 * 4;p[0] = (b[0] + b[1] + b[2]);
+			b = row[1] + x2 * 4;p[1] = (b[0] + b[1] + b[2]);
+			// above, below
+			b = row[0] + x  * 4;p[2] = (b[0] + b[1] + b[2]);
+			b = row[2] + x  * 4;p[3] = (b[0] + b[1] + b[2]);
+			// center
+			b = row[1] + x  * 4;p[4] = (b[0] + b[1] + b[2]);
+			// calculate a normal from the slopes
+			n[0] = p[0] - p[1];
+			n[1] = p[3] - p[2];
 			n[2] = ibumpscale;
 			VectorNormalize(n);
-			/*
-			// this should work for the bottom right triangle if anyone wants
-			// code for that for some reason
-			n[0] = ((p3[0] + p3[1] + p3[2]) - (p1[0] + p1[1] + p1[2]));
-			n[1] = ((p2[0] + p2[1] + p2[2]) - (p3[0] + p3[1] + p3[2]));
-			n[2] = ibumpscale;
-			VectorNormalize(n);
-			*/
+			// turn it into a dot3 rgb vector texture
 			out[0] = (int)(128.0f + n[0] * 127.0f);
 			out[1] = (int)(128.0f + n[1] * 127.0f);
 			out[2] = (int)(128.0f + n[2] * 127.0f);
-			out[3] = (p0[0] + p0[1] + p0[2]) / 3;
+			out[3] = (p[4]) / 3;
 			out += 4;
 		}
 	}
 }
-
-int image_loadskin(imageskin_t *s, const char *shadername)
-{
-	int j;
-	unsigned char *bumppixels;
-	int bumppixels_width, bumppixels_height;
-	char name[MAX_QPATH];
-	Image_StripImageExtension(shadername, name, sizeof(name));
-	memset(s, 0, sizeof(*s));
-	s->basepixels = loadimagepixels(name, false, 0, 0);
-	if (s->basepixels == NULL)
-		return false;
-	s->basepixels_width = image_width;
-	s->basepixels_height = image_height;
-
-	bumppixels = NULL;bumppixels_width = 0;bumppixels_height = 0;
-	for (j = 3;j < s->basepixels_width * s->basepixels_height * 4;j += 4)
-		if (s->basepixels[j] < 255)
-			break;
-	if (j < s->basepixels_width * s->basepixels_height * 4)
-	{
-		// has transparent pixels
-		s->maskpixels = (unsigned char *)Mem_Alloc(loadmodel->mempool, s->basepixels_width * s->basepixels_height * 4);
-		s->maskpixels_width = s->basepixels_width;
-		s->maskpixels_height = s->basepixels_height;
-		memcpy(s->maskpixels, s->basepixels, s->maskpixels_width * s->maskpixels_height * 4);
-		for (j = 0;j < s->basepixels_width * s->basepixels_height * 4;j += 4)
-		{
-			s->maskpixels[j+0] = 255;
-			s->maskpixels[j+1] = 255;
-			s->maskpixels[j+2] = 255;
-		}
-	}
-
-	// _luma is supported for tenebrae compatibility
-	// (I think it's a very stupid name, but oh well)
-	if ((s->glowpixels = loadimagepixels(va("%s_glow", name), false, 0, 0)) != NULL
-	 || (s->glowpixels = loadimagepixels(va("%s_luma", name), false, 0, 0)) != NULL)
-	{
-		s->glowpixels_width = image_width;
-		s->glowpixels_height = image_height;
-	}
-	// _norm is the name used by tenebrae
-	// (I don't like the name much)
-	if ((s->nmappixels = loadimagepixels(va("%s_norm", name), false, 0, 0)) != NULL)
-	{
-		s->nmappixels_width = image_width;
-		s->nmappixels_height = image_height;
-	}
-	else if ((bumppixels = loadimagepixels(va("%s_bump", name), false, 0, 0)) != NULL)
-	{
-		bumppixels_width = image_width;
-		bumppixels_height = image_height;
-	}
-	if ((s->glosspixels = loadimagepixels(va("%s_gloss", name), false, 0, 0)) != NULL)
-	{
-		s->glosspixels_width = image_width;
-		s->glosspixels_height = image_height;
-	}
-	if ((s->pantspixels = loadimagepixels(va("%s_pants", name), false, 0, 0)) != NULL)
-	{
-		s->pantspixels_width = image_width;
-		s->pantspixels_height = image_height;
-	}
-	if ((s->shirtpixels = loadimagepixels(va("%s_shirt", name), false, 0, 0)) != NULL)
-	{
-		s->shirtpixels_width = image_width;
-		s->shirtpixels_height = image_height;
-	}
-
-	if (s->nmappixels == NULL)
-	{
-		if (bumppixels != NULL)
-		{
-			if (r_shadow_bumpscale_bumpmap.value > 0)
-			{
-				s->nmappixels = (unsigned char *)Mem_Alloc(loadmodel->mempool, bumppixels_width * bumppixels_height * 4);
-				s->nmappixels_width = bumppixels_width;
-				s->nmappixels_height = bumppixels_height;
-				Image_HeightmapToNormalmap(bumppixels, s->nmappixels, s->nmappixels_width, s->nmappixels_height, false, r_shadow_bumpscale_bumpmap.value);
-			}
-		}
-		else
-		{
-			if (r_shadow_bumpscale_basetexture.value > 0)
-			{
-				s->nmappixels = (unsigned char *)Mem_Alloc(loadmodel->mempool, s->basepixels_width * s->basepixels_height * 4);
-				s->nmappixels_width = s->basepixels_width;
-				s->nmappixels_height = s->basepixels_height;
-				Image_HeightmapToNormalmap(s->basepixels, s->nmappixels, s->nmappixels_width, s->nmappixels_height, false, r_shadow_bumpscale_basetexture.value);
-			}
-		}
-	}
-	if (bumppixels != NULL)
-		Mem_Free(bumppixels);
-	return true;
-}
-
-void image_freeskin(imageskin_t *s)
-{
-	if (s->basepixels)
-		Mem_Free(s->basepixels);
-	if (s->maskpixels)
-		Mem_Free(s->maskpixels);
-	if (s->nmappixels)
-		Mem_Free(s->nmappixels);
-	if (s->glowpixels)
-		Mem_Free(s->glowpixels);
-	if (s->glosspixels)
-		Mem_Free(s->glosspixels);
-	if (s->pantspixels)
-		Mem_Free(s->pantspixels);
-	if (s->shirtpixels)
-		Mem_Free(s->shirtpixels);
-	memset(s, 0, sizeof(*s));
-}
-
