@@ -53,6 +53,23 @@ typedef struct skinframe_s
 	rtexture_t *gloss; // glossmap (for dot3)
 	rtexture_t *glow; // glow only (fullbrights)
 	rtexture_t *fog; // alpha of the base texture (if not opaque)
+	// accounting data for hash searches:
+	// the compare variables are used to identify internal skins from certain
+	// model formats
+	// (so that two q1bsp maps with the same texture name for different
+	//  textures do not have any conflicts)
+	struct skinframe_s *next; // next on hash chain
+	char basename[MAX_QPATH]; // name of this
+	int textureflags; // texture flags to use
+	int comparewidth;
+	int compareheight;
+	int comparecrc;
+	// mark and sweep garbage collection, this value is updated to a new value
+	// on each level change for the used skinframes, if some are not used they
+	// are freed
+	int loadsequence;
+	// on 32bit systems this makes the struct 128 bytes long
+	int padding;
 }
 skinframe_t;
 
@@ -79,9 +96,13 @@ struct trivertex_s;
 // (the surfaces reference portions of these meshes)
 typedef struct surfmesh_s
 {
+	// triangle data in system memory
 	int num_triangles; // number of triangles in the mesh
 	int *data_element3i; // int[tris*3] triangles of the mesh, 3 indices into vertex arrays for each
 	int *data_neighbor3i; // int[tris*3] neighboring triangle on each edge (-1 if none)
+	// element buffer object (stores triangles in video memory)
+	int ebo;
+	// vertex data in system memory
 	int num_vertices; // number of vertices in the mesh
 	float *data_vertex3f; // float[verts*3] vertex locations
 	float *data_svector3f; // float[verts*3] direction of 'S' (right) texture axis for each vertex
@@ -91,6 +112,15 @@ typedef struct surfmesh_s
 	float *data_texcoordlightmap2f; // float[verts*2] texcoords for lightmap texture
 	float *data_lightmapcolor4f;
 	int *data_lightmapoffsets; // index into surface's lightmap samples for vertex lighting
+	// vertex buffer object (stores geometry in video memory)
+	int vbo;
+	size_t vbooffset_vertex3f;
+	size_t vbooffset_svector3f;
+	size_t vbooffset_tvector3f;
+	size_t vbooffset_normal3f;
+	size_t vbooffset_texcoordtexture2f;
+	size_t vbooffset_texcoordlightmap2f;
+	size_t vbooffset_lightmapcolor4f;
 	// morph blending, these are zero if model is skeletal or static
 	int num_morphframes;
 	struct md3vertex_s *data_morphmd3vertex;
@@ -138,10 +168,201 @@ typedef struct shadowmesh_s
 	// these are NULL after Mod_ShadowMesh_Finish is performed, only used
 	// while building meshes
 	shadowmeshvertexhash_t **vertexhashtable, *vertexhashentries;
+	// element buffer object (stores triangles in video memory)
+	// (created by Mod_ShadowMesh_Finish if possible)
+	int ebo;
+	// vertex buffer object (stores vertices in video memory)
+	// (created by Mod_ShadowMesh_Finish if possible)
+	int vbo;
+	size_t vbooffset_vertex3f;
+	size_t vbooffset_svector3f;
+	size_t vbooffset_tvector3f;
+	size_t vbooffset_normal3f;
+	size_t vbooffset_texcoord2f;
 }
 shadowmesh_t;
 
+// various flags from shaders, used for special effects not otherwise classified
+// TODO: support these features more directly
+#define Q3TEXTUREFLAG_TWOSIDED 1
+#define Q3TEXTUREFLAG_NOPICMIP 16
+
+#define Q3PATHLENGTH 64
 #define TEXTURE_MAXFRAMES 64
+#define Q3WAVEPARMS 4
+#define Q3DEFORM_MAXPARMS 3
+#define Q3SHADER_MAXLAYERS 8
+#define Q3RGBGEN_MAXPARMS 3
+#define Q3ALPHAGEN_MAXPARMS 1
+#define Q3TCGEN_MAXPARMS 6
+#define Q3TCMOD_MAXPARMS 6
+#define Q3MAXTCMODS 4
+#define Q3MAXDEFORMS 4
+
+typedef enum q3wavefunc_e
+{
+	Q3WAVEFUNC_NONE,
+	Q3WAVEFUNC_INVERSESAWTOOTH,
+	Q3WAVEFUNC_NOISE,
+	Q3WAVEFUNC_SAWTOOTH,
+	Q3WAVEFUNC_SIN,
+	Q3WAVEFUNC_SQUARE,
+	Q3WAVEFUNC_TRIANGLE,
+	Q3WAVEFUNC_COUNT
+}
+q3wavefunc_t;
+
+typedef enum q3deform_e
+{
+	Q3DEFORM_NONE,
+	Q3DEFORM_PROJECTIONSHADOW,
+	Q3DEFORM_AUTOSPRITE,
+	Q3DEFORM_AUTOSPRITE2,
+	Q3DEFORM_TEXT0,
+	Q3DEFORM_TEXT1,
+	Q3DEFORM_TEXT2,
+	Q3DEFORM_TEXT3,
+	Q3DEFORM_TEXT4,
+	Q3DEFORM_TEXT5,
+	Q3DEFORM_TEXT6,
+	Q3DEFORM_TEXT7,
+	Q3DEFORM_BULGE,
+	Q3DEFORM_WAVE,
+	Q3DEFORM_NORMAL,
+	Q3DEFORM_MOVE,
+	Q3DEFORM_COUNT
+}
+q3deform_t;
+
+typedef enum q3rgbgen_e
+{
+	Q3RGBGEN_IDENTITY,
+	Q3RGBGEN_CONST,
+	Q3RGBGEN_ENTITY,
+	Q3RGBGEN_EXACTVERTEX,
+	Q3RGBGEN_IDENTITYLIGHTING,
+	Q3RGBGEN_LIGHTINGDIFFUSE,
+	Q3RGBGEN_ONEMINUSENTITY,
+	Q3RGBGEN_ONEMINUSVERTEX,
+	Q3RGBGEN_VERTEX,
+	Q3RGBGEN_WAVE,
+	Q3RGBGEN_COUNT
+}
+q3rgbgen_t;
+
+typedef enum q3alphagen_e
+{
+	Q3ALPHAGEN_IDENTITY,
+	Q3ALPHAGEN_CONST,
+	Q3ALPHAGEN_ENTITY,
+	Q3ALPHAGEN_LIGHTINGSPECULAR,
+	Q3ALPHAGEN_ONEMINUSENTITY,
+	Q3ALPHAGEN_ONEMINUSVERTEX,
+	Q3ALPHAGEN_PORTAL,
+	Q3ALPHAGEN_VERTEX,
+	Q3ALPHAGEN_WAVE,
+	Q3ALPHAGEN_COUNT
+}
+q3alphagen_t;
+
+typedef enum q3tcgen_e
+{
+	Q3TCGEN_NONE,
+	Q3TCGEN_TEXTURE, // very common
+	Q3TCGEN_ENVIRONMENT, // common
+	Q3TCGEN_LIGHTMAP,
+	Q3TCGEN_VECTOR,
+	Q3TCGEN_COUNT
+}
+q3tcgen_t;
+
+typedef enum q3tcmod_e
+{
+	Q3TCMOD_NONE,
+	Q3TCMOD_ENTITYTRANSLATE,
+	Q3TCMOD_ROTATE,
+	Q3TCMOD_SCALE,
+	Q3TCMOD_SCROLL,
+	Q3TCMOD_STRETCH,
+	Q3TCMOD_TRANSFORM,
+	Q3TCMOD_TURBULENT,
+	Q3TCMOD_COUNT
+}
+q3tcmod_t;
+
+typedef struct q3shaderinfo_layer_rgbgen_s
+{
+	q3rgbgen_t rgbgen;
+	float parms[Q3RGBGEN_MAXPARMS];
+	q3wavefunc_t wavefunc;
+	float waveparms[Q3WAVEPARMS];
+}
+q3shaderinfo_layer_rgbgen_t;
+
+typedef struct q3shaderinfo_layer_alphagen_s
+{
+	q3alphagen_t alphagen;
+	float parms[Q3ALPHAGEN_MAXPARMS];
+	q3wavefunc_t wavefunc;
+	float waveparms[Q3WAVEPARMS];
+}
+q3shaderinfo_layer_alphagen_t;
+
+typedef struct q3shaderinfo_layer_tcgen_s
+{
+	q3tcgen_t tcgen;
+	float parms[Q3TCGEN_MAXPARMS];
+}
+q3shaderinfo_layer_tcgen_t;
+
+typedef struct q3shaderinfo_layer_tcmod_s
+{
+	q3tcmod_t tcmod;
+	float parms[Q3TCMOD_MAXPARMS];
+	q3wavefunc_t wavefunc;
+	float waveparms[Q3WAVEPARMS];
+}
+q3shaderinfo_layer_tcmod_t;
+
+typedef struct q3shaderinfo_layer_s
+{
+	int alphatest;
+	int clampmap;
+	float framerate;
+	int numframes;
+	char texturename[TEXTURE_MAXFRAMES][Q3PATHLENGTH];
+	int blendfunc[2];
+	q3shaderinfo_layer_rgbgen_t rgbgen;
+	q3shaderinfo_layer_alphagen_t alphagen;
+	q3shaderinfo_layer_tcgen_t tcgen;
+	q3shaderinfo_layer_tcmod_t tcmods[Q3MAXTCMODS];
+}
+q3shaderinfo_layer_t;
+
+typedef struct q3shaderinfo_deform_s
+{
+	q3deform_t deform;
+	float parms[Q3DEFORM_MAXPARMS];
+	q3wavefunc_t wavefunc;
+	float waveparms[Q3WAVEPARMS];
+}
+q3shaderinfo_deform_t;
+
+typedef struct q3shaderinfo_s
+{
+	char name[Q3PATHLENGTH];
+	int surfaceparms;
+	int textureflags;
+	int numlayers;
+	qboolean lighting;
+	qboolean vertexalpha;
+	qboolean textureblendalpha;
+	q3shaderinfo_layer_t *primarylayer, *backgroundlayer;
+	q3shaderinfo_layer_t layers[Q3SHADER_MAXLAYERS];
+	char skyboxname[Q3PATHLENGTH];
+	q3shaderinfo_deform_t deforms[Q3MAXDEFORMS];
+}
+q3shaderinfo_t;
 
 typedef enum texturelayertype_e
 {
@@ -190,11 +411,20 @@ typedef struct texture_s
 	// current material flags (updated each bmodel render)
 	int currentmaterialflags;
 
+	// current PolygonOffset values for rendering this material
+	float currentpolygonfactor;
+	float currentpolygonoffset;
+
 	// textures to use when rendering this material
 	skinframe_t *currentskinframe;
 	int numskinframes;
 	float skinframerate;
-	skinframe_t skinframes[TEXTURE_MAXFRAMES];
+	skinframe_t *skinframes[TEXTURE_MAXFRAMES];
+	// background layer (for terrain texture blending)
+	skinframe_t *backgroundcurrentskinframe;
+	int backgroundnumskinframes;
+	float backgroundskinframerate;
+	skinframe_t *backgroundskinframes[TEXTURE_MAXFRAMES];
 
 	// total frames in sequence and alternate sequence
 	int anim_total[2];
@@ -212,11 +442,22 @@ typedef struct texture_s
 	// current texture transform matrix (used for water scrolling)
 	matrix4x4_t currenttexmatrix;
 
+	// various q3 shader features
+	q3shaderinfo_layer_rgbgen_t rgbgen;
+	q3shaderinfo_layer_alphagen_t alphagen;
+	q3shaderinfo_layer_tcgen_t tcgen;
+	q3shaderinfo_layer_tcmod_t tcmods[Q3MAXTCMODS];
+	q3shaderinfo_deform_t deforms[Q3MAXDEFORMS];
+
 	qboolean colormapping;
 	rtexture_t *basetexture;
 	rtexture_t *glosstexture;
+	rtexture_t *backgroundbasetexture;
+	rtexture_t *backgroundglosstexture;
 	float specularscale;
 	float specularpower;
+	// color tint (colormod * currentalpha) used for rtlighting this material
+	float dlightcolor[3];
 
 	// from q3 shaders
 	int customblendfunc[2];
@@ -230,8 +471,6 @@ typedef struct texture_s
 	int supercontents;
 	int surfaceparms;
 	int textureflags;
-
-	//skinframe_t skin;
 }
 texture_t;
 
@@ -388,12 +627,16 @@ typedef struct model_brush_s
 	// these are actually only found on brushq1, but NULL is handled gracefully
 	void (*AmbientSoundLevelsForPoint)(struct model_s *model, const vec3_t p, unsigned char *out, int outsize);
 	void (*RoundUpToHullSize)(struct model_s *cmodel, const vec3_t inmins, const vec3_t inmaxs, vec3_t outmins, vec3_t outmaxs);
+	// trace a line of sight through this model (returns false if the line if sight is definitely blocked)
+	qboolean (*TraceLineOfSight)(struct model_s *model, const vec3_t start, const vec3_t end);
 
 	char skybox[MAX_QPATH];
 
 	rtexture_t *solidskytexture;
 	rtexture_t *alphaskytexture;
 
+	qboolean supportwateralpha;
+
 	// QuakeWorld
 	int qw_md4sum;
 	int qw_md4sum2;
@@ -468,8 +711,12 @@ typedef struct model_brushq3_s
 	q3deffect_t *data_effects;
 
 	// lightmap textures
-	int num_lightmaps;
+	int num_originallightmaps;
+	int num_mergedlightmaps;
+	int num_lightmapmergepower;
+	int num_lightmapmerge;
 	rtexture_t **data_lightmaps;
+	rtexture_t **data_deluxemaps;
 
 	// voxel light data with directional shading
 	int num_lightgrid;
@@ -515,10 +762,8 @@ typedef struct model_s
 	mempool_t		*mempool;
 	// all models use textures...
 	rtexturepool_t	*texturepool;
-	// flags from the model file
-	int				flags;
-	// engine calculated flags, ones that can not be set in the file
-	int				flags2;
+	// EF_* flags (translated from the model file's different flags layout)
+	int				effects;
 	// number of QC accessible frame(group)s in the model
 	int				numframes;
 	// number of QC accessible skin(group)s in the model
@@ -556,10 +801,10 @@ typedef struct model_s
 	aliasbone_t		*data_bones;
 	int				num_poses;
 	float			*data_poses;
-	float			*data_basebonepose;
 	float			*data_baseboneposeinverse;
 	// textures of this model
 	int				num_textures;
+	int				num_texturesperskin;
 	texture_t		*data_textures;
 	// surfaces of this model
 	int				num_surfaces;
@@ -572,14 +817,16 @@ typedef struct model_s
 	void(*DrawSky)(struct entity_render_s *ent);
 	// draw the model using lightmap/dlight shading
 	void(*Draw)(struct entity_render_s *ent);
+	// draw the model to the depth buffer (no color rendering at all)
+	void(*DrawDepth)(struct entity_render_s *ent);
 	// gathers info on which clusters and surfaces are lit by light, as well as calculating a bounding box
-	void(*GetLightInfo)(struct entity_render_s *ent, vec3_t relativelightorigin, float lightradius, vec3_t outmins, vec3_t outmaxs, int *outleaflist, unsigned char *outleafpvs, int *outnumleafspointer, int *outsurfacelist, unsigned char *outsurfacepvs, int *outnumsurfacespointer);
+	void(*GetLightInfo)(struct entity_render_s *ent, vec3_t relativelightorigin, float lightradius, vec3_t outmins, vec3_t outmaxs, int *outleaflist, unsigned char *outleafpvs, int *outnumleafspointer, int *outsurfacelist, unsigned char *outsurfacepvs, int *outnumsurfacespointer, unsigned char *outshadowtrispvs, unsigned char *outlighttrispvs);
 	// compile a shadow volume for the model based on light source
 	void(*CompileShadowVolume)(struct entity_render_s *ent, vec3_t relativelightorigin, vec3_t relativelightdirection, float lightradius, int numsurfaces, const int *surfacelist);
 	// draw a shadow volume for the model based on light source
 	void(*DrawShadowVolume)(struct entity_render_s *ent, vec3_t relativelightorigin, vec3_t relativelightdirection, float lightradius, int numsurfaces, const int *surfacelist, const vec3_t lightmins, const vec3_t lightmaxs);
 	// draw the lighting on a model (through stencil)
-	void(*DrawLight)(struct entity_render_s *ent, int numsurfaces, const int *surfacelist);
+	void(*DrawLight)(struct entity_render_s *ent, int numsurfaces, const int *surfacelist, const unsigned char *trispvs);
 	// trace a box against this model
 	void (*TraceBox)(struct model_s *model, int frame, struct trace_s *trace, const vec3_t start, const vec3_t boxmins, const vec3_t boxmaxs, const vec3_t end, int hitsupercontentsmask);
 	// fields belonging to some types of model
@@ -635,12 +882,13 @@ int Mod_ShadowMesh_AddVertex(shadowmesh_t *mesh, float *vertex14f);
 void Mod_ShadowMesh_AddTriangle(mempool_t *mempool, shadowmesh_t *mesh, rtexture_t *map_diffuse, rtexture_t *map_specular, rtexture_t *map_normal, float *vertex14f);
 void Mod_ShadowMesh_AddMesh(mempool_t *mempool, shadowmesh_t *mesh, rtexture_t *map_diffuse, rtexture_t *map_specular, rtexture_t *map_normal, const float *vertex3f, const float *svector3f, const float *tvector3f, const float *normal3f, const float *texcoord2f, int numtris, const int *element3i);
 shadowmesh_t *Mod_ShadowMesh_Begin(mempool_t *mempool, int maxverts, int maxtriangles, rtexture_t *map_diffuse, rtexture_t *map_specular, rtexture_t *map_normal, int light, int neighbors, int expandable);
-shadowmesh_t *Mod_ShadowMesh_Finish(mempool_t *mempool, shadowmesh_t *firstmesh, int light, int neighbors);
+shadowmesh_t *Mod_ShadowMesh_Finish(mempool_t *mempool, shadowmesh_t *firstmesh, qboolean light, qboolean neighbors, qboolean createvbo);
 void Mod_ShadowMesh_CalcBBox(shadowmesh_t *firstmesh, vec3_t mins, vec3_t maxs, vec3_t center, float *radius);
 void Mod_ShadowMesh_Free(shadowmesh_t *mesh);
 
-int Mod_LoadSkinFrame(skinframe_t *skinframe, const char *basename, int textureflags, int loadpantsandshirt, int loadglowtexture);
-int Mod_LoadSkinFrame_Internal(skinframe_t *skinframe, const char *basename, int textureflags, int loadpantsandshirt, int loadglowtexture, const unsigned char *skindata, int width, int height, int bitsperpixel, const unsigned int *palette, const unsigned int *alphapalette);
+void Mod_LoadQ3Shaders(void);
+q3shaderinfo_t *Mod_LookupQ3Shader(const char *name);
+qboolean Mod_LoadTextureFromQ3Shader(texture_t *texture, const char *name, qboolean q1bsp, qboolean q3bsp, qboolean md3);
 
 extern cvar_t r_mipskins;
 
@@ -665,6 +913,7 @@ int Mod_CountSkinFiles(skinfile_t *skinfile);
 
 void Mod_SnapVertices(int numcomponents, int numvertices, float *vertices, float snap);
 int Mod_RemoveDegenerateTriangles(int numtriangles, const int *inelement3i, int *outelement3i, const float *vertex3f);
+void Mod_VertexRangeFromElements(int numelements, const int *elements, int *firstvertexpointer, int *lastvertexpointer);
 
 // bsp models
 void Mod_BrushInit(void);
@@ -676,10 +925,11 @@ int Mod_Q1BSP_SuperContentsFromNativeContents(struct model_s *model, int nativec
 struct entity_render_s;
 void R_Q1BSP_DrawSky(struct entity_render_s *ent);
 void R_Q1BSP_Draw(struct entity_render_s *ent);
-void R_Q1BSP_GetLightInfo(struct entity_render_s *ent, vec3_t relativelightorigin, float lightradius, vec3_t outmins, vec3_t outmaxs, int *outleaflist, unsigned char *outleafpvs, int *outnumleafspointer, int *outsurfacelist, unsigned char *outsurfacepvs, int *outnumsurfacespointer);
+void R_Q1BSP_DrawDepth(struct entity_render_s *ent);
+void R_Q1BSP_GetLightInfo(struct entity_render_s *ent, vec3_t relativelightorigin, float lightradius, vec3_t outmins, vec3_t outmaxs, int *outleaflist, unsigned char *outleafpvs, int *outnumleafspointer, int *outsurfacelist, unsigned char *outsurfacepvs, int *outnumsurfacespointer, unsigned char *outshadowtrispvs, unsigned char *outlighttrispvs);
 void R_Q1BSP_CompileShadowVolume(struct entity_render_s *ent, vec3_t relativelightorigin, vec3_t relativelightdirection, float lightradius, int numsurfaces, const int *surfacelist);
 void R_Q1BSP_DrawShadowVolume(struct entity_render_s *ent, vec3_t relativelightorigin, vec3_t relativelightdirection, float lightradius, int numsurfaces, const int *surfacelist, const vec3_t lightmins, const vec3_t lightmaxs);
-void R_Q1BSP_DrawLight(struct entity_render_s *ent, int numsurfaces, const int *surfacelist);
+void R_Q1BSP_DrawLight(struct entity_render_s *ent, int numsurfaces, const int *surfacelist, const unsigned char *trispvs);
 
 // alias models
 struct frameblend_s;
@@ -704,5 +954,8 @@ void Mod_PSKMODEL_Load(model_t *mod, void *buffer, void *bufferend);
 void Mod_IDSP_Load(model_t *mod, void *buffer, void *bufferend);
 void Mod_IDS2_Load(model_t *mod, void *buffer, void *bufferend);
 
+// utility
+qboolean Mod_CanSeeBox_Trace(int numsamples, float t, model_t *model, vec3_t eye, vec3_t minsX, vec3_t maxsX);
+
 #endif	// MODEL_SHARED_H
 
