@@ -38,8 +38,7 @@ static cvar_t sv_masters [] =
 	{CVAR_SAVE, "sv_master4", "", "user-chosen master server 4"},
 	{0, "sv_masterextra1", "ghdigital.com", "default master server 1 (admin: LordHavoc)"}, // admin: LordHavoc
 	{0, "sv_masterextra2", "dpmaster.deathmask.net", "default master server 2 (admin: Willis)"}, // admin: Willis
-	{0, "sv_masterextra3", "12.166.196.192", "default master server 3 (admin: Venim)"}, // admin: Venim
-	{0, "sv_masterextra4", "excalibur.nvg.ntnu.no", "default master server 4 (admin: tChr)"}, // admin: tChr
+	{0, "sv_masterextra3", "excalibur.nvg.ntnu.no", "default master server 3 (admin: tChr)"}, // admin: tChr
 	{0, NULL, NULL, NULL}
 };
 
@@ -78,7 +77,8 @@ cvar_t hostname = {CVAR_SAVE, "hostname", "UNNAMED", "server message to show in
 cvar_t developer_networking = {0, "developer_networking", "0", "prints all received and sent packets (recommended only for debugging)"};
 
 cvar_t cl_netlocalping = {0, "cl_netlocalping","0", "lags local loopback connection by this much ping time (useful to play more fairly on your own server with people with higher pings)"};
-static cvar_t cl_netpacketloss = {0, "cl_netpacketloss","0", "drops this percentage of packets (incoming and outgoing), useful for testing network protocol robustness (effects failing to start, sounds failing to play, etc)"};
+static cvar_t cl_netpacketloss_send = {0, "cl_netpacketloss_send","0", "drops this percentage of outgoing packets, useful for testing network protocol robustness (jerky movement, prediction errors, etc)"};
+static cvar_t cl_netpacketloss_receive = {0, "cl_netpacketloss_receive","0", "drops this percentage of incoming packets, useful for testing network protocol robustness (jerky movement, effects failing to start, sounds failing to play, etc)"};
 static cvar_t net_slist_queriespersecond = {0, "net_slist_queriespersecond", "20", "how many server information requests to send per second"};
 static cvar_t net_slist_queriesperframe = {0, "net_slist_queriesperframe", "4", "maximum number of server information requests to send each rendered frame (guards against low framerates causing problems)"};
 static cvar_t net_slist_timeout = {0, "net_slist_timeout", "4", "how long to listen for a server information response before giving up"};
@@ -186,6 +186,12 @@ static qboolean _ServerList_Entry_Compare( serverlist_entry_t *A, serverlist_ent
 		case SLIF_NUMPLAYERS:
 			result = A->info.numplayers - B->info.numplayers;
 			break;
+		case SLIF_NUMBOTS:
+			result = A->info.numbots - B->info.numbots;
+			break;
+		case SLIF_NUMHUMANS:
+			result = A->info.numhumans - B->info.numhumans;
+			break;
 		case SLIF_PROTOCOL:
 			result = A->info.protocol - B->info.protocol;
 			break;
@@ -287,6 +293,10 @@ static qboolean _ServerList_Entry_Mask( serverlist_mask_t *mask, serverlist_info
 		return false;
 	if( !_ServerList_CompareInt( info->numplayers, mask->tests[SLIF_NUMPLAYERS], mask->info.numplayers ) )
 		return false;
+	if( !_ServerList_CompareInt( info->numbots, mask->tests[SLIF_NUMBOTS], mask->info.numbots ) )
+		return false;
+	if( !_ServerList_CompareInt( info->numhumans, mask->tests[SLIF_NUMHUMANS], mask->info.numhumans ) )
+		return false;
 	if( !_ServerList_CompareInt( info->protocol, mask->tests[SLIF_PROTOCOL], mask->info.protocol ))
 		return false;
 	if( *mask->info.cname
@@ -384,8 +394,15 @@ void ServerList_RebuildViewList(void)
 
 void ServerList_ResetMasks(void)
 {
+	int i;
+
 	memset( &serverlist_andmasks, 0, sizeof( serverlist_andmasks ) );
 	memset( &serverlist_ormasks, 0, sizeof( serverlist_ormasks ) );
+	// numbots needs to be compared to -1 to always succeed
+	for(i = 0; i < SERVERLIST_ANDMASKCOUNT; ++i)
+		serverlist_andmasks[i].info.numbots = -1;
+	for(i = 0; i < SERVERLIST_ORMASKCOUNT; ++i)
+		serverlist_ormasks[i].info.numbots = -1;
 }
 
 #if 0
@@ -406,8 +423,7 @@ static void _ServerList_Test(void)
 
 void ServerList_QueryList(qboolean querydp, qboolean queryqw)
 {
-	//masterquerytime = realtime;
-	masterquerytime = Sys_DoubleTime();
+	masterquerytime = realtime;
 	masterquerycount = 0;
 	masterreplycount = 0;
 	serverquerycount = 0;
@@ -429,9 +445,9 @@ int NetConn_Read(lhnetsocket_t *mysocket, void *data, int maxlength, lhnetaddres
 	int i;
 	if (length == 0)
 		return 0;
-	if (cl_netpacketloss.integer)
+	if (cl_netpacketloss_receive.integer)
 		for (i = 0;i < cl_numsockets;i++)
-			if (cl_sockets[i] == mysocket && (rand() % 100) < cl_netpacketloss.integer)
+			if (cl_sockets[i] == mysocket && (rand() % 100) < cl_netpacketloss_receive.integer)
 				return 0;
 	if (developer_networking.integer)
 	{
@@ -453,9 +469,9 @@ int NetConn_Write(lhnetsocket_t *mysocket, const void *data, int length, const l
 {
 	int ret;
 	int i;
-	if (cl_netpacketloss.integer)
+	if (cl_netpacketloss_send.integer)
 		for (i = 0;i < cl_numsockets;i++)
-			if (cl_sockets[i] == mysocket && (rand() % 100) < cl_netpacketloss.integer)
+			if (cl_sockets[i] == mysocket && (rand() % 100) < cl_netpacketloss_send.integer)
 				return length;
 	ret = LHNET_Write(mysocket, data, length, peeraddress);
 	if (developer_networking.integer)
@@ -475,8 +491,31 @@ int NetConn_WriteString(lhnetsocket_t *mysocket, const char *string, const lhnet
 	return NetConn_Write(mysocket, string, (int)strlen(string), peeraddress);
 }
 
-int NetConn_SendUnreliableMessage(netconn_t *conn, sizebuf_t *data, protocolversion_t protocol)
+qboolean NetConn_CanSend(netconn_t *conn)
 {
+	conn->outgoing_packetcounter = (conn->outgoing_packetcounter + 1) % NETGRAPH_PACKETS;
+	conn->outgoing_unreliablesize[conn->outgoing_packetcounter] = NETGRAPH_NOPACKET;
+	conn->outgoing_reliablesize[conn->outgoing_packetcounter] = NETGRAPH_NOPACKET;
+	conn->outgoing_acksize[conn->outgoing_packetcounter] = NETGRAPH_NOPACKET;
+	if (realtime > conn->cleartime)
+		return true;
+	else
+	{
+		conn->outgoing_unreliablesize[conn->outgoing_packetcounter] = NETGRAPH_CHOKEDPACKET;
+		return false;
+	}
+}
+
+int NetConn_SendUnreliableMessage(netconn_t *conn, sizebuf_t *data, protocolversion_t protocol, int rate)
+{
+	int totallen = 0;
+
+	// if this packet was supposedly choked, but we find ourselves sending one
+	// anyway, make sure the size counting starts at zero
+	// (this mostly happens on level changes and disconnects and such)
+	if (conn->outgoing_unreliablesize[conn->outgoing_packetcounter] == NETGRAPH_CHOKEDPACKET)
+		conn->outgoing_unreliablesize[conn->outgoing_packetcounter] = NETGRAPH_NOPACKET;
+
 	if (protocol == PROTOCOL_QUAKEWORLD)
 	{
 		int packetLen;
@@ -517,16 +556,22 @@ int NetConn_SendUnreliableMessage(netconn_t *conn, sizebuf_t *data, protocolvers
 			Con_Printf ("NetConn_SendUnreliableMessage: reliable message too big %u\n", data->cursize);
 			return -1;
 		}
+
+		conn->outgoing_unreliablesize[conn->outgoing_packetcounter] += packetLen;
+
 		// add the reliable message if there is one
 		if (sendreliable)
 		{
+			conn->outgoing_reliablesize[conn->outgoing_packetcounter] += conn->sendMessageLength;
 			memcpy(sendbuffer + packetLen, conn->sendMessage, conn->sendMessageLength);
 			packetLen += conn->sendMessageLength;
 			conn->qw.last_reliable_sequence = conn->qw.outgoing_sequence;
 		}
+
 		// add the unreliable message if possible
 		if (packetLen + data->cursize <= 1400)
 		{
+			conn->outgoing_unreliablesize[conn->outgoing_packetcounter] += data->cursize;
 			memcpy(sendbuffer + packetLen, data->data, data->cursize);
 			packetLen += data->cursize;
 		}
@@ -536,10 +581,7 @@ int NetConn_SendUnreliableMessage(netconn_t *conn, sizebuf_t *data, protocolvers
 		packetsSent++;
 		unreliableMessagesSent++;
 
-		// delay later packets to obey rate limit
-		conn->qw.cleartime = max(conn->qw.cleartime, realtime) + packetLen * conn->qw.rate;
-
-		return 0;
+		totallen += packetLen + 28;
 	}
 	else
 	{
@@ -569,11 +611,15 @@ int NetConn_SendUnreliableMessage(netconn_t *conn, sizebuf_t *data, protocolvers
 			header[1] = BigLong(conn->nq.sendSequence - 1);
 			memcpy(sendbuffer + NET_HEADERSIZE, conn->sendMessage, dataLen);
 
+			conn->outgoing_reliablesize[conn->outgoing_packetcounter] += packetLen;
+
 			if (NetConn_Write(conn->mysocket, (void *)&sendbuffer, packetLen, &conn->peeraddress) == (int)packetLen)
 			{
 				conn->lastSendTime = realtime;
 				packetsReSent++;
 			}
+
+			totallen += packetLen + 28;
 		}
 
 		// if we have a new reliable message to send, do so
@@ -616,11 +662,15 @@ int NetConn_SendUnreliableMessage(netconn_t *conn, sizebuf_t *data, protocolvers
 
 			conn->nq.sendSequence++;
 
+			conn->outgoing_reliablesize[conn->outgoing_packetcounter] += packetLen;
+
 			NetConn_Write(conn->mysocket, (void *)&sendbuffer, packetLen, &conn->peeraddress);
 
 			conn->lastSendTime = realtime;
 			packetsSent++;
 			reliableMessagesSent++;
+
+			totallen += packetLen + 28;
 		}
 
 		// if we have an unreliable message to send, do so
@@ -641,13 +691,25 @@ int NetConn_SendUnreliableMessage(netconn_t *conn, sizebuf_t *data, protocolvers
 
 			conn->nq.unreliableSendSequence++;
 
+			conn->outgoing_unreliablesize[conn->outgoing_packetcounter] += packetLen;
+
 			NetConn_Write(conn->mysocket, (void *)&sendbuffer, packetLen, &conn->peeraddress);
 
 			packetsSent++;
 			unreliableMessagesSent++;
+
+			totallen += packetLen + 28;
 		}
-		return 0;
 	}
+
+	// delay later packets to obey rate limit
+	if (conn->cleartime < realtime - 0.1)
+		conn->cleartime = realtime - 0.1;
+	conn->cleartime = conn->cleartime + (double)totallen / (double)rate;
+	if (conn->cleartime < realtime)
+		conn->cleartime = realtime;
+
+	return 0;
 }
 
 void NetConn_CloseClientPorts(void)
@@ -853,11 +915,10 @@ void NetConn_UpdateSockets(void)
 
 static int NetConn_ReceivedMessage(netconn_t *conn, unsigned char *data, int length, protocolversion_t protocol, double newtimeout)
 {
+	int originallength = length;
 	if (length < 8)
 		return 0;
 
-	// TODO: add netgraph stuff rather than just packetloss counting...
-
 	if (protocol == PROTOCOL_QUAKEWORLD)
 	{
 		int sequence, sequence_ack;
@@ -898,12 +959,16 @@ static int NetConn_ReceivedMessage(netconn_t *conn, unsigned char *data, int len
 			//Con_DPrintf("Dropped %u datagram(s)\n", count);
 			while (count--)
 			{
-				conn->packetlost[conn->packetlostcounter] = true;
-				conn->packetlostcounter = (conn->packetlostcounter + 1) % 100;
+				conn->incoming_packetcounter = (conn->incoming_packetcounter + 1) % NETGRAPH_PACKETS;
+				conn->incoming_unreliablesize[conn->incoming_packetcounter] = NETGRAPH_LOSTPACKET;
+				conn->incoming_reliablesize[conn->incoming_packetcounter] = NETGRAPH_NOPACKET;
+				conn->incoming_acksize[conn->incoming_packetcounter] = NETGRAPH_NOPACKET;
 			}
 		}
-		conn->packetlost[conn->packetlostcounter] = false;
-		conn->packetlostcounter = (conn->packetlostcounter + 1) % 100;
+		conn->incoming_packetcounter = (conn->incoming_packetcounter + 1) % NETGRAPH_PACKETS;
+		conn->incoming_unreliablesize[conn->incoming_packetcounter] = originallength;
+		conn->incoming_reliablesize[conn->incoming_packetcounter] = NETGRAPH_NOPACKET;
+		conn->incoming_acksize[conn->incoming_packetcounter] = NETGRAPH_NOPACKET;
 		if (reliable_ack == conn->qw.reliable_sequence)
 		{
 			// received, now we will be able to send another reliable message
@@ -953,12 +1018,16 @@ static int NetConn_ReceivedMessage(netconn_t *conn, unsigned char *data, int len
 						//Con_DPrintf("Dropped %u datagram(s)\n", count);
 						while (count--)
 						{
-							conn->packetlost[conn->packetlostcounter] = true;
-							conn->packetlostcounter = (conn->packetlostcounter + 1) % 100;
+							conn->incoming_packetcounter = (conn->incoming_packetcounter + 1) % NETGRAPH_PACKETS;
+							conn->incoming_unreliablesize[conn->incoming_packetcounter] = NETGRAPH_LOSTPACKET;
+							conn->incoming_reliablesize[conn->incoming_packetcounter] = NETGRAPH_NOPACKET;
+							conn->incoming_acksize[conn->incoming_packetcounter] = NETGRAPH_NOPACKET;
 						}
 					}
-					conn->packetlost[conn->packetlostcounter] = false;
-					conn->packetlostcounter = (conn->packetlostcounter + 1) % 100;
+					conn->incoming_packetcounter = (conn->incoming_packetcounter + 1) % NETGRAPH_PACKETS;
+					conn->incoming_unreliablesize[conn->incoming_packetcounter] = originallength;
+					conn->incoming_reliablesize[conn->incoming_packetcounter] = NETGRAPH_NOPACKET;
+					conn->incoming_acksize[conn->incoming_packetcounter] = NETGRAPH_NOPACKET;
 					conn->nq.unreliableReceiveSequence = sequence + 1;
 					conn->lastMessageTime = realtime;
 					conn->timeout = realtime + newtimeout;
@@ -977,6 +1046,7 @@ static int NetConn_ReceivedMessage(netconn_t *conn, unsigned char *data, int len
 			}
 			else if (flags & NETFLAG_ACK)
 			{
+				conn->incoming_acksize[conn->incoming_packetcounter] += originallength;
 				if (sequence == (conn->nq.sendSequence - 1))
 				{
 					if (sequence == conn->nq.ackSequence)
@@ -1035,6 +1105,8 @@ static int NetConn_ReceivedMessage(netconn_t *conn, unsigned char *data, int len
 			else if (flags & NETFLAG_DATA)
 			{
 				unsigned int temppacket[2];
+				conn->incoming_reliablesize[conn->incoming_packetcounter] += originallength;
+				conn->outgoing_acksize[conn->outgoing_packetcounter] += 8;
 				temppacket[0] = BigLong(8 | NETFLAG_ACK);
 				temppacket[1] = BigLong(sequence);
 				NetConn_Write(conn->mysocket, (unsigned char *)temppacket, 8, &conn->peeraddress);
@@ -1093,6 +1165,9 @@ void NetConn_ConnectionEstablished(lhnetsocket_t *mysocket, lhnetaddress_t *peer
 	cls.state = ca_connected;
 	cls.signon = 0;				// need all the signon messages before playing
 	cls.protocol = initialprotocol;
+	// reset move sequence numbering on this new connection
+	cls.movesequence = 1;
+	cls.servermovesequence = 0;
 	if (cls.protocol == PROTOCOL_QUAKEWORLD)
 		Cmd_ForwardStringToServer("new");
 	if (cls.protocol == PROTOCOL_QUAKE)
@@ -1105,7 +1180,7 @@ void NetConn_ConnectionEstablished(lhnetsocket_t *mysocket, lhnetaddress_t *peer
 		msg.data = buf;
 		msg.maxsize = sizeof(buf);
 		MSG_WriteChar(&msg, clc_nop);
-		NetConn_SendUnreliableMessage(cls.netcon, &msg, cls.protocol);
+		NetConn_SendUnreliableMessage(cls.netcon, &msg, cls.protocol, 10000);
 	}
 }
 
@@ -1116,61 +1191,6 @@ int NetConn_IsLocalGame(void)
 	return false;
 }
 
-static int NetConn_ClientParsePacket_ServerList_ProcessReply(const char *addressstring)
-{
-	int n;
-	int pingtime;
-	// search the cache for this server and update it
-	for (n = 0;n < serverlist_cachecount;n++)
-		if (!strcmp(addressstring, serverlist_cache[n].info.cname))
-			break;
-	if (n == serverlist_cachecount)
-	{
-		// LAN search doesnt require an answer from the master server so we wont
-		// know the ping nor will it be initialized already...
-
-		// find a slot
-		if (serverlist_cachecount == SERVERLIST_TOTALSIZE)
-			return -1;
-
-		memset(&serverlist_cache[serverlist_cachecount], 0, sizeof(serverlist_cache[serverlist_cachecount]));
-		// store the data the engine cares about (address and ping)
-		strlcpy(serverlist_cache[serverlist_cachecount].info.cname, addressstring, sizeof(serverlist_cache[serverlist_cachecount].info.cname));
-		serverlist_cache[serverlist_cachecount].info.ping = 100000;
-		//serverlist_cache[serverlist_cachecount].querytime = realtime;
-		serverlist_cache[serverlist_cachecount].querytime = Sys_DoubleTime();
-		// if not in the slist menu we should print the server to console
-		if (serverlist_consoleoutput)
-			Con_Printf("querying %s\n", addressstring);
-		++serverlist_cachecount;
-	}
-	// if this is the first reply from this server, count it as having replied
-	if (serverlist_cache[n].info.ping == 100000)
-		serverreplycount++;
-	pingtime = (int)((Sys_DoubleTime() - serverlist_cache[n].querytime) * 1000.0 + 0.5);
-	pingtime = bound(0, pingtime, 9999);
-	// update the ping
-	serverlist_cache[n].info.ping = min(serverlist_cache[n].info.ping, pingtime);
-	// other server info is updated by the caller
-	return n;
-}
-
-static void NetConn_ClientParsePacket_ServerList_UpdateCache(int n)
-{
-	serverlist_info_t *info = &serverlist_cache[n].info;
-	// update description strings for engine menu and console output
-	dpsnprintf(serverlist_cache[n].line1, sizeof(serverlist_cache[n].line1), "^%c%5d^7 ^%c%3u^7/%3u %-65.65s", info->ping >= 300 ? '1' : (info->ping >= 200 ? '3' : '7'), (int)info->ping, ((info->numplayers > 0 && info->numplayers < info->maxplayers) ? (info->numplayers >= 4 ? '7' : '3') : '1'), info->numplayers, info->maxplayers, info->name);
-	dpsnprintf(serverlist_cache[n].line2, sizeof(serverlist_cache[n].line2), "^4%-21.21s %-19.19s ^%c%-17.17s^4 %-20.20s", info->cname, info->game, (info->gameversion != gameversion.integer) ? '1' : '4', info->mod, info->map);
-	if (serverlist_cache[n].query == SQS_QUERIED)
-		ServerList_ViewList_Remove(&serverlist_cache[n]);
-	// if not in the slist menu we should print the server to console (if wanted)
-	else if( serverlist_consoleoutput )
-		Con_Printf("%s\n%s\n", serverlist_cache[n].line1, serverlist_cache[n].line2);
-	// and finally, update the view set
-	ServerList_ViewList_Insert( &serverlist_cache[n] );
-	serverlist_cache[n].query = SQS_QUERIED;
-}
-
 static int NetConn_IsBanned(lhnetaddress_t *peeraddress)
 {
 	const char *banlist = net_banlist.string;
@@ -1231,6 +1251,60 @@ static int NetConn_IsBanned(lhnetaddress_t *peeraddress)
 	return false;
 }
 
+static int NetConn_ClientParsePacket_ServerList_ProcessReply(const char *addressstring)
+{
+	int n;
+	int pingtime;
+	// search the cache for this server and update it
+	for (n = 0;n < serverlist_cachecount;n++)
+		if (!strcmp(addressstring, serverlist_cache[n].info.cname))
+			break;
+	if (n == serverlist_cachecount)
+	{
+		// LAN search doesnt require an answer from the master server so we wont
+		// know the ping nor will it be initialized already...
+
+		// find a slot
+		if (serverlist_cachecount == SERVERLIST_TOTALSIZE)
+			return -1;
+
+		memset(&serverlist_cache[serverlist_cachecount], 0, sizeof(serverlist_cache[serverlist_cachecount]));
+		// store the data the engine cares about (address and ping)
+		strlcpy(serverlist_cache[serverlist_cachecount].info.cname, addressstring, sizeof(serverlist_cache[serverlist_cachecount].info.cname));
+		serverlist_cache[serverlist_cachecount].info.ping = 100000;
+		serverlist_cache[serverlist_cachecount].querytime = realtime;
+		// if not in the slist menu we should print the server to console
+		if (serverlist_consoleoutput)
+			Con_Printf("querying %s\n", addressstring);
+		++serverlist_cachecount;
+	}
+	// if this is the first reply from this server, count it as having replied
+	if (serverlist_cache[n].info.ping == 100000)
+		serverreplycount++;
+	pingtime = (int)((realtime - serverlist_cache[n].querytime) * 1000.0 + 0.5);
+	pingtime = bound(0, pingtime, 9999);
+	// update the ping
+	serverlist_cache[n].info.ping = min(serverlist_cache[n].info.ping, pingtime);
+	// other server info is updated by the caller
+	return n;
+}
+
+static void NetConn_ClientParsePacket_ServerList_UpdateCache(int n)
+{
+	serverlist_info_t *info = &serverlist_cache[n].info;
+	// update description strings for engine menu and console output
+	dpsnprintf(serverlist_cache[n].line1, sizeof(serverlist_cache[n].line1), "^%c%5d^7 ^%c%3u^7/%3u %-65.65s", info->ping >= 300 ? '1' : (info->ping >= 200 ? '3' : '7'), (int)info->ping, ((info->numhumans > 0 && info->numhumans < info->maxplayers) ? (info->numhumans >= 4 ? '7' : '3') : '1'), info->numplayers, info->maxplayers, info->name);
+	dpsnprintf(serverlist_cache[n].line2, sizeof(serverlist_cache[n].line2), "^4%-21.21s %-19.19s ^%c%-17.17s^4 %-20.20s", info->cname, info->game, (info->gameversion != gameversion.integer) ? '1' : '4', info->mod, info->map);
+	if (serverlist_cache[n].query == SQS_QUERIED)
+		ServerList_ViewList_Remove(&serverlist_cache[n]);
+	// if not in the slist menu we should print the server to console (if wanted)
+	else if( serverlist_consoleoutput )
+		Con_Printf("%s\n%s\n", serverlist_cache[n].line1, serverlist_cache[n].line2);
+	// and finally, update the view set
+	ServerList_ViewList_Insert( &serverlist_cache[n] );
+	serverlist_cache[n].query = SQS_QUERIED;
+}
+
 static int NetConn_ClientParsePacket(lhnetsocket_t *mysocket, unsigned char *data, int length, lhnetaddress_t *peeraddress)
 {
 	qboolean fromserver;
@@ -1308,14 +1382,25 @@ static int NetConn_ClientParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 				return true;
 
 			info = &serverlist_cache[n].info;
-			if ((s = SearchInfostring(string, "gamename"     )) != NULL) strlcpy(info->game, s, sizeof (info->game));else info->game[0] = 0;
-			if ((s = SearchInfostring(string, "modname"      )) != NULL) strlcpy(info->mod , s, sizeof (info->mod ));else info->mod[0]  = 0;
-			if ((s = SearchInfostring(string, "mapname"      )) != NULL) strlcpy(info->map , s, sizeof (info->map ));else info->map[0]  = 0;
-			if ((s = SearchInfostring(string, "hostname"     )) != NULL) strlcpy(info->name, s, sizeof (info->name));else info->name[0] = 0;
-			if ((s = SearchInfostring(string, "protocol"     )) != NULL) info->protocol = atoi(s);else info->protocol = -1;
-			if ((s = SearchInfostring(string, "clients"      )) != NULL) info->numplayers = atoi(s);else info->numplayers = 0;
-			if ((s = SearchInfostring(string, "sv_maxclients")) != NULL) info->maxplayers = atoi(s);else info->maxplayers  = 0;
-			if ((s = SearchInfostring(string, "gameversion"  )) != NULL) info->gameversion = atoi(s);else info->gameversion = 0;
+			info->game[0] = 0;
+			info->mod[0]  = 0;
+			info->map[0]  = 0;
+			info->name[0] = 0;
+			info->protocol = -1;
+			info->numplayers = 0;
+			info->numbots = -1;
+			info->maxplayers  = 0;
+			info->gameversion = 0;
+			if ((s = SearchInfostring(string, "gamename"     )) != NULL) strlcpy(info->game, s, sizeof (info->game));
+			if ((s = SearchInfostring(string, "modname"      )) != NULL) strlcpy(info->mod , s, sizeof (info->mod ));
+			if ((s = SearchInfostring(string, "mapname"      )) != NULL) strlcpy(info->map , s, sizeof (info->map ));
+			if ((s = SearchInfostring(string, "hostname"     )) != NULL) strlcpy(info->name, s, sizeof (info->name));
+			if ((s = SearchInfostring(string, "protocol"     )) != NULL) info->protocol = atoi(s);
+			if ((s = SearchInfostring(string, "clients"      )) != NULL) info->numplayers = atoi(s);
+			if ((s = SearchInfostring(string, "bots"         )) != NULL) info->numbots = atoi(s);
+			if ((s = SearchInfostring(string, "sv_maxclients")) != NULL) info->maxplayers = atoi(s);
+			if ((s = SearchInfostring(string, "gameversion"  )) != NULL) info->gameversion = atoi(s);
+			info->numhumans = info->numplayers - max(0, info->numbots);
 
 			NetConn_ClientParsePacket_ServerList_UpdateCache(n);
 
@@ -1516,9 +1601,12 @@ static int NetConn_ClientParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 		int n;
 		serverlist_info_t *info;
 
-		c = data[4];
-		data += 5;
-		length -= 5;
+		data += 4;
+		length -= 4;
+		SZ_Clear(&net_message);
+		SZ_Write(&net_message, data, length);
+		MSG_BeginReading();
+		c = MSG_ReadByte();
 		switch (c)
 		{
 		case CCREP_ACCEPT:
@@ -1528,13 +1616,7 @@ static int NetConn_ClientParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 			{
 				lhnetaddress_t clientportaddress;
 				clientportaddress = *peeraddress;
-				if (length >= 4)
-				{
-					unsigned int port = (data[0] << 0) | (data[1] << 8) | (data[2] << 16) | (data[3] << 24);
-					data += 4;
-					length -= 4;
-					LHNETADDRESS_SetPort(&clientportaddress, port);
-				}
+				LHNETADDRESS_SetPort(&clientportaddress, MSG_ReadLong());
 				// update the server IP in the userinfo (QW servers expect this, and it is used by the reconnect command)
 				InfoString_SetValue(cls.userinfo, sizeof(cls.userinfo), "*ip", addressstring2);
 				M_Update_Return_Reason("Accepted");
@@ -1545,7 +1627,7 @@ static int NetConn_ClientParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 			if (developer.integer >= 10)
 				Con_Printf("Datagram_ParseConnectionless: received CCREP_REJECT from %s.\n", addressstring2);
 			cls.connect_trying = false;
-			M_Update_Return_Reason((char *)data);
+			M_Update_Return_Reason((char *)MSG_ReadString());
 			break;
 		case CCREP_SERVER_INFO:
 			if (developer.integer >= 10)
@@ -1583,6 +1665,7 @@ static int NetConn_ClientParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 		default:
 			break;
 		}
+		SZ_Clear(&net_message);
 		// we may not have liked the packet, but it was a valid control
 		// packet, so we're done processing this packet now
 		return true;
@@ -1654,8 +1737,7 @@ void NetConn_QueryQueueFrame(void)
 					NetConn_WriteString(cl_sockets[socket], "\377\377\377\377getinfo", &address);
 			}
 
-			//entry->querytime = realtime;
-			entry->querytime = Sys_DoubleTime();
+			entry->querytime = realtime;
 			entry->querycounter++;
 
 			// if not in the slist menu we should print the server to console
@@ -1739,24 +1821,30 @@ static void NetConn_BuildChallengeString(char *buffer, int bufferlength)
 
 static qboolean NetConn_BuildStatusResponse(const char* challenge, char* out_msg, size_t out_size, qboolean fullstatus)
 {
-	unsigned int nb_clients = 0, i;
+	unsigned int nb_clients = 0, nb_bots = 0, i;
 	int length;
 
 	// How many clients are there?
 	for (i = 0;i < (unsigned int)svs.maxclients;i++)
+	{
 		if (svs.clients[i].active)
+		{
 			nb_clients++;
+			if (!svs.clients[i].netconnection)
+				nb_bots++;
+		}
+	}
 
 	// TODO: we should add more information for the full status string
 	length = dpsnprintf(out_msg, out_size,
 						"\377\377\377\377%s\x0A"
 						"\\gamename\\%s\\modname\\%s\\gameversion\\%d\\sv_maxclients\\%d"
-						"\\clients\\%d\\mapname\\%s\\hostname\\%s\\protocol\\%d"
+						"\\clients\\%d\\bots\\%d\\mapname\\%s\\hostname\\%s\\protocol\\%d"
 						"%s%s"
 						"%s",
 						fullstatus ? "statusResponse" : "infoResponse",
 						gamename, com_modname, gameversion.integer, svs.maxclients,
-						nb_clients, sv.name, hostname.string, NET_PROTOCOL_VERSION,
+						nb_clients, nb_bots, sv.name, hostname.string, NET_PROTOCOL_VERSION,
 						challenge ? "\\challenge\\" : "", challenge ? challenge : "",
 						fullstatus ? "\n" : "");
 
@@ -1882,6 +1970,7 @@ static int NetConn_ServerParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 	double besttime;
 	client_t *client;
 	char *s, *string, response[1400], addressstring2[128], stringbuf[16384];
+	qboolean islocal = (LHNETADDRESS_GetAddressType(peeraddress) == LHNETADDRESSTYPE_LOOP);
 
 	if (!sv.active)
 		return false;
@@ -1915,7 +2004,7 @@ static int NetConn_ServerParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 			Com_HexDumpToConsole(data, length);
 		}
 
-		if (length >= 12 && !memcmp(string, "getchallenge", 12) && sv_public.integer > -2)
+		if (length >= 12 && !memcmp(string, "getchallenge", 12) && (islocal || sv_public.integer > -2))
 		{
 			for (i = 0, best = 0, besttime = realtime;i < MAX_CHALLENGES;i++)
 			{
@@ -1937,7 +2026,7 @@ static int NetConn_ServerParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 			NetConn_WriteString(mysocket, va("\377\377\377\377challenge %s", challenge[i].string), peeraddress);
 			return true;
 		}
-		if (length > 8 && !memcmp(string, "connect\\", 8) && sv_public.integer > -2)
+		if (length > 8 && !memcmp(string, "connect\\", 8) && (islocal || sv_public.integer > -2))
 		{
 			string += 7;
 			length -= 7;
@@ -2027,7 +2116,7 @@ static int NetConn_ServerParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 
 			return true;
 		}
-		if (length >= 7 && !memcmp(string, "getinfo", 7) && sv_public.integer > -1)
+		if (length >= 7 && !memcmp(string, "getinfo", 7) && (islocal || sv_public.integer > -1))
 		{
 			const char *challenge = NULL;
 
@@ -2043,7 +2132,7 @@ static int NetConn_ServerParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 			}
 			return true;
 		}
-		if (length >= 9 && !memcmp(string, "getstatus", 9) && sv_public.integer > -1)
+		if (length >= 9 && !memcmp(string, "getstatus", 9) && (islocal || sv_public.integer > -1))
 		{
 			const char *challenge = NULL;
 
@@ -2119,20 +2208,26 @@ static int NetConn_ServerParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 	//  Quake packet flood Denial Of Service attacks)
 	if (length >= 5 && (i = BigLong(*((int *)data))) && (i & (~NETFLAG_LENGTH_MASK)) == (int)NETFLAG_CTL && (i & NETFLAG_LENGTH_MASK) == length && (sv.protocol == PROTOCOL_QUAKE || sv.protocol == PROTOCOL_QUAKEDP || sv.protocol == PROTOCOL_NEHAHRAMOVIE || sv.protocol == PROTOCOL_DARKPLACES1 || sv.protocol == PROTOCOL_DARKPLACES2 || sv.protocol == PROTOCOL_DARKPLACES3))
 	{
-		int c = data[4];
-		data += 5;
-		length -= 5;
+		int c;
+		int protocolnumber;
+		const char *protocolname;
+		data += 4;
+		length -= 4;
+		SZ_Clear(&net_message);
+		SZ_Write(&net_message, data, length);
+		MSG_BeginReading();
+		c = MSG_ReadByte();
 		switch (c)
 		{
 		case CCREQ_CONNECT:
 			if (developer.integer >= 10)
 				Con_Printf("Datagram_ParseConnectionless: received CCREQ_CONNECT from %s.\n", addressstring2);
-			if (length < (int)strlen("QUAKE") + 1 + 1)
-				break;
-			if(sv_public.integer <= -2)
+			if(!islocal && sv_public.integer <= -2)
 				break;
 
-			if (memcmp(data, "QUAKE", strlen("QUAKE") + 1) != 0 || (int)data[strlen("QUAKE") + 1] != NET_PROTOCOL_VERSION)
+			protocolname = MSG_ReadString();
+			protocolnumber = MSG_ReadByte();
+			if (strcmp(protocolname, "QUAKE") || protocolnumber != NET_PROTOCOL_VERSION)
 			{
 				if (developer.integer >= 10)
 					Con_Printf("Datagram_ParseConnectionless: sending CCREP_REJECT \"Incompatible version.\" to %s.\n", addressstring2);
@@ -2228,7 +2323,7 @@ static int NetConn_ServerParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 		case CCREQ_SERVER_INFO:
 			if (developer.integer >= 10)
 				Con_Printf("Datagram_ParseConnectionless: received CCREQ_SERVER_INFO from %s.\n", addressstring2);
-			if(sv_public.integer <= -1)
+			if(!islocal && sv_public.integer <= -1)
 				break;
 			if (sv.active && !strcmp(MSG_ReadString(), "QUAKE"))
 			{
@@ -2259,7 +2354,7 @@ static int NetConn_ServerParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 		case CCREQ_PLAYER_INFO:
 			if (developer.integer >= 10)
 				Con_Printf("Datagram_ParseConnectionless: received CCREQ_PLAYER_INFO from %s.\n", addressstring2);
-			if(sv_public.integer <= -1)
+			if(!islocal && sv_public.integer <= -1)
 				break;
 			if (sv.active)
 			{
@@ -2280,7 +2375,7 @@ static int NetConn_ServerParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 					MSG_WriteByte(&net_message, playerNumber);
 					MSG_WriteString(&net_message, client->name);
 					MSG_WriteLong(&net_message, client->colors);
-					MSG_WriteLong(&net_message, (int)client->edict->fields.server->frags);
+					MSG_WriteLong(&net_message, client->frags);
 					MSG_WriteLong(&net_message, (int)(realtime - client->connecttime));
 					MSG_WriteString(&net_message, client->netconnection ? client->netconnection->address : "botclient");
 					*((int *)net_message.data) = BigLong(NETFLAG_CTL | (net_message.cursize & NETFLAG_LENGTH_MASK));
@@ -2292,7 +2387,7 @@ static int NetConn_ServerParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 		case CCREQ_RULE_INFO:
 			if (developer.integer >= 10)
 				Con_Printf("Datagram_ParseConnectionless: received CCREQ_RULE_INFO from %s.\n", addressstring2);
-			if(sv_public.integer <= -1)
+			if(!islocal && sv_public.integer <= -1)
 				break;
 			if (sv.active)
 			{
@@ -2321,6 +2416,7 @@ static int NetConn_ServerParsePacket(lhnetsocket_t *mysocket, unsigned char *dat
 		default:
 			break;
 		}
+		SZ_Clear(&net_message);
 		// we may not have liked the packet, but it was a valid control
 		// packet, so we're done processing this packet now
 		return true;
@@ -2358,6 +2454,11 @@ void NetConn_ServerFrame(void)
 	}
 }
 
+void NetConn_SleepMicroseconds(int microseconds)
+{
+	LHNET_SleepUntilPacket_Microseconds(microseconds);
+}
+
 void NetConn_QueryMasters(qboolean querydp, qboolean queryqw)
 {
 	int i;
@@ -2579,7 +2680,8 @@ void NetConn_Init(void)
 	Cvar_RegisterVariable(&net_connecttimeout);
 	Cvar_RegisterVariable(&net_connectfloodblockingtimeout);
 	Cvar_RegisterVariable(&cl_netlocalping);
-	Cvar_RegisterVariable(&cl_netpacketloss);
+	Cvar_RegisterVariable(&cl_netpacketloss_send);
+	Cvar_RegisterVariable(&cl_netpacketloss_receive);
 	Cvar_RegisterVariable(&hostname);
 	Cvar_RegisterVariable(&developer_networking);
 	Cvar_RegisterVariable(&cl_netport);
