@@ -1,26 +1,19 @@
 
 #include "quakedef.h"
 
-#define E5_PROTOCOL_PRIORITYLEVELS 32
-
 // this is 88 bytes (must match entity_state_t in protocol.h)
 entity_state_t defaultstate =
 {
 	// ! means this is not sent to client
 	0,//double time; // ! time this state was built (used on client for interpolation)
+	{0,0,0},//float netcenter[3]; // ! for network prioritization, this is the center of the bounding box (which may differ from the origin)
 	{0,0,0},//float origin[3];
 	{0,0,0},//float angles[3];
 	0,//int number; // entity number this state is for
 	0,//int effects;
-	0,//unsigned int customizeentityforclient; // !
 	0,//unsigned short modelindex;
 	0,//unsigned short frame;
 	0,//unsigned short tagentity;
-	0,//unsigned short specialvisibilityradius; // ! larger if it has effects/light
-	0,//unsigned short viewmodelforclient; // !
-	0,//unsigned short exteriormodelforclient; // ! not shown if first person viewing from this entity, shown in all other cases
-	0,//unsigned short nodrawtoclient; // !
-	0,//unsigned short drawonlytoclient; // !
 	{0,0,0,0},//unsigned short light[4]; // color*256 (0.00 to 255.996), and radius*1
 	0,//unsigned char active; // true if a valid state
 	0,//unsigned char lightstyle;
@@ -32,10 +25,11 @@ entity_state_t defaultstate =
 	0,//unsigned char glowsize;
 	254,//unsigned char glowcolor;
 	0,//unsigned char flags;
+	0,//unsigned char internaleffects; // INTEF_FLAG1QW and so on
 	0,//unsigned char tagindex;
 	{32, 32, 32},//unsigned char colormod[3];
 	// padding to a multiple of 8 bytes (to align the double time)
-	{0,0,0,0,0,0}//unsigned char unused[6]; // !
+	0//unsigned char unused; // !
 };
 
 // LordHavoc: I own protocol ranges 96, 97, 3500-3599
@@ -58,21 +52,10 @@ protocolversioninfo[] =
 	{15, "QUAKEDP"},
 	{250, "NEHAHRAMOVIE"},
 	{15, "QUAKE"},
-	{28, "QUAKEWORLD"},
+	{28, "QW"},
 	{0, NULL}
 };
 
-static mempool_t *sv2csqc = NULL;
-int csqc_clent = 0;
-sizebuf_t *sv2csqcbuf = NULL;
-static unsigned char *sv2csqcents_version[MAX_SCOREBOARD];
-
-static entity_frame_t deltaframe; // FIXME?
-static entity_frame_t framedata; // FIXME?
-
-int entityframe5_prioritychaincounts[E5_PROTOCOL_PRIORITYLEVELS];
-unsigned short entityframe5_prioritychains[E5_PROTOCOL_PRIORITYLEVELS][ENTITYFRAME5_MAXSTATES];
-
 protocolversion_t Protocol_EnumForName(const char *s)
 {
 	int i;
@@ -110,8 +93,8 @@ void Protocol_Names(char *buffer, size_t buffersize)
 	for (i = 1;protocolversioninfo[i].name;i++)
 	{
 		if (i > 1)
-			strlcat(buffer, " ", sizeof(buffer));
-		strlcat(buffer, protocolversioninfo[i].name, sizeof(buffer));
+			strlcat(buffer, " ", buffersize);
+		strlcat(buffer, protocolversioninfo[i].name, buffersize);
 	}
 }
 
@@ -249,132 +232,212 @@ void EntityFrameQuake_ISeeDeadEntities(void)
 	}
 }
 
-void EntityFrameCSQC_ClearVersions (void)
-{
-	if(sv2csqc)
-	{
-		Mem_FreePool(&sv2csqc);
-		sv2csqc = NULL;
-	}
-	memset(sv2csqcents_version, 0, MAX_SCOREBOARD*sizeof(unsigned char *));
-}
+// FIXME FIXME FIXME: at this time the CSQC entity writing does not store
+// packet logs and thus if an update is lost it is never repeated, this makes
+// csqc entities useless at the moment.
 
-void EntityFrameCSQC_InitClientVersions (int client, qboolean clear)
+void EntityFrameCSQC_WriteState (sizebuf_t *msg, int number, qboolean doupdate, qboolean *sectionstarted)
 {
-	if(!sv2csqc)
-		sv2csqc = Mem_AllocPool("SV2CSQC", 0, NULL);
-	if(sv2csqcents_version[client])
+	int version;
+	prvm_eval_t *val, *val2;
+	version = 0;
+	if (doupdate)
 	{
-		Mem_Free(sv2csqcents_version[client]);
-		sv2csqcents_version[client] = NULL;
+		if (msg->cursize + !*sectionstarted + 2 + 1 + 2 > msg->maxsize)
+			return;
+		val2 = PRVM_EDICTFIELDVALUE((&prog->edicts[number]), prog->fieldoffsets.Version);
+		version = (int)val2->_float;
+		// LordHavoc: do some validity checks on self.Version
+		// if self.Version reaches 255, it will soon exceed the byte used to
+		// store an entity version in the client struct, so we need to reset
+		// all the version to 1 and force all the existing clients' version of
+		// it to 255 (which we're not allowing to actually occur)
+		if (version < 0)
+			val2->_float = 0;
+		if (version >= 255)
+		{
+			int i;
+			val2->_float = version = 1;
+			// since we just reset the Version field to 1, it may accidentally
+			// end up being equal to an existing client version now or in the
+			// future, so to fix this situation we have to loop over all
+			// clients and change their versions for this entity to be -1
+			// which never matches, thus causing them to receive the update
+			// soon, as they should
+			for (i = 0;i < svs.maxclients;i++)
+				if (svs.clients[sv.writeentitiestoclient_clientnumber].csqcentityversion[number])
+					svs.clients[sv.writeentitiestoclient_clientnumber].csqcentityversion[number] = 255;
+		}
 	}
-	sv2csqcents_version[client] = (unsigned char *)Mem_Alloc(sv2csqc, MAX_EDICTS);
-	memset(sv2csqcents_version[client], 0, MAX_EDICTS);
+	// if the version already matches, we don't need to do anything as the
+	// latest version has already been sent.
+	if (svs.clients[sv.writeentitiestoclient_clientnumber].csqcentityversion[number] == version)
+		return;
+	if (version)
+	{
+		// if there's no SendEntity function, treat it as a remove
+		val = PRVM_EDICTFIELDVALUE((&prog->edicts[number]), prog->fieldoffsets.SendEntity);
+		if (val->function)
+		{
+			// there is a function to call, save the cursize value incase we
+			// have to do a rollback due to overflow
+			int oldcursize = msg->cursize;
+			if(!*sectionstarted)
+				MSG_WriteByte(msg, svc_csqcentities);
+			MSG_WriteShort(msg, number);
+			((int *)prog->globals.generic)[OFS_PARM0] = sv.writeentitiestoclient_cliententitynumber;
+			prog->globals.server->self = number;
+			PRVM_ExecuteProgram(val->function, "Null SendEntity\n");
+			if(prog->globals.generic[OFS_RETURN])
+			{
+				if (msg->cursize + 2 > msg->maxsize)
+				{
+					// if the packet no longer has enough room to write the
+					// final index code that ends the message, rollback to the
+					// state before we tried to write anything and then return
+					msg->cursize = oldcursize;
+					msg->overflowed = false;
+					return;
+				}
+				// an update has been successfully written, update the version
+				svs.clients[sv.writeentitiestoclient_clientnumber].csqcentityversion[number] = version;
+				// and take note that we have begun the svc_csqcentities
+				// section of the packet
+				*sectionstarted = 1;
+				return;
+			}
+			else
+			{
+				// rollback the buffer to its state before the writes
+				msg->cursize = oldcursize;
+				// if the function returned FALSE, simply write a remove
+				// this is done by falling through to the remove code below
+				version = 0;
+			}
+		}
+	}
+	// write a remove message if needed
+	// if already removed, do nothing
+	if (!svs.clients[sv.writeentitiestoclient_clientnumber].csqcentityversion[number])
+		return;
+	// if there isn't enough room to write the remove message, just return, as
+	// it will be handled in a later packet
+	if (msg->cursize + !*sectionstarted + 2 + 2 > msg->maxsize)
+		return;
+	// first write the message identifier if needed
+	if(!*sectionstarted)
+	{
+		*sectionstarted = 1;
+		MSG_WriteByte(msg, svc_csqcentities);
+	}
+	// write the remove message
+	MSG_WriteShort(msg, (unsigned short)number | 0x8000);
+	svs.clients[sv.writeentitiestoclient_clientnumber].csqcentityversion[number] = 0;
 }
 
 //[515]: we use only one array per-client for SendEntity feature
 void EntityFrameCSQC_WriteFrame (sizebuf_t *msg, int numstates, const entity_state_t *states)
 {
-	sizebuf_t				buf;
-	unsigned char					data[2048];
-	const entity_state_t	*s;
-	unsigned short			i, t, t2, t0;
-	prvm_eval_t				*val, *val2;
-	int						csqcents = 0;
+	int i, num;
+	qboolean sectionstarted = false;
+	const entity_state_t *n;
 
-	if(!eval_SendEntity || !eval_Version)
+	// if this server progs is not CSQC-aware, return early
+	if(prog->fieldoffsets.SendEntity < 0 || prog->fieldoffsets.Version < 0)
+		return;
+	// make sure there is enough room to store the svc_csqcentities byte,
+	// the terminator (0x0000) and at least one entity update
+	if (msg->cursize + 32 >= msg->maxsize)
 		return;
-	--csqc_clent;
-	if(!sv2csqcents_version[csqc_clent])
-		EntityFrameCSQC_InitClientVersions(csqc_clent, false);
 
-	for (csqcents = i = 0, s = states;i < numstates;i++, s++)
+	num = 1;
+	for (i = 0, n = states;i < numstates;i++, n++)
 	{
-		//[515]: entities remove
-		if(i+1 >= numstates)
-			t2 = prog->num_edicts;
-		else
-			t2 = states[i+1].number;
-		if(!i)
-		{
-			t0 = 1;
-			t2 = s->number;
-		}
-		else
-			t0 = s->number+1;
-		for(t=t0; t<t2 ;t++)
-			if(sv2csqcents_version[csqc_clent][t])
-			{
-				if(!csqcents)
-				{
-					csqcents = 1;
-					memset(&buf, 0, sizeof(buf));
-					buf.data = data;
-					buf.maxsize = sizeof(data);
-					sv2csqcbuf = &buf;
-					SZ_Clear(&buf);
-					MSG_WriteByte(&buf, svc_csqcentities);
-				}
-				sv2csqcents_version[csqc_clent][t] = 0;
-				MSG_WriteShort(&buf, (unsigned short)t | 0x8000);
-				csqcents++;
-			}
-		//[515]: entities remove
+		// all entities between the previous entity state and this one are dead
+		for (;num < n->number;num++)
+			if(svs.clients[sv.writeentitiestoclient_clientnumber].csqcentityversion[num])
+				EntityFrameCSQC_WriteState(msg, num, false, &sectionstarted);
+		// update this entity
+		EntityFrameCSQC_WriteState(msg, num, true, &sectionstarted);
+		// advance to next entity so the next iteration doesn't immediately remove it
+		num++;
+	}
+	// all remaining entities are dead
+	for (;num < prog->num_edicts;num++)
+		if(svs.clients[sv.writeentitiestoclient_clientnumber].csqcentityversion[num])
+			EntityFrameCSQC_WriteState(msg, num, false, &sectionstarted);
+	if (sectionstarted)
+	{
+		// write index 0 to end the update (0 is never used by real entities)
+		MSG_WriteShort(msg, 0);
+	}
+}
 
-//		if(!s->active)
-//			continue;
-		val = PRVM_GETEDICTFIELDVALUE((&prog->edicts[s->number]), eval_SendEntity);
-		if(val->function)
+void Protocol_UpdateClientStats(const int *stats)
+{
+	int i;
+	// update the stats array and set deltabits for any changed stats
+	for (i = 0;i < MAX_CL_STATS;i++)
+	{
+		if (host_client->stats[i] != stats[i])
 		{
-			val2 = PRVM_GETEDICTFIELDVALUE((&prog->edicts[s->number]), eval_Version);
-			if(sv2csqcents_version[csqc_clent][s->number] == (unsigned char)val2->_float)
-				continue;
-			if(!csqcents)
+			host_client->statsdeltabits[i >> 3] |= 1 << (i & 7);
+			host_client->stats[i] = stats[i];
+		}
+	}
+}
+
+void Protocol_WriteStatsReliable(void)
+{
+	int i;
+	if (!host_client->netconnection)
+		return;
+	// detect changes in stats and write reliable messages
+	// this only deals with 32 stats because the older protocols which use
+	// this function can only cope with 32 stats,
+	// they also do not support svc_updatestatubyte which was introduced in
+	// DP6 protocol (except for QW)
+	for (i = 0;i < 32;i++)
+	{
+		// quickly skip zero bytes
+		if (!host_client->statsdeltabits[i >> 3])
+		{
+			i |= 7;
+			continue;
+		}
+		// check if this bit is set
+		if (host_client->statsdeltabits[i >> 3] & (1 << (i & 7)))
+		{
+			host_client->statsdeltabits[i >> 3] -= (1 << (i & 7));
+			// send the stat as a byte if possible
+			if (sv.protocol == PROTOCOL_QUAKEWORLD)
 			{
-				csqcents = 1;
-				memset(&buf, 0, sizeof(buf));
-				buf.data = data;
-				buf.maxsize = sizeof(data);
-				sv2csqcbuf = &buf;
-				SZ_Clear(&buf);
-				MSG_WriteByte(&buf, svc_csqcentities);
-			}
-			if((unsigned char)val2->_float == 0)
-				val2->_float = 1;
-			MSG_WriteShort(&buf, s->number);
-			((int *)prog->globals.generic)[OFS_PARM0] = csqc_clent+1;
-			prog->globals.server->self = s->number;
-			PRVM_ExecuteProgram(val->function, "Null SendEntity\n");
-			if(!prog->globals.generic[OFS_RETURN])
-			{
-				buf.cursize -= 2;
-				if(sv2csqcents_version[csqc_clent][s->number])
+				if (host_client->stats[i] >= 0 && host_client->stats[i] < 256)
 				{
-					sv2csqcents_version[csqc_clent][s->number] = 0;
-					MSG_WriteShort(&buf, (unsigned short)s->number | 0x8000);
-					csqcents++;
+					MSG_WriteByte(&host_client->netconnection->message, qw_svc_updatestat);
+					MSG_WriteByte(&host_client->netconnection->message, i);
+					MSG_WriteByte(&host_client->netconnection->message, host_client->stats[i]);
+				}
+				else
+				{
+					MSG_WriteByte(&host_client->netconnection->message, qw_svc_updatestatlong);
+					MSG_WriteByte(&host_client->netconnection->message, i);
+					MSG_WriteLong(&host_client->netconnection->message, host_client->stats[i]);
 				}
 			}
 			else
 			{
-				sv2csqcents_version[csqc_clent][s->number] = (unsigned char)val2->_float;
-				csqcents++;
+				// this could make use of svc_updatestatubyte in DP6 and later
+				// protocols but those protocols do not use this function
+				MSG_WriteByte(&host_client->netconnection->message, svc_updatestat);
+				MSG_WriteByte(&host_client->netconnection->message, i);
+				MSG_WriteLong(&host_client->netconnection->message, host_client->stats[i]);
 			}
-			if (msg->cursize + buf.cursize > msg->maxsize)
-				break;
 		}
 	}
-	if(csqcents)
-	{
-		if(csqcents > 1)
-		{
-			MSG_WriteShort(&buf, 0);
-			SZ_Write(msg, buf.data, buf.cursize);
-		}
-		sv2csqcbuf = NULL;
-	}
 }
 
+
 void EntityFrameQuake_WriteFrame(sizebuf_t *msg, int numstates, const entity_state_t *states)
 {
 	const entity_state_t *s;
@@ -391,7 +454,7 @@ void EntityFrameQuake_WriteFrame(sizebuf_t *msg, int numstates, const entity_sta
 
 	for (i = 0, s = states;i < numstates;i++, s++)
 	{
-		val = PRVM_GETEDICTFIELDVALUE((&prog->edicts[s->number]), eval_SendEntity);
+		val = PRVM_EDICTFIELDVALUE((&prog->edicts[s->number]), prog->fieldoffsets.SendEntity);
 		if(val && val->function)
 			continue;
 
@@ -455,6 +518,8 @@ void EntityFrameQuake_WriteFrame(sizebuf_t *msg, int numstates, const entity_sta
 			bits |= U_GLOWSIZE;
 		if (baseline.glowcolor != s->glowcolor)
 			bits |= U_GLOWCOLOR;
+		if (!VectorCompare(baseline.colormod, s->colormod))
+			bits |= U_COLORMOD;
 
 		// if extensions are disabled, clear the relevant update flags
 		if (sv.protocol == PROTOCOL_QUAKE || sv.protocol == PROTOCOL_NEHAHRAMOVIE)
@@ -1014,7 +1079,7 @@ void EntityFrame_AddFrame(entityframe_database_t *d, vec3_t eye, int framenum, i
 void EntityFrame_WriteFrame(sizebuf_t *msg, entityframe_database_t *d, int numstates, const entity_state_t *states, int viewentnum)
 {
 	int i, onum, number;
-	entity_frame_t *o = &deltaframe;
+	entity_frame_t *o = &d->deltaframe;
 	const entity_state_t *ent, *delta;
 	vec3_t eye;
 	prvm_eval_t *val;
@@ -1048,7 +1113,7 @@ void EntityFrame_WriteFrame(sizebuf_t *msg, entityframe_database_t *d, int numst
 		ent = states + i;
 		number = ent->number;
 
-		val = PRVM_GETEDICTFIELDVALUE((&prog->edicts[number]), eval_SendEntity);
+		val = PRVM_EDICTFIELDVALUE((&prog->edicts[number]), prog->fieldoffsets.SendEntity);
 		if(val && val->function)
 				continue;
 		for (;onum < o->numentities && o->entitydata[onum].number < number;onum++)
@@ -1082,13 +1147,15 @@ void EntityFrame_WriteFrame(sizebuf_t *msg, entityframe_database_t *d, int numst
 void EntityFrame_CL_ReadFrame(void)
 {
 	int i, number, removed;
-	entity_frame_t *f = &framedata, *delta = &deltaframe;
+	entity_frame_t *f, *delta;
 	entity_state_t *e, *old, *oldend;
 	entity_t *ent;
 	entityframe_database_t *d;
 	if (!cl.entitydatabase)
 		cl.entitydatabase = EntityFrame_AllocDatabase(cls.levelmempool);
 	d = cl.entitydatabase;
+	f = &d->framedata;
+	delta = &d->deltaframe;
 
 	EntityFrame_Clear(f, NULL, -1);
 
@@ -1367,7 +1434,7 @@ void EntityFrame4_CL_ReadFrame(void)
 	cl.latestframenums[LATESTFRAMENUMS-1] = framenum = MSG_ReadLong();
 	// read the start number
 	enumber = (unsigned short) MSG_ReadShort();
-	if (developer_networkentities.integer >= 1)
+	if (developer_networkentities.integer >= 10)
 	{
 		Con_Printf("recv svc_entities num:%i ref:%i database: ref:%i commits:", framenum, referenceframenum, d->referenceframenum);
 		for (i = 0;i < MAX_ENTITY_HISTORY;i++)
@@ -1519,7 +1586,7 @@ void EntityFrame4_WriteFrame(sizebuf_t *msg, entityframe4_database_t *d, int num
 	MSG_WriteByte(msg, svc_entities);
 	MSG_WriteLong(msg, d->referenceframenum);
 	MSG_WriteLong(msg, d->currentcommit->framenum);
-	if (developer_networkentities.integer >= 1)
+	if (developer_networkentities.integer >= 10)
 	{
 		Con_Printf("send svc_entities num:%i ref:%i (database: ref:%i commits:", d->currentcommit->framenum, d->referenceframenum, d->referenceframenum);
 		for (i = 0;i < MAX_ENTITY_HISTORY;i++)
@@ -1537,7 +1604,7 @@ void EntityFrame4_WriteFrame(sizebuf_t *msg, entityframe4_database_t *d, int num
 	d->currententitynumber = 1;
 	for (i = 0, n = startnumber;n < prog->max_edicts;n++)
 	{
-		val = PRVM_GETEDICTFIELDVALUE((&prog->edicts[n]), eval_SendEntity);
+		val = PRVM_EDICTFIELDVALUE((&prog->edicts[n]), prog->fieldoffsets.SendEntity);
 		if(val && val->function)
 			continue;
 		// find the old state to delta from
@@ -1645,54 +1712,45 @@ void EntityFrame5_ExpandEdicts(entityframe5_database_t *d, int newmax)
 
 int EntityState5_Priority(entityframe5_database_t *d, int stateindex)
 {
-	int lowprecision, limit, priority;
-	double distance;
-	int changedbits;
-	int age;
-	entity_state_t *view, *s;
-	changedbits = d->deltabits[stateindex];
-	if (!changedbits)
-		return 0;
-	if (!d->states[stateindex].active/* && changedbits & E5_FULLUPDATE*/)
-		return E5_PROTOCOL_PRIORITYLEVELS - 1;
-	// check whole attachment chain to judge relevance to player
-	view = d->states + d->viewentnum;
-	lowprecision = false;
+	int limit, priority;
+	entity_state_t *s;
+	// if it is the player, update urgently
+	if (stateindex == d->viewentnum)
+		return ENTITYFRAME5_PRIORITYLEVELS - 1;
+	// priority increases each frame no matter what happens
+	priority = d->priorities[stateindex] + 1;
+	// players get an extra priority boost
+	if (stateindex <= svs.maxclients)
+		priority++;
+	// remove dead entities very quickly because they are just 2 bytes
+	if (!d->states[stateindex].active)
+	{
+		priority++;
+		return bound(1, priority, ENTITYFRAME5_PRIORITYLEVELS - 1);
+	}
+	// certain changes are more noticable than others
+	if (d->deltabits[stateindex] & (E5_FULLUPDATE | E5_ATTACHMENT | E5_MODEL | E5_FLAGS | E5_COLORMAP))
+		priority++;
+	// find the root entity this one is attached to, and judge relevance by it
 	for (limit = 0;limit < 256;limit++)
 	{
+		s = d->states + stateindex;
+		if (s->flags & RENDER_VIEWMODEL)
+			stateindex = d->viewentnum;
+		else if (s->tagentity)
+			stateindex = s->tagentity;
+		else
+			break;
 		if (d->maxedicts < stateindex)
 			EntityFrame5_ExpandEdicts(d, (stateindex+256)&~255);
-		s = d->states + stateindex;
-		if (s == view)
-			return E5_PROTOCOL_PRIORITYLEVELS - 1;
-		if (s->flags & RENDER_VIEWMODEL)
-			return E5_PROTOCOL_PRIORITYLEVELS - 1;
-		if (s->flags & RENDER_LOWPRECISION)
-			lowprecision = true;
-		if (!s->tagentity)
-		{
-			if (VectorCompare(s->origin, view->origin))
-				return E5_PROTOCOL_PRIORITYLEVELS - 1;
-			break;
-		}
-		stateindex = s->tagentity;
 	}
 	if (limit >= 256)
-	{
-		Con_Printf("Protocol: Runaway loop recursing tagentity links on entity %i\n", stateindex);
-		return 0;
-	}
-	// it's not a viewmodel for this client
-	distance = VectorDistance(view->origin, s->origin);
-	age = d->latestframenum - d->updateframenum[stateindex];
-	priority = (E5_PROTOCOL_PRIORITYLEVELS / 2) + age - (int)(distance * (E5_PROTOCOL_PRIORITYLEVELS / 16384.0f));
-	if (lowprecision)
-		priority -= (E5_PROTOCOL_PRIORITYLEVELS / 4);
-	//if (changedbits & E5_FULLUPDATE)
-	//	priority += 4;
-	//if (changedbits & (E5_ATTACHMENT | E5_MODEL | E5_FLAGS | E5_COLORMAP))
-	//	priority += 4;
-	return (int) bound(1, priority, E5_PROTOCOL_PRIORITYLEVELS - 1);
+		Con_DPrintf("Protocol: Runaway loop recursing tagentity links on entity %i\n", stateindex);
+	// now that we have the parent entity we can make some decisions based on
+	// distance from the player
+	if (VectorDistance(d->states[d->viewentnum].netcenter, s->netcenter) < 1024.0f)
+		priority++;
+	return bound(1, priority, ENTITYFRAME5_PRIORITYLEVELS - 1);
 }
 
 void EntityState5_WriteUpdate(int number, const entity_state_t *s, int changedbits, sizebuf_t *msg)
@@ -1700,7 +1758,7 @@ void EntityState5_WriteUpdate(int number, const entity_state_t *s, int changedbi
 	unsigned int bits = 0;
 
 	prvm_eval_t *val;
-	val = PRVM_GETEDICTFIELDVALUE((&prog->edicts[s->number]), eval_SendEntity);
+	val = PRVM_EDICTFIELDVALUE((&prog->edicts[s->number]), prog->fieldoffsets.SendEntity);
 	if(val && val->function)
 		return;
 
@@ -1709,7 +1767,7 @@ void EntityState5_WriteUpdate(int number, const entity_state_t *s, int changedbi
 	else
 	{
 		bits = changedbits;
-		if ((bits & E5_ORIGIN) && (!(s->flags & RENDER_LOWPRECISION) || s->origin[0] < -4096 || s->origin[0] >= 4096 || s->origin[1] < -4096 || s->origin[1] >= 4096 || s->origin[2] < -4096 || s->origin[2] >= 4096))
+		if ((bits & E5_ORIGIN) && ((s->flags & RENDER_EXTERIORMODEL) || s->origin[0] < -4096 || s->origin[0] >= 4096 || s->origin[1] < -4096 || s->origin[1] >= 4096 || s->origin[2] < -4096 || s->origin[2] >= 4096))
 			bits |= E5_ORIGIN32;
 		if ((bits & E5_ANGLES) && !(s->flags & RENDER_LOWPRECISION))
 			bits |= E5_ANGLES16;
@@ -1719,9 +1777,9 @@ void EntityState5_WriteUpdate(int number, const entity_state_t *s, int changedbi
 			bits |= E5_FRAME16;
 		if (bits & E5_EFFECTS)
 		{
-			if (s->effects >= 65536)
+			if (s->effects & 0xFFFF0000)
 				bits |= E5_EFFECTS32;
-			else if (s->effects >= 256)
+			else if (s->effects & 0xFFFFFF00)
 				bits |= E5_EFFECTS16;
 		}
 		if (bits >= 256)
@@ -1829,7 +1887,7 @@ void EntityState5_WriteUpdate(int number, const entity_state_t *s, int changedbi
 	}
 }
 
-void EntityState5_ReadUpdate(entity_state_t *s)
+void EntityState5_ReadUpdate(entity_state_t *s, int number)
 {
 	int bits;
 	bits = MSG_ReadByte();
@@ -1940,7 +1998,7 @@ void EntityState5_ReadUpdate(entity_state_t *s)
 
 	if (developer_networkentities.integer >= 2)
 	{
-		Con_Printf("ReadFields e%i", s->number);
+		Con_Printf("ReadFields e%i", number);
 
 		if (bits & E5_ORIGIN)
 			Con_Printf(" E5_ORIGIN %f %f %f", s->origin[0], s->origin[1], s->origin[2]);
@@ -1973,6 +2031,8 @@ void EntityState5_ReadUpdate(entity_state_t *s)
 				Con_Print(" SHADOW");
 			if (s->flags & RENDER_LIGHT)
 				Con_Print(" LIGHT");
+			if (s->flags & RENDER_NOSELFSHADOW)
+				Con_Print(" NOSELFSHADOW");
 			Con_Print(")");
 		}
 		if (bits & E5_ALPHA)
@@ -2044,10 +2104,10 @@ void EntityFrame5_CL_ReadFrame(void)
 	for (i = 0;i < LATESTFRAMENUMS-1;i++)
 		cl.latestframenums[i] = cl.latestframenums[i+1];
 	cl.latestframenums[LATESTFRAMENUMS-1] = MSG_ReadLong();
-	if (developer_networkentities.integer)
+	if (developer_networkentities.integer >= 10)
 		Con_Printf("recv: svc_entities %i\n", cl.latestframenums[LATESTFRAMENUMS-1]);
 	if (cls.protocol != PROTOCOL_QUAKE && cls.protocol != PROTOCOL_QUAKEDP && cls.protocol != PROTOCOL_NEHAHRAMOVIE && cls.protocol != PROTOCOL_DARKPLACES1 && cls.protocol != PROTOCOL_DARKPLACES2 && cls.protocol != PROTOCOL_DARKPLACES3 && cls.protocol != PROTOCOL_DARKPLACES4 && cls.protocol != PROTOCOL_DARKPLACES5 && cls.protocol != PROTOCOL_DARKPLACES6)
-		cl.servermovesequence = MSG_ReadLong();
+		cls.servermovesequence = MSG_ReadLong();
 	// read entity numbers until we find a 0x8000
 	// (which would be remove world entity, but is actually a terminator)
 	while ((n = (unsigned short)MSG_ReadShort()) != 0x8000 && !msg_badread)
@@ -2075,7 +2135,7 @@ void EntityFrame5_CL_ReadFrame(void)
 		else
 		{
 			// update entity
-			EntityState5_ReadUpdate(s);
+			EntityState5_ReadUpdate(s, enumber);
 		}
 		// set the cl.entities_active flag
 		cl.entities_active[enumber] = s->active;
@@ -2136,20 +2196,24 @@ void EntityFrame5_LostFrame(entityframe5_database_t *d, int framenum)
 				if (bits)
 				{
 					d->deltabits[s->number] |= bits;
-					d->priorities[s->number] = EntityState5_Priority(d, s->number);
+					// if it was a very important update, set priority higher
+					if (bits & (E5_FULLUPDATE | E5_ATTACHMENT | E5_MODEL || E5_COLORMAP))
+						d->priorities[s->number] = max(d->priorities[s->number], 4);
+					else
+						d->priorities[s->number] = max(d->priorities[s->number], 1);
 				}
 			}
 			// mark lost stats
 			for (j = 0;j < MAX_CL_STATS;j++)
 			{
 				for (l = 0;l < (MAX_CL_STATS+7)/8;l++)
-					statsdeltabits[l] = p->statsdeltabits[l] & ~d->statsdeltabits[l];
+					statsdeltabits[l] = p->statsdeltabits[l] & ~host_client->statsdeltabits[l];
 				for (k = 0, p2 = d->packetlog;k < ENTITYFRAME5_MAXPACKETLOGS;k++, p2++)
 					if (p2->packetnumber > framenum)
 						for (l = 0;l < (MAX_CL_STATS+7)/8;l++)
 							statsdeltabits[l] = p->statsdeltabits[l] & ~p2->statsdeltabits[l];
 				for (l = 0;l < (MAX_CL_STATS+7)/8;l++)
-					d->statsdeltabits[l] |= statsdeltabits[l];
+					host_client->statsdeltabits[l] |= statsdeltabits[l];
 			}
 			// delete this packet log as it is now obsolete
 			p->packetnumber = 0;
@@ -2166,7 +2230,7 @@ void EntityFrame5_AckFrame(entityframe5_database_t *d, int framenum)
 			d->packetlog[i].packetnumber = 0;
 }
 
-void EntityFrame5_WriteFrame(sizebuf_t *msg, entityframe5_database_t *d, int numstates, const entity_state_t *states, int viewentnum, int *stats, int movesequence)
+void EntityFrame5_WriteFrame(sizebuf_t *msg, entityframe5_database_t *d, int numstates, const entity_state_t *states, int viewentnum, int movesequence)
 {
 	const entity_state_t *n;
 	int i, num, l, framenum, packetlognumber, priority;
@@ -2197,18 +2261,8 @@ void EntityFrame5_WriteFrame(sizebuf_t *msg, entityframe5_database_t *d, int num
 	buf.data = data;
 	buf.maxsize = sizeof(data);
 
-	// detect changes in stats
-	for (i = 0;i < MAX_CL_STATS;i++)
-	{
-		if (d->stats[i] != stats[i])
-		{
-			d->statsdeltabits[i>>3] |= (1<<(i&7));
-			d->stats[i] = stats[i];
-		}
-	}
-
 	// detect changes in states
-	num = 0;
+	num = 1;
 	for (i = 0, n = states;i < numstates;i++, n++)
 	{
 		// mark gaps in entity numbering as removed entities
@@ -2219,7 +2273,7 @@ void EntityFrame5_WriteFrame(sizebuf_t *msg, entityframe5_database_t *d, int num
 			{
 				CLEARPVSBIT(d->visiblebits, num);
 				d->deltabits[num] = E5_FULLUPDATE;
-				d->priorities[num] = EntityState5_Priority(d, num);
+				d->priorities[num] = max(d->priorities[num], 8); // removal is cheap
 				d->states[num] = defaultstate;
 				d->states[num].number = num;
 			}
@@ -2230,10 +2284,13 @@ void EntityFrame5_WriteFrame(sizebuf_t *msg, entityframe5_database_t *d, int num
 			// entity just spawned in, don't let it completely hog priority
 			// because of being ancient on the first frame
 			d->updateframenum[num] = framenum;
+			// initial priority is a bit high to make projectiles send on the
+			// first frame, among other things
+			d->priorities[num] = max(d->priorities[num], 4);
 		}
 		SETPVSBIT(d->visiblebits, num);
 		d->deltabits[num] |= EntityState5_DeltaBits(d->states + num, n);
-		d->priorities[num] = EntityState5_Priority(d, num);
+		d->priorities[num] = max(d->priorities[num], 1);
 		d->states[num] = *n;
 		d->states[num].number = num;
 		// advance to next entity so the next iteration doesn't immediately remove it
@@ -2246,7 +2303,7 @@ void EntityFrame5_WriteFrame(sizebuf_t *msg, entityframe5_database_t *d, int num
 		{
 			CLEARPVSBIT(d->visiblebits, num);
 			d->deltabits[num] = E5_FULLUPDATE;
-			d->priorities[num] = EntityState5_Priority(d, num);
+			d->priorities[num] = max(d->priorities[num], 8); // removal is cheap
 			d->states[num] = defaultstate;
 			d->states[num].number = num;
 		}
@@ -2258,16 +2315,23 @@ void EntityFrame5_WriteFrame(sizebuf_t *msg, entityframe5_database_t *d, int num
 		return;
 
 	// build lists of entities by priority level
-	memset(entityframe5_prioritychaincounts, 0, sizeof(entityframe5_prioritychaincounts));
+	memset(d->prioritychaincounts, 0, sizeof(d->prioritychaincounts));
 	l = 0;
 	for (num = 0;num < d->maxedicts;num++)
 	{
 		if (d->priorities[num])
 		{
-			l = num;
-			priority = d->priorities[num];
-			if (entityframe5_prioritychaincounts[priority] < ENTITYFRAME5_MAXSTATES)
-				entityframe5_prioritychains[priority][entityframe5_prioritychaincounts[priority]++] = num;
+			if (d->deltabits[num])
+			{
+				if (d->priorities[num] < (ENTITYFRAME5_PRIORITYLEVELS - 1))
+					d->priorities[num] = EntityState5_Priority(d, num);
+				l = num;
+				priority = d->priorities[num];
+				if (d->prioritychaincounts[priority] < ENTITYFRAME5_MAXSTATES)
+					d->prioritychains[priority][d->prioritychaincounts[priority]++] = num;
+			}
+			else
+				d->priorities[num] = 0;
 		}
 	}
 
@@ -2280,38 +2344,38 @@ void EntityFrame5_WriteFrame(sizebuf_t *msg, entityframe5_database_t *d, int num
 	{
 		for (i = 0;i < MAX_CL_STATS && msg->cursize + 6 + 11 <= msg->maxsize;i++)
 		{
-			if (d->statsdeltabits[i>>3] & (1<<(i&7)))
+			if (host_client->statsdeltabits[i>>3] & (1<<(i&7)))
 			{
-				d->statsdeltabits[i>>3] &= ~(1<<(i&7));
+				host_client->statsdeltabits[i>>3] &= ~(1<<(i&7));
 				packetlog->statsdeltabits[i>>3] |= (1<<(i&7));
-				if (d->stats[i] >= 0 && d->stats[i] < 256)
+				if (host_client->stats[i] >= 0 && host_client->stats[i] < 256)
 				{
 					MSG_WriteByte(msg, svc_updatestatubyte);
 					MSG_WriteByte(msg, i);
-					MSG_WriteByte(msg, d->stats[i]);
+					MSG_WriteByte(msg, host_client->stats[i]);
 				}
 				else
 				{
 					MSG_WriteByte(msg, svc_updatestat);
 					MSG_WriteByte(msg, i);
-					MSG_WriteLong(msg, d->stats[i]);
+					MSG_WriteLong(msg, host_client->stats[i]);
 				}
 			}
 		}
 	}
 	// write state updates
-	if (developer_networkentities.integer)
+	if (developer_networkentities.integer >= 10)
 		Con_Printf("send: svc_entities %i\n", framenum);
 	d->latestframenum = framenum;
 	MSG_WriteByte(msg, svc_entities);
 	MSG_WriteLong(msg, framenum);
 	if (sv.protocol != PROTOCOL_QUAKE && sv.protocol != PROTOCOL_QUAKEDP && sv.protocol != PROTOCOL_NEHAHRAMOVIE && sv.protocol != PROTOCOL_DARKPLACES1 && sv.protocol != PROTOCOL_DARKPLACES2 && sv.protocol != PROTOCOL_DARKPLACES3 && sv.protocol != PROTOCOL_DARKPLACES4 && sv.protocol != PROTOCOL_DARKPLACES5 && sv.protocol != PROTOCOL_DARKPLACES6)
 		MSG_WriteLong(msg, movesequence);
-	for (priority = E5_PROTOCOL_PRIORITYLEVELS - 1;priority >= 0 && packetlog->numstates < ENTITYFRAME5_MAXSTATES;priority--)
+	for (priority = ENTITYFRAME5_PRIORITYLEVELS - 1;priority >= 0 && packetlog->numstates < ENTITYFRAME5_MAXSTATES;priority--)
 	{
-		for (i = 0;i < entityframe5_prioritychaincounts[priority] && packetlog->numstates < ENTITYFRAME5_MAXSTATES;i++)
+		for (i = 0;i < d->prioritychaincounts[priority] && packetlog->numstates < ENTITYFRAME5_MAXSTATES;i++)
 		{
-			num = entityframe5_prioritychains[priority][i];
+			num = d->prioritychains[priority][i];
 			n = d->states + num;
 			if (d->deltabits[num] & E5_FULLUPDATE)
 				d->deltabits[num] = E5_FULLUPDATE | EntityState5_DeltaBits(&defaultstate, n);
@@ -2337,43 +2401,43 @@ void EntityFrame5_WriteFrame(sizebuf_t *msg, entityframe5_database_t *d, int num
 }
 
 
-static int QW_TranslateEffects(int qweffects, int number)
+static void QW_TranslateEffects(entity_state_t *s, int qweffects)
 {
-	int effects = 0;
+	s->effects = 0;
+	s->internaleffects = 0;
 	if (qweffects & QW_EF_BRIGHTFIELD)
-		effects |= EF_BRIGHTFIELD;
+		s->effects |= EF_BRIGHTFIELD;
 	if (qweffects & QW_EF_MUZZLEFLASH)
-		effects |= EF_MUZZLEFLASH;
+		s->effects |= EF_MUZZLEFLASH;
 	if (qweffects & QW_EF_FLAG1)
 	{
 		// mimic FTEQW's interpretation of EF_FLAG1 as EF_NODRAW on non-player entities
-		if (number > cl.maxclients)
-			effects |= EF_NODRAW;
+		if (s->number > cl.maxclients)
+			s->effects |= EF_NODRAW;
 		else
-			effects |= EF_FLAG1QW;
+			s->internaleffects |= INTEF_FLAG1QW;
 	}
 	if (qweffects & QW_EF_FLAG2)
 	{
 		// mimic FTEQW's interpretation of EF_FLAG2 as EF_ADDITIVE on non-player entities
-		if (number > cl.maxclients)
-			effects |= EF_ADDITIVE;
+		if (s->number > cl.maxclients)
+			s->effects |= EF_ADDITIVE;
 		else
-			effects |= EF_FLAG2QW;
+			s->internaleffects |= INTEF_FLAG2QW;
 	}
 	if (qweffects & QW_EF_RED)
 	{
 		if (qweffects & QW_EF_BLUE)
-			effects |= EF_RED | EF_BLUE;
+			s->effects |= EF_RED | EF_BLUE;
 		else
-			effects |= EF_RED;
+			s->effects |= EF_RED;
 	}
 	else if (qweffects & QW_EF_BLUE)
-		effects |= EF_BLUE;
+		s->effects |= EF_BLUE;
 	else if (qweffects & QW_EF_BRIGHTLIGHT)
-		effects |= EF_BRIGHTLIGHT;
+		s->effects |= EF_BRIGHTLIGHT;
 	else if (qweffects & QW_EF_DIMLIGHT)
-		effects |= EF_DIMLIGHT;
-	return effects;
+		s->effects |= EF_DIMLIGHT;
 }
 
 void EntityStateQW_ReadPlayerUpdate(void)
@@ -2397,6 +2461,7 @@ void EntityStateQW_ReadPlayerUpdate(void)
 	s = &ent->state_current;
 	*s = defaultstate;
 	s->active = true;
+	s->number = enumber;
 	s->colormap = enumber;
 	playerflags = MSG_ReadShort();
 	MSG_ReadVector(s->origin, cls.protocol);
@@ -2449,7 +2514,7 @@ void EntityStateQW_ReadPlayerUpdate(void)
 	if (playerflags & QW_PF_SKINNUM)
 		s->skin = MSG_ReadByte();
 	if (playerflags & QW_PF_EFFECTS)
-		s->effects = QW_TranslateEffects(MSG_ReadByte(), enumber);
+		QW_TranslateEffects(s, MSG_ReadByte());
 	if (playerflags & QW_PF_WEAPONFRAME)
 		weaponframe = MSG_ReadByte();
 	else
@@ -2470,9 +2535,6 @@ void EntityStateQW_ReadPlayerUpdate(void)
 	// if this is an update on our player, update interpolation state
 	if (enumber == cl.playerentity)
 	{
-		// force a recalculation of the player prediction
-		cl.movement_replay = true;
-
 		VectorCopy (cl.mpunchangle[0], cl.mpunchangle[1]);
 		VectorCopy (cl.mpunchvector[0], cl.mpunchvector[1]);
 		VectorCopy (cl.mvelocity[0], cl.mvelocity[1]);
@@ -2504,8 +2566,6 @@ void EntityStateQW_ReadPlayerUpdate(void)
 	cl.entities_active[enumber] = s->active;
 	// set the update time
 	s->time = cl.mtime[0] - msec * 0.001; // qw has no clock
-	// fix the number (it gets wiped occasionally by copying from defaultstate)
-	s->number = enumber;
 	// check if we need to update the lerp stuff
 	if (s->active)
 		CL_MoveLerpEntityStates(&cl.entities[enumber]);
@@ -2531,7 +2591,7 @@ static void EntityStateQW_ReadEntityUpdate(entity_state_t *s, int bits)
 	if (bits & QW_U_SKIN)
 		s->skin = MSG_ReadByte();
 	if (bits & QW_U_EFFECTS)
-		s->effects = QW_TranslateEffects(qweffects = MSG_ReadByte(), s->number);
+		QW_TranslateEffects(s, qweffects = MSG_ReadByte());
 	if (bits & QW_U_ORIGIN1)
 		s->origin[0] = MSG_ReadCoord13i();
 	if (bits & QW_U_ANGLE1)
