@@ -24,9 +24,17 @@
 #include <dlfcn.h>
 #include <signal.h>
 #include <AGL/agl.h>
+#include <OpenGL/OpenGL.h>
 #include <Carbon/Carbon.h>
+#include <IOKit/hidsystem/IOHIDLib.h>
+#include <IOKit/hidsystem/IOHIDParameter.h>
+#include <IOKit/hidsystem/event_status_driver.h>
+#include "vid_agl_mackeys.h" // this is SDL/src/video/maccommon/SDL_mackeys.h
 #include "quakedef.h"
 
+#ifndef kCGLCEMPEngine
+#define kCGLCEMPEngine 313
+#endif
 
 // Tell startup code that we have a client
 int cl_available = true;
@@ -45,7 +53,11 @@ GLboolean (*qaglSetDrawable) (AGLContext ctx, AGLDrawable draw);
 GLboolean (*qaglSetFullScreen) (AGLContext ctx, GLsizei width, GLsizei height, GLsizei freq, GLint device);
 GLboolean (*qaglSetInteger) (AGLContext ctx, GLenum pname, const GLint *params);
 void (*qaglSwapBuffers) (AGLContext ctx);
+CGLError (*qCGLEnable) (CGLContextObj ctx, CGLContextEnable pname);
+CGLError (*qCGLDisable) (CGLContextObj ctx, CGLContextEnable pname);
+CGLContextObj (*qCGLGetCurrentContext) (void);
 
+static qboolean multithreadedgl;
 static qboolean mouse_avail = true;
 static qboolean vid_usingmouse = false;
 static float mouse_x, mouse_y;
@@ -57,9 +69,34 @@ static qboolean sound_active = true;
 
 static int scr_width, scr_height;
 
+static cvar_t apple_multithreadedgl = {CVAR_SAVE, "apple_multithreadedgl", "1", "makes use of a second thread for the OpenGL driver (if possible) rather than using the engine thread (note: this is done automatically on most other operating systems)"};
+static cvar_t apple_mouse_noaccel = {CVAR_SAVE, "apple_mouse_noaccel", "1", "disables mouse acceleration while DarkPlaces is active"};
+
 static AGLContext context;
 static WindowRef window;
 
+static double originalMouseSpeed = -1.0;
+
+io_connect_t IN_GetIOHandle()
+{
+	io_connect_t iohandle = MACH_PORT_NULL;
+	kern_return_t status;
+	io_service_t iohidsystem = MACH_PORT_NULL;
+	mach_port_t masterport;
+
+	status = IOMasterPort(MACH_PORT_NULL, &masterport);
+	if(status != KERN_SUCCESS)
+		return 0;
+
+	iohidsystem = IORegistryEntryFromPath(masterport, kIOServicePlane ":/IOResources/IOHIDSystem");
+	if(!iohidsystem)
+		return 0;
+
+	status = IOServiceOpen(iohidsystem, mach_task_self(), kIOHIDParamConnectType, &iohandle);
+	IOObjectRelease(iohidsystem);
+
+	return iohandle;
+}
 
 void VID_GetWindowSize (int *x, int *y, int *width, int *height)
 {
@@ -89,6 +126,36 @@ static void IN_Activate( qboolean grab )
 			// Lock the mouse pointer at its current position
 			CGAssociateMouseAndMouseCursorPosition(false);
 
+			// Save the status of mouse acceleration
+			originalMouseSpeed = -1.0; // in case of error
+			if(apple_mouse_noaccel.integer)
+			{
+				io_connect_t mouseDev = IN_GetIOHandle();
+				if(mouseDev != 0)
+				{
+					if(IOHIDGetAccelerationWithKey(mouseDev, CFSTR(kIOHIDMouseAccelerationType), &originalMouseSpeed) == kIOReturnSuccess)
+					{
+						Con_DPrintf("previous mouse acceleration: %f\n", originalMouseSpeed);
+						if(IOHIDSetAccelerationWithKey(mouseDev, CFSTR(kIOHIDMouseAccelerationType), -1.0) != kIOReturnSuccess)
+						{
+							Con_Print("Could not disable mouse acceleration (failed at IOHIDSetAccelerationWithKey).\n");
+							Cvar_SetValueQuick(&apple_mouse_noaccel, 0);
+						}
+					}
+					else
+					{
+						Con_Print("Could not disable mouse acceleration (failed at IOHIDGetAccelerationWithKey).\n");
+						Cvar_SetValueQuick(&apple_mouse_noaccel, 0);
+					}
+					IOServiceClose(mouseDev);
+				}
+				else
+				{
+					Con_Print("Could not disable mouse acceleration (failed at IO_GetIOHandle).\n");
+					Cvar_SetValueQuick(&apple_mouse_noaccel, 0);
+				}
+			}
+
 			mouse_x = mouse_y = 0;
 			vid_usingmouse = true;
 		}
@@ -97,6 +164,20 @@ static void IN_Activate( qboolean grab )
 	{
 		if (vid_usingmouse)
 		{
+			if(originalMouseSpeed != -1.0)
+			{
+				io_connect_t mouseDev = IN_GetIOHandle();
+				if(mouseDev != 0)
+				{
+					Con_DPrintf("restoring mouse acceleration to: %f\n", originalMouseSpeed);
+					if(IOHIDSetAccelerationWithKey(mouseDev, CFSTR(kIOHIDMouseAccelerationType), originalMouseSpeed) != kIOReturnSuccess)
+						Con_Print("Could not re-enable mouse acceleration (failed at IOHIDSetAccelerationWithKey).\n");
+					IOServiceClose(mouseDev);
+				}
+				else
+					Con_Print("Could not re-enable mouse acceleration (failed at IO_GetIOHandle).\n");
+			}
+
 			CGAssociateMouseAndMouseCursorPosition(true);
 			CGDisplayShowCursor(CGMainDisplayID());
 
@@ -146,6 +227,42 @@ void VID_Finish (qboolean allowmousegrab)
 		qaglSwapBuffers(context);
 	}
 	VID_UpdateGamma(false, GAMMA_TABLE_SIZE);
+
+	if (apple_multithreadedgl.integer)
+	{
+		if (!multithreadedgl)
+		{
+			if(qCGLGetCurrentContext && qCGLEnable && qCGLDisable)
+			{
+				CGLContextObj ctx = qCGLGetCurrentContext();
+				CGLError e = qCGLEnable(ctx, kCGLCEMPEngine);
+				if(e == kCGLNoError)
+					multithreadedgl = true;
+				else
+				{
+					Con_Printf("WARNING: can't enable multithreaded GL, error %d\n", (int) e);
+					Cvar_SetValueQuick(&apple_multithreadedgl, 0);
+				}
+			}
+			else
+			{
+				Con_Printf("WARNING: can't enable multithreaded GL, CGL functions not present\n");
+				Cvar_SetValueQuick(&apple_multithreadedgl, 0);
+			}
+		}
+	}
+	else
+	{
+		if (multithreadedgl)
+		{
+			if(qCGLGetCurrentContext && qCGLEnable && qCGLDisable)
+			{
+				CGLContextObj ctx = qCGLGetCurrentContext();
+				qCGLDisable(ctx, kCGLCEMPEngine);
+				multithreadedgl = false;
+			}
+		}
+	}
 }
 
 int VID_SetGamma(unsigned short *ramps, int rampsize)
@@ -207,8 +324,7 @@ void signal_handler(int sig)
 {
 	printf("Received signal %d, exiting...\n", sig);
 	VID_RestoreSystemGamma();
-	Sys_Quit();
-	exit(0);
+	Sys_Quit(1);
 }
 
 void InitSig(void)
@@ -228,15 +344,21 @@ void InitSig(void)
 void VID_Init(void)
 {
 	InitSig(); // trap evil signals
+	Cvar_RegisterVariable(&apple_multithreadedgl);
+	Cvar_RegisterVariable(&apple_mouse_noaccel);
 // COMMANDLINEOPTION: Input: -nomouse disables mouse support (see also vid_mouse cvar)
-	if (COM_CheckParm ("-nomouse") || COM_CheckParm("-safe"))
+	if (COM_CheckParm ("-nomouse"))
 		mouse_avail = false;
 }
 
 static void *prjobj = NULL;
+static void *cglobj = NULL;
 
 static void GL_CloseLibrary(void)
 {
+	if (cglobj)
+		dlclose(cglobj);
+	cglobj = NULL;
 	if (prjobj)
 		dlclose(prjobj);
 	prjobj = NULL;
@@ -249,6 +371,7 @@ static void GL_CloseLibrary(void)
 static int GL_OpenLibrary(void)
 {
 	const char *name = "/System/Library/Frameworks/AGL.framework/AGL";
+	const char *name2 = "/System/Library/Frameworks/OpenGL.framework/OpenGL";
 
 	Con_Printf("Loading OpenGL driver %s\n", name);
 	GL_CloseLibrary();
@@ -258,6 +381,11 @@ static int GL_OpenLibrary(void)
 		return false;
 	}
 	strlcpy(gl_driver, name, sizeof(gl_driver));
+
+	Con_Printf("Loading OpenGL driver %s\n", name2);
+	if (!(cglobj = dlopen(name2, RTLD_LAZY)))
+		Con_Printf("Unable to open symbol list for %s; multithreaded GL disabled\n", name);
+
 	return true;
 }
 
@@ -266,6 +394,13 @@ void *GL_GetProcAddress(const char *name)
 	return dlsym(prjobj, name);
 }
 
+static void *CGL_GetProcAddress(const char *name)
+{
+	if(!cglobj)
+		return NULL;
+	return dlsym(cglobj, name);
+}
+
 void VID_Shutdown(void)
 {
 	if (context == NULL && window == NULL)
@@ -357,7 +492,7 @@ static void VID_ProcessPendingAsyncEvents (void)
 
 	// Closed
 	if (AsyncEvent_Quitting)
-		Sys_Quit();
+		Sys_Quit(0);
 }
 
 static void VID_BuildAGLAttrib(GLint *attrib, qboolean stencil, qboolean fullscreen, qboolean stereobuffer)
@@ -421,6 +556,12 @@ int VID_InitMode(int fullscreen, int width, int height, int bpp, int refreshrate
 		return false;
 	}
 
+	qCGLEnable = (CGLError (*) (CGLContextObj ctx, CGLContextEnable pname)) CGL_GetProcAddress("CGLEnable");
+	qCGLDisable = (CGLError (*) (CGLContextObj ctx, CGLContextEnable pname)) CGL_GetProcAddress("CGLDisable");
+	qCGLGetCurrentContext = (CGLContextObj (*) (void)) CGL_GetProcAddress("CGLGetCurrentContext");
+	if(!qCGLEnable || !qCGLDisable || !qCGLGetCurrentContext)
+		Con_Printf("CGL functions not found; disabling multithreaded OpenGL\n");
+
 	// Ignore the events from the previous window
 	AsyncEvent_Quitting = false;
 	AsyncEvent_Collapsed = false;
@@ -553,6 +694,7 @@ int VID_InitMode(int fullscreen, int width, int height, int bpp, int refreshrate
 	gl_platform = "AGL";
 	gl_videosyncavailable = true;
 
+	multithreadedgl = false;
 	vid_isfullscreen = fullscreen;
 	vid_usingmouse = false;
 	vid_hidden = false;
@@ -597,72 +739,176 @@ static void Handle_KeyMod(UInt32 keymod)
 	prev_keymod = keymod;
 }
 
-static void Handle_Key(unsigned char charcode, qboolean keypressed)
+static void Handle_Key(unsigned char charcode, UInt32 mackeycode, qboolean keypressed)
 {
 	unsigned int keycode = 0;
 	char ascii = '\0';
 
-	switch (charcode)
+	switch (mackeycode)
 	{
-		case kHomeCharCode:
-			keycode = K_HOME;
-			break;
-		case kEnterCharCode:
-			keycode = K_KP_ENTER;
-			break;
-		case kEndCharCode:
-			keycode = K_END;
-			break;
-		case kBackspaceCharCode:
-			keycode = K_BACKSPACE;
-			break;
-		case kTabCharCode:
-			keycode = K_TAB;
-			break;
-		case kPageUpCharCode:
-			keycode = K_PGUP;
-			break;
-		case kPageDownCharCode:
-			keycode = K_PGDN;
-			break;
-		case kReturnCharCode:
-			keycode = K_ENTER;
-			break;
-		case kEscapeCharCode:
+		case MK_ESCAPE:
 			keycode = K_ESCAPE;
 			break;
-		case kLeftArrowCharCode:
-			keycode = K_LEFTARROW;
+		case MK_F1:
+			keycode = K_F1;
 			break;
-		case kRightArrowCharCode:
-			keycode = K_RIGHTARROW;
+		case MK_F2:
+			keycode = K_F2;
 			break;
-		case kUpArrowCharCode:
-			keycode = K_UPARROW;
+		case MK_F3:
+			keycode = K_F3;
 			break;
-		case kDownArrowCharCode :
-			keycode = K_DOWNARROW;
+		case MK_F4:
+			keycode = K_F4;
 			break;
-		case kDeleteCharCode:
+		case MK_F5:
+			keycode = K_F5;
+			break;
+		case MK_F6:
+			keycode = K_F6;
+			break;
+		case MK_F7:
+			keycode = K_F7;
+			break;
+		case MK_F8:
+			keycode = K_F8;
+			break;
+		case MK_F9:
+			keycode = K_F9;
+			break;
+		case MK_F10:
+			keycode = K_F10;
+			break;
+		case MK_F11:
+			keycode = K_F11;
+			break;
+		case MK_F12:
+			keycode = K_F12;
+			break;
+		case MK_SCROLLOCK:
+			keycode = K_SCROLLOCK;
+			break;
+		case MK_PAUSE:
+			keycode = K_PAUSE;
+			break;
+		case MK_BACKSPACE:
+			keycode = K_BACKSPACE;
+			break;
+		case MK_INSERT:
+			keycode = K_INS;
+			break;
+		case MK_HOME:
+			keycode = K_HOME;
+			break;
+		case MK_PAGEUP:
+			keycode = K_PGUP;
+			break;
+		case MK_NUMLOCK:
+			keycode = K_NUMLOCK;
+			break;
+		case MK_KP_EQUALS:
+			keycode = K_KP_EQUALS;
+			break;
+		case MK_KP_DIVIDE:
+			keycode = K_KP_DIVIDE;
+			break;
+		case MK_KP_MULTIPLY:
+			keycode = K_KP_MULTIPLY;
+			break;
+		case MK_TAB:
+			keycode = K_TAB;
+			break;
+		case MK_DELETE:
 			keycode = K_DEL;
 			break;
-		case 0:
-		case 191:
-			// characters 0 and 191 are sent by the mouse buttons (?!)
+		case MK_END:
+			keycode = K_END;
+			break;
+		case MK_PAGEDOWN:
+			keycode = K_PGDN;
+			break;
+		case MK_KP7:
+			keycode = K_KP_7;
+			break;
+		case MK_KP8:
+			keycode = K_KP_8;
+			break;
+		case MK_KP9:
+			keycode = K_KP_9;
+			break;
+		case MK_KP_MINUS:
+			keycode = K_KP_MINUS;
+			break;
+		case MK_CAPSLOCK:
+			keycode = K_CAPSLOCK;
+			break;
+		case MK_RETURN:
+			keycode = K_ENTER;
+			break;
+		case MK_KP4:
+			keycode = K_KP_4;
+			break;
+		case MK_KP5:
+			keycode = K_KP_5;
+			break;
+		case MK_KP6:
+			keycode = K_KP_6;
+			break;
+		case MK_KP_PLUS:
+			keycode = K_KP_PLUS;
+			break;
+		case MK_KP1:
+			keycode = K_KP_1;
+			break;
+		case MK_KP2:
+			keycode = K_KP_2;
+			break;
+		case MK_KP3:
+			keycode = K_KP_3;
+			break;
+		case MK_KP_ENTER:
+		case MK_IBOOK_ENTER:
+			keycode = K_KP_ENTER;
+			break;
+		case MK_KP0:
+			keycode = K_KP_0;
+			break;
+		case MK_KP_PERIOD:
+			keycode = K_KP_PERIOD;
 			break;
 		default:
-			if ('A' <= charcode && charcode <= 'Z')
+			switch(charcode)
 			{
-				keycode = charcode + ('a' - 'A');  // lowercase it
-				ascii = charcode;
+				case kUpArrowCharCode:
+					keycode = K_UPARROW;
+					break;
+				case kLeftArrowCharCode:
+					keycode = K_LEFTARROW;
+					break;
+				case kDownArrowCharCode:
+					keycode = K_DOWNARROW;
+					break;
+				case kRightArrowCharCode:
+					keycode = K_RIGHTARROW;
+					break;
+				case 0:
+				case 191:
+					// characters 0 and 191 are sent by the mouse buttons (?!)
+					break;
+				default:
+					if ('A' <= charcode && charcode <= 'Z')
+					{
+						keycode = charcode + ('a' - 'A');  // lowercase it
+						ascii = charcode;
+					}
+					else if (charcode >= 32)
+					{
+						keycode = charcode;
+						ascii = charcode;
+					}
+					else
+						Con_DPrintf(">> UNKNOWN char/keycode: %d/%u <<\n", charcode, (unsigned) mackeycode);
 			}
-			else if (charcode >= 32)
-			{
-				keycode = charcode;
-				ascii = charcode;
-			}
-			else
-				Con_Printf(">> UNKNOWN charcode: %d <<\n", charcode);
 	}
 
 	if (keycode != 0)
@@ -746,21 +992,24 @@ void Sys_SendKeyEvents(void)
 
 			case kEventClassKeyboard:
 			{
-				char keycode;
+				char charcode;
+				UInt32 keycode;
 
 				switch (eventKind)
 				{
 					case kEventRawKeyDown:
-						GetEventParameter(theEvent, kEventParamKeyMacCharCodes, typeChar, NULL, sizeof(keycode), NULL, &keycode);
-						Handle_Key(keycode, true);
+						GetEventParameter(theEvent, kEventParamKeyMacCharCodes, typeChar, NULL, sizeof(charcode), NULL, &charcode);
+						GetEventParameter(theEvent, kEventParamKeyCode, typeUInt32, NULL, sizeof(keycode), NULL, &keycode);
+						Handle_Key(charcode, keycode, true);
 						break;
 
 					case kEventRawKeyRepeat:
 						break;
 
 					case kEventRawKeyUp:
-						GetEventParameter(theEvent, kEventParamKeyMacCharCodes, typeChar, NULL, sizeof(keycode), NULL, &keycode);
-						Handle_Key(keycode, false);
+						GetEventParameter(theEvent, kEventParamKeyMacCharCodes, typeChar, NULL, sizeof(charcode), NULL, &charcode);
+						GetEventParameter(theEvent, kEventParamKeyCode, typeUInt32, NULL, sizeof(keycode), NULL, &keycode);
+						Handle_Key(charcode, keycode, false);
 						break;
 
 					case kEventRawKeyModifiersChanged:
@@ -801,7 +1050,7 @@ void Sys_SendKeyEvents(void)
 						VID_AppFocusChanged(false);
 						break;
 					case kEventAppQuit:
-						Sys_Quit();
+						Sys_Quit(0);
 						break;
 					case kEventAppActiveWindowChanged:
 						break;
