@@ -34,16 +34,16 @@ when crossing a water boudnary.
 cvar_t cl_rollspeed = {0, "cl_rollspeed", "200", "how much strafing is necessary to tilt the view"};
 cvar_t cl_rollangle = {0, "cl_rollangle", "2.0", "how much to tilt the view when strafing"};
 
-cvar_t cl_bob = {0, "cl_bob","0.02", "view bobbing amount"};
-cvar_t cl_bobcycle = {0, "cl_bobcycle","0.6", "view bobbing speed"};
-cvar_t cl_bobup = {0, "cl_bobup","0.5", "view bobbing adjustment that makes the up or down swing of the bob last longer"};
+cvar_t cl_bob = {CVAR_SAVE, "cl_bob","0.02", "view bobbing amount"};
+cvar_t cl_bobcycle = {CVAR_SAVE, "cl_bobcycle","0.6", "view bobbing speed"};
+cvar_t cl_bobup = {CVAR_SAVE, "cl_bobup","0.5", "view bobbing adjustment that makes the up or down swing of the bob last longer"};
 
 cvar_t cl_bobmodel = {CVAR_SAVE, "cl_bobmodel", "1", "enables gun bobbing"};
-cvar_t cl_bobmodel_side = {0, "cl_bobmodel_side", "0.05", "gun bobbing sideways sway amount"};
-cvar_t cl_bobmodel_up = {0, "cl_bobmodel_up", "0.02", "gun bobbing upward movement amount"};
-cvar_t cl_bobmodel_speed = {0, "cl_bobmodel_speed", "7", "gun bobbing speed"};
+cvar_t cl_bobmodel_side = {CVAR_SAVE, "cl_bobmodel_side", "0.15", "gun bobbing sideways sway amount"};
+cvar_t cl_bobmodel_up = {CVAR_SAVE, "cl_bobmodel_up", "0.06", "gun bobbing upward movement amount"};
+cvar_t cl_bobmodel_speed = {CVAR_SAVE, "cl_bobmodel_speed", "7", "gun bobbing speed"};
 
-cvar_t cl_viewmodel_scale = {0, "cl_viewmodel_scale", "0.3", "changes size of gun model, lower values prevent poking into walls but cause strange artifacts on lighting and especially r_stereo/vid_stereobuffer options where the size of the gun becomes visible"};
+cvar_t cl_viewmodel_scale = {0, "cl_viewmodel_scale", "1", "changes size of gun model, lower values prevent poking into walls but cause strange artifacts on lighting and especially r_stereo/vid_stereobuffer options where the size of the gun becomes visible"};
 
 cvar_t v_kicktime = {0, "v_kicktime", "0.5", "how long a view kick from damage lasts"};
 cvar_t v_kickroll = {0, "v_kickroll", "0.6", "how much a view kick from damage rolls your view"};
@@ -313,32 +313,6 @@ extern matrix4x4_t viewmodelmatrix;
 #include "cl_collision.h"
 #include "csprogs.h"
 
-/*
-==================
-CL_StairSmoothing
-
-==================
-*/
-void CL_StairSmoothing (void)
-{
-	if (v_dmg_time > 0)
-		v_dmg_time -= bound(0, cl.time - cl.oldtime, 0.1);
-
-	// stair smoothing
-	if (cl.onground && cl.stairoffset < 0)
-	{
-		cl.stairoffset += bound(0, cl.time - cl.oldtime, 0.1) * cl_stairsmoothspeed.value;
-		cl.stairoffset = bound(-16, cl.stairoffset, 0);
-	}
-	else if (cl.onground && cl.stairoffset > 0)
-	{
-		cl.stairoffset -= bound(0, cl.time - cl.oldtime, 0.1) * cl_stairsmoothspeed.value;
-		cl.stairoffset = bound(0, cl.stairoffset, 16);
-	}
-	else
-		cl.stairoffset = 0;
-}
-
 /*
 ==================
 V_CalcRefdef
@@ -347,12 +321,9 @@ V_CalcRefdef
 */
 void V_CalcRefdef (void)
 {
-	static float oldz;
 	entity_t *ent;
-	float vieworg[3], gunorg[3], viewangles[3];
+	float vieworg[3], gunorg[3], viewangles[3], smoothtime;
 	trace_t trace;
-	if(csqc_loaded)
-		return;
 	VectorClear(gunorg);
 	viewmodelmatrix = identitymatrix;
 	r_view.matrix = identitymatrix;
@@ -365,27 +336,14 @@ void V_CalcRefdef (void)
 		Matrix4x4_OriginFromMatrix(&ent->render.matrix, vieworg);
 		VectorCopy(cl.viewangles, viewangles);
 
-		// update the stairoffset if the player entity has gone up or down without leaving the ground
-		//Con_Printf("cl.onground %i oldz %f newz %f vel %f %f %f\n", cl.onground, oldz, vieworg[2], cl.movement_velocity[0], cl.movement_velocity[1], cl.movement_velocity[2]);
-		cl.stairoffset -= vieworg[2] - oldz;
-		oldz = vieworg[2];
-		cl.stairoffset = bound(-16, cl.stairoffset, 16);
+		// calculate how much time has passed since the last V_CalcRefdef
+		smoothtime = bound(0, cl.time - cl.stairsmoothtime, 0.1);
+		cl.stairsmoothtime = cl.time;
+
+		// fade damage flash
+		if (v_dmg_time > 0)
+			v_dmg_time -= bound(0, smoothtime, 0.1);
 
-		// interpolate the angles if playing a demo or spectating someone
-		if (cls.demoplayback || cl.fixangle[0])
-		{
-			int i;
-			float frac = bound(0, (cl.time - cl.mtime[1]) / (cl.mtime[0] - cl.mtime[1]), 1);
-			for (i = 0;i < 3;i++)
-			{
-				float d = cl.mviewangles[0][i] - cl.mviewangles[1][i];
-				if (d > 180)
-					d -= 360;
-				else if (d < -180)
-					d += 360;
-				viewangles[i] = cl.mviewangles[1][i] + frac * d;
-			}
-		}
 		if (cl.intermission)
 		{
 			// entity is a fixed camera, just copy the matrix
@@ -400,13 +358,21 @@ void V_CalcRefdef (void)
 		}
 		else
 		{
+			// smooth stair stepping, but only if onground and enabled
+			if (!cl.onground || cl_stairsmoothspeed.value <= 0)
+				cl.stairsmoothz = vieworg[2];
+			else
+			{
+				if (cl.stairsmoothz < vieworg[2])
+					vieworg[2] = cl.stairsmoothz = bound(vieworg[2] - 16, cl.stairsmoothz + smoothtime * cl_stairsmoothspeed.value, vieworg[2]);
+				else if (cl.stairsmoothz > vieworg[2])
+					vieworg[2] = cl.stairsmoothz = bound(vieworg[2], cl.stairsmoothz - smoothtime * cl_stairsmoothspeed.value, vieworg[2] + 16);
+			}
+
 			// apply qw weapon recoil effect (this did not work in QW)
 			// TODO: add a cvar to disable this
 			viewangles[PITCH] += cl.qw_weaponkick;
 
-			// bias by stair smoothing offset
-			vieworg[2] += cl.stairoffset;
-
 			if (chase_active.value)
 			{
 				// observing entity from third person
@@ -435,7 +401,7 @@ void V_CalcRefdef (void)
 				chase_dest[0] = vieworg[0] + forward[0] * dist;
 				chase_dest[1] = vieworg[1] + forward[1] * dist;
 				chase_dest[2] = vieworg[2] + forward[2] * dist + camup;
-				trace = CL_TraceBox(vieworg, vec3_origin, vec3_origin, chase_dest, true, NULL, SUPERCONTENTS_SOLID | SUPERCONTENTS_BODY | SUPERCONTENTS_SKY, false);
+				trace = CL_Move(vieworg, vec3_origin, vec3_origin, chase_dest, MOVE_NOMONSTERS, NULL, SUPERCONTENTS_SOLID | SUPERCONTENTS_BODY | SUPERCONTENTS_SKY, true, false, NULL, false);
 				VectorMAMAM(1, trace.endpos, 8, forward, 4, trace.plane.normal, vieworg);
 			}
 			else
@@ -481,7 +447,7 @@ void V_CalcRefdef (void)
 
 					VectorCopy(vieworg, gunorg);
 
-					if (cl_bobmodel.value)
+					if (cl_bob.value && cl_bobmodel.value)
 					{
 						// calculate for swinging gun model
 						// the gun bobs when running on the ground, but doesn't bob when you're in the air.
@@ -516,9 +482,9 @@ void V_CalcRefdef (void)
 
 						bspeed = bound (0, xyspeed, 400) * 0.01f;
 						AngleVectors (viewangles, forward, right, up);
-						bob = bspeed * cl_bobmodel_side.value * sin (s) * t;
+						bob = bspeed * cl_bobmodel_side.value * cl_viewmodel_scale.value * sin (s) * t;
 						VectorMA (gunorg, bob, right, gunorg);
-						bob = bspeed * cl_bobmodel_up.value * cos (s * 2) * t;
+						bob = bspeed * cl_bobmodel_up.value * cl_viewmodel_scale.value * cos (s * 2) * t;
 						VectorMA (gunorg, bob, up, gunorg);
 					}
 				}
@@ -530,8 +496,8 @@ void V_CalcRefdef (void)
 				Matrix4x4_CreateFromQuakeEntity(&r_view.matrix, vieworg[0], vieworg[1], vieworg[2], viewangles[0], viewangles[1], viewangles[2] + v_idlescale.value * sin(cl.time*v_iroll_cycle.value) * v_iroll_level.value, 1);
 			// calculate a viewmodel matrix for use in view-attached entities
 			Matrix4x4_CreateFromQuakeEntity(&viewmodelmatrix, gunorg[0], gunorg[1], gunorg[2], viewangles[0], viewangles[1], viewangles[2], cl_viewmodel_scale.value);
-			VectorCopy(vieworg, csqc_origin);
-			VectorCopy(viewangles, csqc_angles);
+			VectorCopy(vieworg, cl.csqc_origin);
+			VectorCopy(viewangles, cl.csqc_angles);
 		}
 	}
 }
